﻿##########################################################################
#比率の推測（1つの2項分布）
#■入力
#x: 正反応数
#n: ベルヌイ試行数
#prob=c(0.025, 0.05, 0.5, 0.95, 0.975): 事後分布で報告する確率点
#see=1234,cha=5,war=1000,ite=21000 : MCMC関連のパラメタ
#■出力
#fit:stanの出力, par:母数リスト, prob:確率ベクトル, 
#theta: 母比率, xaste: 予測分布, Odds:オッズ
#log_lik;対数尤度
##########################################################################
Bi01 <- function(x,n,prob=c(0.025, 0.05, 0.5, 0.95, 0.975),
                 see=1234, cha=5, war=1000, ite=21000)
{
   library(cmdstanr)              #パッケージcmdstanrの呼び出し
   library(posterior)             #パッケージposteriorの呼び出し
   dat <- list(x=x,n=n)
   scr <- "stan/Bi01.stan"                                # Stan ファイル名
   par<-c("theta","xaste","Odds","log_lik")               # sampling変数
      mod <- cmdstan_model(scr)                   #コンパイル
      csrfit <- mod$sample(data=dat,chains=cha,iter_sampling=ite,
         iter_warmup=war,parallel_chains=cha,seed=see)    #MCMC
      draw<-csrfit$draws(par)
      ext<-as_draws_df(draw)
      (colnames(ext) <- gsub("\\[|\\]|,", "", colnames(ext)))
   out<-list(ext=ext,draw=draw,par=par,prob=prob,
        theta=ext$theta,xaste=ext$xaste, Odds=ext$Odds,log_lik=ext$log_lik);
   class(out)<-'Bi01'
   return(invisible(out))
}
