﻿##########################################################################
#対応のない2群の比率に関する統計的推測（2つの2項分布）
#■入力
#x: 正反応数(長さ2のベクトル、標本比率の大きいものを先に指定する)
#n: ベルヌイ試行数(長さ2のベクトル)
#prob=c(0.025, 0.05, 0.5, 0.95, 0.975): 事後分布で報告する確率点
#see=1234,cha=5,war=1000,ite=21000 : MCMC関連のパラメタ
#■出力
#fit:stanの出力, par:母数リスト, prob:確率ベクトル, 
#p: 母比率(2)
#xaste: 予測分布のx*(2)
#p_sa: 比率の差
#p_hi: 比率の比
#Odds: オッズ(2)
#Odds_hi: オッズ比
#log_lik;対数尤度
##########################################################################
Bi02 <- function(x,n,prob=c(0.025, 0.05, 0.5, 0.95, 0.975),
                 see=1234, cha=5, war=1000, ite=21000)
{
   library(cmdstanr)              #パッケージcmdstanrの呼び出し
   library(posterior)             #パッケージposteriorの呼び出し
   library(rstan)
   dat <- list(x=x,n=n)
   scr <- "stan/Bi02.stan"                                     # Stanファイル名
   par<-c("p","xaste","p_sa","p_hi","Odds","Odds_hi","log_lik")# sampling変数
      mod <- cmdstan_model(scr)                   #コンパイル
      csrfit <- mod$sample(data=dat,chains=cha,iter_sampling=ite,
         iter_warmup=war,parallel_chains=cha,seed=see)    #MCMC
      draw<-csrfit$draws(par)
      ext<-as_draws_df(draw)
      colnames(ext) <- gsub("\\[|\\]|,", "", colnames(ext))
   out<-list(ext=ext,draw=draw,par=par,prob=prob,
         p=as.matrix(ext[,1:2]), xaste=as.matrix(ext[,3:4]), 
         p_sa=as.matrix(ext$p_sa), p_hi=as.matrix(ext$p_hi),
         Odds=as.matrix(ext[,7:8]), Odds_hi=as.matrix(ext$Odds_hi),
         log_lik=ext$log_lik);  
   class(out)<-'Bi02'
   return(invisible(out))
}
