﻿##########################################################################
#対応のないg群の比率に関する統計的推測（g個の2項分布）
#■入力
#x: 正反応数(長さgのベクトル)
#n: ベルヌイ試行数(長さgのベクトル)
#prob=c(0.025, 0.05, 0.5, 0.95, 0.975): 事後分布で報告する確率点
#see=1234,cha=5,war=1000,ite=21000 : MCMC関連のパラメタ
#■出力
#fit:stanの出力, par:母数リスト, prob:確率ベクトル, 
#p: 母比率(2)
#xaste: 予測分布のx*(2)
#U2: 行の母比率が列の母比率より大きいとき1を、それ以外は0 (,g*g)
#log_lik;対数尤度
##########################################################################
Bi03 <- function(x,n,prob=c(0.025, 0.05, 0.5, 0.95, 0.975),
                 see=1234, cha=5, war=1000, ite=21000)
{
   library(cmdstanr)              #パッケージcmdstanrの呼び出し
   library(posterior)             #パッケージposteriorの呼び出し
   library(rstan)
   g <- length(x)
   dat <- list(g=g,x=x,n=n)
   scr <- "stan/Bi03.stan"                                # Stanファイル名
   par<-c("p","xaste","U2","log_lik")                     # sampling変数
      mod <- cmdstan_model(scr)                   #コンパイル
      csrfit <- mod$sample(data=dat,chains=cha,iter_sampling=ite,
         iter_warmup=war,parallel_chains=cha,seed=see)    #MCMC
      draw<-csrfit$draws(par)
      ext<-as_draws_df(draw)
      colnames(ext) <- gsub("\\[|\\]|,", "", colnames(ext))
   out<-list(ext=ext,draw=draw,par=par,prob=prob,
             p=ext[,1:g], xaste=ext[,c((g+1):(2*g))], 
             log_lik=ext$log_lik);
   class(out)<-'Bi03'
   return(invisible(out))
}
