﻿##########################################################################
#対応のある２群の差を推測する
#■入力
#x:n行2列の行列形式のデータ (平均値の大きな群は1列目に指定)
#EQU=1 : 論理値。1なら等分散。0なら異なる分散。
#prior=F : 論理値。Tなら事前分布の範囲を指定。Fなら指定せず。
#mL=-1000, mH=1000, sL=0, sH=100 : 事前分布のパラメタ
#prob=c(0.025, 0.05, 0.5, 0.95, 0.975): 事後分布で報告する確率点
#see=1234,cha=5,war=1000,ite=21000 : MCMC関連のパラメタ
#■出力
# fit:stanの出力, par:母数リスト, prob:確率ベクトル, 
# mu1:第1群平均, mu2:第2群平均, sigma1:第1群標準偏差, sigma2:第2群標準偏差, 
# rho:相関係数, xaste1:予測分布1, xaste2:予測分布2, log_lik:対数尤度
##########################################################################
G2pair <- function(x,EQU=1,prior=F,mL=-1000, mH=1000, sL=0, sH=100,
            prob=c(0.025, 0.05, 0.5, 0.95, 0.975),
            see=1234, cha=5, war=1000, ite=21000)
{
   library(cmdstanr)              #パッケージcmdstanrの呼び出し
   library(posterior)             #パッケージposteriorの呼び出し
   if (prior) {
     dat <- list(n=nrow(x),x=x,EQU=EQU,mL=mL,mH=mH,sL=sL,sH=sH)
     scr <- "stan/G2pairPT.stan"                           # Stan ファイル名
   } else {
     dat <- list(n=nrow(x),x=x,EQU=EQU)
     scr <- "stan/G2pairPF.stan"                           # Stan ファイル名
   }
   par<-c("mu","sigma1","sigma2","rho","xaste","log_lik")# サンプリング変数
      mod <- cmdstan_model(scr)                   #コンパイル
      csrfit <- mod$sample(data=dat,chains=cha,iter_sampling=ite,
         iter_warmup=war,parallel_chains=cha,seed=see)    #MCMC
      draw<-csrfit$draws(par)
      ext<-as_draws_df(draw)
      (colnames(ext) <- gsub("\\[|\\]|,", "", colnames(ext)))
   mu1<-ext$mu1; mu2<-ext$mu2; 
   sigma1<-ext$sigma1; sigma2<-ext$sigma2; rho<-ext$rho;
   xaste1<-ext$xaste1; xaste2<-ext$xaste2; log_lik<-ext$log_lik
   out<-list(EQU=EQU,prior=prior,ext=ext,draw=draw,par=par,
       prob=prob,mu1=mu1,mu2=mu2,sigma1=sigma1,sigma2=sigma2,rho=rho,
       xaste1=xaste1,xaste2=xaste2,log_lik=log_lik)
   class(out)<-'G2pair'
   return(invisible(out))
}
