﻿##########################################################################
#カテゴリ数がkの比率の統計的推測
#■入力
#x: 正反応数(長さkのベクトル)
#prob=c(0.025, 0.05, 0.5, 0.95, 0.975): 事後分布で報告する確率点
#see=1234,cha=5,war=1000,ite=21000 : MCMC関連のパラメタ
#■出力
#fit:stanの出力, par:母数リスト, prob:確率ベクトル, 
#pi: 母比率(k)
#U2: 行の母比率が列の母比率より大きいとき1を、それ以外は0 (,k*k)
#log_lik;対数尤度
##########################################################################
Mu01 <- function(x,prob=c(0.025, 0.05, 0.5, 0.95, 0.975),
                 see=1234, cha=5, war=1000, ite=21000)
{
   library(cmdstanr)              #パッケージcmdstanrの呼び出し
   library(posterior)             #パッケージposteriorの呼び出し
   library(rstan)
   k <- length(x)
   dat <- list(k=k,x=x)
   scr <- "stan/Mu01.stan"                         # Stanファイル名
   par<-c("pi","U2","log_lik")                     # sampling変数
      mod <- cmdstan_model(scr)                   #コンパイル
      csrfit <- mod$sample(data=dat,chains=cha,iter_sampling=ite,
         iter_warmup=war,parallel_chains=cha,seed=see)    #MCMC
      draw<-csrfit$draws(par)
      ext<-as_draws_df(draw)
      (colnames(ext) <- gsub("\\[|\\]|,", "", colnames(ext)))
   out<-list(ext=ext,draw=draw,par=par,prob=prob,
        pi=as.matrix(ext[,1:k]), U2=as.matrix(ext[,c((k+1):(k+k*k))]),
       log_lik=ext$log_lik);
   class(out)<-'Mu01'
   return(invisible(out))
}
