﻿##########################################################################
#対応のあるa×bのクロス表に関する統計的推測
#■入力
#x: クロス表（a ×　b)行列形式で指定）
#prob=c(0.025, 0.05, 0.5, 0.95, 0.975): 事後分布で報告する確率点
#see=1234,cha=5,war=1000,ite=21000 : MCMC関連のパラメタ
#■出力
#fit:stanの出力, par:母数リスト, prob:確率ベクトル, 
#pim: i行j列の母比率(a × b)
#xaste: 予測分布のx* (a × b)
#res: ピアソン残差(a × b)
#pa: 第i行の周辺確率(a)
#pb: 第j列の周辺確率(b)
#Up: ピアソン残差が0より大きい確率(a × b)
#Um: ピアソン残差が0以下の確率(a × b)
#L : 同時確率と周辺確率の積との比(a × b)
#V: クラメルの連関係数
#log_lik: 対数尤度
##########################################################################
Mu02 <- function(x,prob=c(0.025, 0.05, 0.5, 0.95, 0.975),
                 see=1234, cha=5, war=1000, ite=21000)
{
   library(cmdstanr)              #パッケージcmdstanrの呼び出し
   library(posterior)             #パッケージposteriorの呼び出し
   a<-nrow(x); b<-ncol(x);  dat <- list(a=a, b=b, x=x); ab<-a*b;
   scr <- "stan/Mu02.stan"                               # Stanファイル名
   par<-c("pim","xaste","res","Up","Um","L","pa","pb","V","log_lik") 
      mod <- cmdstan_model(scr)                   #コンパイル
      csrfit <- mod$sample(data=dat,chains=cha,iter_sampling=ite,
         iter_warmup=war,parallel_chains=cha,seed=see)    #MCMC
      draw<-csrfit$draws(par)
      ext<-as_draws_df(draw)
      (colnames(ext) <- gsub("\\[|\\]|,", "", colnames(ext)))
   out<-list(ext=ext,draw=draw,par=par,prob=prob,
      pim=as.matrix(ext[,1:ab]), xaste=as.matrix(ext[,c((ab+1):(2*ab))]),
      res=as.matrix(ext[,c((2*ab+1):(3*ab))]),
      Up =as.matrix(ext[,c((3*ab+1):(4*ab))]),
      Um =as.matrix(ext[,c((4*ab+1):(5*ab))]),
      L  =as.matrix(ext[,c((5*ab+1):(6*ab))]),
      pa =as.matrix(ext[,c((6*ab+1):(6*ab+a))]),
      pb =as.matrix(ext[,c((6*ab+a+1):(6*ab+a+b))]),
      V=ext$V,log_lik=ext$log_lik);
   class(out)<-'Mu02'
   return(invisible(out))
}
