﻿###########第1章　単回帰分析
# Working directoryが 'toyoda' であることの確認
(n_wd<-getwd())                #working directory(wd)の確認
(setwd('./scrT'))              #wdの移動
source('../myfunc/myfunc.R')   #自作関数の読み込み
(setwd(n_wd))                  #wdを戻す
library(cmdstanr)              #パッケージcmdstanrの呼び出し
library(posterior)             #パッケージposteriorの呼び出し
prob<-c(0.025, 0.05, 0.5, 0.95, 0.975) #確率点の定義

########## 第1章で利用する関数  単回帰の初期値を最小2乗法で計算
initi01<-function(y,x){        # y:基準変数, x:予測変数
 lmm<-lm(y ~x)                 # 最小2乗法の解を格納する
 a<-as.numeric(coef(lmm)[1])   # 第1要素の切片の取り出し
 b<-as.numeric(coef(lmm)[2])   # 第2要素の回帰係数の取り出し
 sigma<-as.numeric(summary(lmm)$sigma)  #σe (1.5)式 の取り出し
 list(a=a,b=b, sigma=sigma)    # リストでまとめて戻り値にする
}

########## 1.1 Galton (1886) の親子の身長データ

########  表1.2 「身長データ」の一部(cm)
親子身長 <- read.csv("./scrT/dat/親子身長.csv", header=T)
head(親子身長,n=14);tail(親子身長,n=14);   #頭としっぽを見る

####### 表1.3 「身長データ」の数値要約
hsd<-function(x){sqrt(mean((x-mean(x))^2))}#標本分散による標準偏差を求める関数
round(apply(親子身長,2,mean),1)            # 平均
round(apply(親子身長,2,hsd),2)             # 標準偏差
round(apply(親子身長,2,quantile),1)        # 分位点

####### 図1.1 「身長データ」+ 乱数の散布図
set.seed(1234);                            #乱数の設定
R親子身長<-親子身長+rnorm(928*2,0,0.8)      #乱数の付加
plot(R親子身長,type="p",xlim=c(155,190),ylim=c(155,190),pch=1,
   cex.axis=1.5,cex.lab=1.3)
#dev.copy2eps(file='z0101.eps',family='Japan1')

####### 図1.3 散布図と回帰直線 (図1.1に重ね書きする)
abline(0,1,lty=2,lwd=2.0)                  #切片0,傾き1の直線
abline(60.856,0.646,lty=1,lwd=2.0)         #点推定値による回帰直線を描く
#dev.copy2eps(file='z0103.eps',family='Japan1')

###############################単回帰モデルのstanコード
reg_01<-'
//単回帰モデル
data {                    //01
  int<lower=0> n;         //02データ数定義 p.2,l.5
  vector[n] y;            //03基準変数定義
  vector[n] X;            //04予測変数定義
}
parameters {              //05
  real a;                 //06切片定義        (1.3)式
  real b;                 //07回帰係数定義    (1.3)式
  real<lower=0> sigma;    //08誤差標準偏差定義(1.5)式
}
transformed parameters {  //09
  vector[n] yhat;         //10予測値定義
  for (i in 1 : n) {      //11 1からnまで
    yhat[i] = a + X[i] * b;
  }                       //12 予測値         (1.2)式
}
model {                   //13
  for (i in 1 : n) {      //14 1からnまで
    y[i] ~ normal(yhat[i], sigma);
  }                       //15正規分布        (1.6)式
}
';

par<-c("a","b","sigma")                         #母数
dataSet_tankaiki01 <-list(n=nrow(親子身長),y=親子身長[,2], X=親子身長[,1] )

###########cmdstanrによる実行
modfile_tankaiki01 <- write_stan_file(reg_01)       #書き出す一時ファイル
mod_tankaiki01 <- cmdstan_model(modfile_tankaiki01) #コンパイル
csrfit_tankaiki01<-mod_tankaiki01$sample(data=dataSet_tankaiki01,chains=5,
   init = function(){initi01(親子身長[,2],親子身長[,1])}, 
   iter_sampling=20000,iter_warmup=1000,parallel_chains=5,seed=1234)  #MCMC
ext_tankaiki01<-as_draws_df(csrfit_tankaiki01$draws(par))
(colnames(ext_tankaiki01) <- gsub("\\[|\\]|,", "", colnames(ext_tankaiki01)))

#save(ext_tankaiki01,file="./scrT/obje/stan_obje0101")
#load(file="./scrT/obje/stan_obje0101"); #予めstan( )で作ったobjectをロード

###### 表1.4 単回帰モデルの母数と，決定係数と相関係数の事後分布の要約（前半）
gqcal(ext_tankaiki01$a)
gqcal(ext_tankaiki01$b)
gqcal(ext_tankaiki01$sigma)

hist(ext_tankaiki01$a,breaks=100)  #切片の事後分布のヒストグラム(教科書にない)
hist(ext_tankaiki01$b,breaks=100)  #回帰係数のヒストグラム(教科書にない)

##############関数を使った単回帰分析の実行
fit_tankaiki02<-Reg(親子身長[,2],親子身長[,1],cha=5, ite=20000)

#save(fit_tankaiki02,file="./scrT/obje/stan_obje0102")
#load(file="./scrT/obje/stan_obje0102"); #予めstan( )で作ったobjectをロード

###### 表1.4 単回帰モデルの母数と，決定係数と相関係数の事後分布の要約（全部）
gqcal(fit_tankaiki02$ext[,1:5])
out2<-print(fit_tankaiki02)

######## 1.4.2 「研究仮説が正しい確率」PHC
bp<-as.vector(fit_tankaiki02$b)
r2p<-as.vector(fit_tankaiki02$r2)
round(mean(bp >0.6),3); #phc(「回帰係数は0.6 より大きい」)  (1.21)式 p.9,l.10
round(mean(r2p>0.2),3); #phc(「決定係数は0.2 より大きい」)  (1.21)式 p.9,l.11

####### 表1.5 回帰係数と決定係数のphc テーブル
phc01(seq(0.54,0.60,0.01),bp,cc="gtc",byoga="no")
phc01(seq(0.14,0.20,0.01),r2p,cc="gtc",byoga="no")

####### 図1.4 回帰係数と決定係数のphc 曲線
par(mfrow=c(2,1));     #図1.4
phc01(seq(0.5,0.75,0.01),bp,cc="gtc",byoga="yes",xlab="回帰係数")
phc01(seq(0.1,0.30,0.01),r2p,cc="gtc",byoga="yes",xlab="決定係数")
par(mfrow=c(1,1))
#dev.copy2eps(file="z0104qq.eps",family="Japan1")

####### 1. 4. 4 回帰効果
####### 表1.6 予測値の事後分布の要約
out3<-print(fit_tankaiki02,degits=1,Xnew=seq(155,190,2))

####### 図1.5 回帰直線・確信区間・予測区間
plot(R親子身長,type="p",xlim=c(155,190),ylim=c(155,190),pch=1,
   cex.axis=1.5,cex.lab=1.3)
abline(60.856,0.646,lty=1,lwd=2.0)
lines(seq(155,190,2),out3$yhatc[,"0.025"],lty=2,lwd=2.0)
lines(seq(155,190,2),out3$yhatc[,"0.975"],lty=2,lwd=2.0)
lines(seq(155,190,2),out3$yastc[,"0.025"],lty=3,lwd=2.0)
lines(seq(155,190,2),out3$yastc[,"0.975"],lty=3,lwd=2.0)
#dev.copy2eps(file='z0104.eps',family='Japan1');            #図1.5ここまで

####### 表1.7 基準変数の事後予測分布の要約
Xnew=seq(155,190,5)
out4<-print(fit_tankaiki02,degits=1,Xnew=Xnew)

####### 図1.6 残差プロット(予測変数と残差の散布図)
plot(親子身長[,1],out2$resi,type="p",xlim=c(155,190),pch=1,
   cex.axis=1.5,cex.lab=1.3,xlab='両親',ylab='残差')
abline(h=0,lwd=2);                                          #図1.6ここまで
#dev.copy2eps(file='z0105.eps',family='Japan1')

##############宿題
#「パスタ」データ入力
x1<- c(130,268,104,185,128,147,162, 68,142,175); #実測
x2<- c(110,232,176,207,122,202,191,124,193,250); #目測
x<-cbind(x1,x2)
