﻿############  第2章　重回帰モデル
# Working directoryが 'toyoda' であることの確認
(n_wd<-getwd())                #working directory(wd)の確認
(setwd('./scrT'))              #wdの移動
source('../myfunc/myfunc.R')   #自作関数の読み込み
(setwd(n_wd))                  #wd戻す
library(cmdstanr)              #パッケージcmdstanrの呼び出し
library(posterior)             #パッケージposteriorの呼び出し
library(psych)
library(rgl)
prob<-c(0.025, 0.05, 0.5, 0.95, 0.975) #確率点の定義

################### 2. 1 直腸がんデータの分析
####### 表2.1 食物供給量と直腸がんの訂正死亡率
(can <- read.csv("./scrT/dat/直腸がん.csv",header=T))

####### 表2.2 「直腸がんデータ」の要約統計量
(平均<-apply(can,2,"mean"));                  #表2.2
vari<-function(x){mean((x-mean(x))^2)}        #標本分散を計算する関数の定義
(ＳＤ<-sqrt(apply(can,2,"vari")))             #標準偏差

####### 図2.1 多変量散布図
pairs.panels(can,smooth=F,density=F,ellipses=F,pch =16,rug=F,cex.axis=1.0)

####### 表2.3 「直腸がんデータ」の共分散行列
round(var(can),0)

####### 表2.4 「直腸がんデータ」の相関行列
round(cor(can),2)


################### 2. 2 重回帰モデル

##############関数を使った重回帰分析の実行
out1<-Reg(can[,3],can[,c(2,1)]/100)
#save(out1,file="./scrT/obje/stan_obje0201")
#load(file="./scrT/obje/stan_obje0201"); #予めstan( )で作ったobjectをロード

####### 表2.5 事後分布の要約
gqcal(out1$ext[,1:6])
pri1<-print(out1,3)


####### 図2.2 決定係数の事後分布(左) と
#######      「乳製品」の標準偏回帰係数の事後分布(右)
par(mfrow=c(1,2))
hist(out1$r2,breaks=50,cex.axis=2.0,cex.lab=2.0,main="",
     xlab="決定係数",ylab="")
hist(out1$sb[,1],breaks=50,cex.axis=2.0,cex.lab=2.0,main="",
     xlab="「乳製品」の標準偏回帰係数",ylab="")
par(mfrow=c(1,1))
#dev.copy2eps(file="z0202sw.eps",family="Japan1")


####### 図2.3 「乳製品」のROPE と「総熱量」のphc 曲線
par(mfrow=c(2,1))
phc01(seq(0,0.5,0.01),out1$sb[,1],cc="rope",byoga="yes",
      xlab="「乳製品」の標準偏回帰係数のROPE")
phc01(seq(0.65,1.2,0.01),out1$sb[,2],cc="gtc",byoga="yes",
      xlab="「総熱量」の標準偏回帰係数")
par(mfrow=c(1,1))
#dev.copy2eps(file="z0203sw.eps",family="Japan1")


####### 表2.6 回帰係数と決定係数のphc テーブル
phc01(seq(0,0.3,0.05),out1$sb[,1],cc="rope",byoga="no")
phc01(seq(0.68,0.8,0.02),out1$sb[,2],cc="gtc",byoga="no")


######## 図2.4  3次元散布図と回帰平面(単位：kcal)
plot3d(can[,"総熱量"],can[,"乳製品"],can[,"直腸がん"],size=7,
       lwd=1.5,type="h",xlab="", ylab="", zlab="");                      
planes3d(a=0.00594,b=-0.00403,c=-1,d=-10.97805, alpha = 0.1)
#rgl.snapshot("z0202.png")


######## 図2.5 残差プロット(予測値^y と残差e の散布図)
plot(pri1$yhat,pri1$resi,type="n",xlab="予測値",ylab="residual",
     cex.axis=2.0,cex.lab=2.0)
abline(h=0,lwd=2)
text(pri1$yhat,pri1$resi,rownames(can),cex=1.5)
#dev.copy2eps(file="z0205ss.eps",family="Japan1")


######## 表2.7 予測値・事後標準偏差・事後分布95%点・事後予測分布95%点
Xnew<-matrix(c(240,2800,140,2700,340,2900),,2,T);#乳製品・総熱量の順番
XnewS<-Xnew/100;             #推定した時の状態に合わせて100で割っている。
pri3<-print(out1,Xnew=XnewS)
(tab1406<-data.frame(
  乳製品=Xnew[,1],
  総熱量=Xnew[,2],
  予測値=round(pri3$yhatc[,"EAP"],1),
  postsd=round(pri3$yhatc[,"post.sd"],1),
#  確信025=round(pri3$yhatc[,"0.025"],1),
  確信95=round(pri3$yhatc[,"0.95"],1),
  sd=round(pri3$yastc[,"sd"],1),
#  予測025=round(pri3$yastc[,"0.025"],1),
  予測95=round(pri3$yastc[,"0.95"],1)))

########自習問題
#1. 決定係数のROPEのphcテーブルをseq(0.0,0.3,0.05)で作成しなさい。
#
#(1) R2=0.3 の列を正しく解釈しなさい。
#(2) 総熱量と乳製品は直腸がんを説明しているのか否か、あなたはどう思うか。
#
#2. 決定係数のphcテーブルをseq(0.6,0.9,0.05)で作成し、次の問いに答えよ。
#
#(1) R2=0.5 の列を正しく解釈しなさい。
#(2) R2=0.7 の列を正しく解釈しなさい。
#(3) 総熱量と乳製品は何％くらい直腸がんを説明していると、あなたは思うか。

