﻿########第3章　偏回帰係数の解釈
# Working directoryが 'toyoda' であることの確認
(n_wd<-getwd())                #working directory(wd)の確認
(setwd('./scrT'))              #wdの移動
source('../myfunc/myfunc.R')   #自作関数の読み込み
(setwd(n_wd))                  #wd戻す
library(cmdstanr)              #パッケージcmdstanrの呼び出し
library(posterior)             #パッケージposteriorの呼び出し
library(psych)
library(rgl)
prob<-c(0.025, 0.05, 0.5, 0.95, 0.975)     #確率点の定義
par<-c("bs","bs12","r2","r","indi","tota");#母数は共通なのでこの章のみ先に宣言

####################### 第3章で利用する関数ここから
###重回帰と単回帰の初期値
initi<-function(y,x1,x2){
 lm2<-lm(y ~x1+x2)
 lm1<-lm(x2~x1)
 ay<-as.numeric(coef(lm2)[1])
 a2<-as.numeric(coef(lm1)[1])
 b<-as.vector(coef(lm2)[c(2,3)])
 b12<- as.numeric(coef(lm1)[2])
 sigma_y<-as.numeric(summary(lm2)$sigma)
 sigma_2<-as.numeric(summary(lm1)$sigma)
 list(ay=ay,a2=a2,b=b, b12=b12,sigma_y=sigma_y, sigma_2=sigma_2)
}
####プリント関数
gqcalc3<-function(ext){
print('偏回帰係数y2');print(gqcal(ext$bs2))
print('偏回帰係数21');print(gqcal(ext$bs12))
print('直接効果y1　');print(gqcal(ext$bs1))
print('間接効果　　');print(gqcal(ext$indi))
print('総合効果　　');print(gqcal(ext$tota))
print('決定係数　　');print(gqcal(ext$r2))
}
####################### 第3章で利用する関数ここまで

###############重回帰と単回帰モデルのstanコード
mu_si_reg<-'
data {
  int<lower=0> n;                       //データ数 
  vector[n] y;                          //基準変数   
  matrix[n, 2] X;                       //予測変数行列   
}
transformed data {
  real sdy;                             //yの標準偏差
  vector[2] sdX;                        //x1,x2の標準偏差
  sdy = sqrt((variance(y) * (n - 1)) / n);            //01  yの標準偏差
  sdX[1] = sqrt((variance(X[ : , 1]) * (n - 1)) / n); //02  x1の標準偏差
  sdX[2] = sqrt((variance(X[ : , 2]) * (n - 1)) / n); //03  x2の標準偏差
}
parameters {
  real ay;                              //切片(重回帰)
  real a2;                              //切片(単回帰)
  vector[2] b;                          //偏回帰係数ベクトル  
  real b12;                             //偏回帰係数
  real<lower=0> sigma_y;                //誤差標準偏差(重回帰)
  real<lower=0> sigma_2;                //誤差標準偏差(単回帰)
}
transformed parameters {
  vector[n] yhat;                       //yhat
  vector[n] x2hat;                      //x2hat
  yhat = ay + X * b;                    //04 重回帰式 (3.4)式
  x2hat = a2 + X[ : , 1] * b12;         //05 単回帰式 (3.5)式
}
model {
  y ~ normal(yhat, sigma_y);            //06 重回帰モデル(2.3)式
  X[ : , 2] ~ normal(x2hat, sigma_2);   //07 単回帰モデル(1.6)式
}
generated quantities {
  vector[2] bs;
  real bs12;
  real<lower=0> vyhat;
  real<lower=0, upper=1> r2;
  real<lower=0, upper=1> r;
  real indi;
  real tota;
  vyhat = (variance(yhat) * (n - 1)) / n;//08 yhatの分散
  r2 = vyhat / (vyhat + sigma_y ^ 2);    //09 決定係数 (1.15)式, (2.8)式
  r = sqrt(r2);                          //10 重相関係数 (2.9)式
  bs[1] = (b[1] * sdX[1]) / sdy;         //11 標準偏回帰係数1 (2.7)式
  bs[2] = (b[2] * sdX[2]) / sdy;         //12 標準偏回帰係数2 (2.7)式
  bs12 = (b12 * sdX[1]) / sdX[2];        //13 標準回帰係数    (2.7)式
  indi = bs12 * bs[2];                   //14 間接効果 (3.6)式
  tota = indi + bs[1];                   //15 総合効果 (3.6)式
}
';

###### 3. 1 予測変数が多い場合の偏回帰係数の解釈の困難性
###### 3. 2 直接効果・間接効果・総合効果

###### 2. 1 直腸がんデータの分析  (第2章からの接続のため分類(3)が先頭)
(can <- read.csv("./scrT/dat/直腸がん.csv",header=T))
dataSet03<-list(n=nrow(can),y=can[,3], X=can[,c(2,1)] );      #入力
ini<-function(){initi(y=can[,3],x1=can[,2],x2=can[,1])};      #初期値
###########cmdstanrによる実行
modfile03 <- write_stan_file(mu_si_reg)             #書き出す一時ファイル
mod03 <- cmdstan_model(modfile03)                   #コンパイル
csrfit03 <- mod03$sample(data = dataSet03,chains = 5,iter_sampling = 20000,
    init =ini,iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext03<-as_draws_df(csrfit03$draws(par))
(colnames(ext03) <- gsub("\\[|\\]|,", "", colnames(ext03)))
#save(ext03,  file="./scrT/obje/stan_obje0303")
#load(file="./scrT/obje/stan_obje0303"); #予めstan( )で作ったobjectをロード

gqcalc3(ext=ext03);                      #推定値　p.30,bl.9



#######　3.3.1 (1)抑制変数が存在する場合
d01生物 <- read.csv("./scrT/dat/ch02/d01生物_raw.csv",header=T)
#######　図3.4 多変量散布図(1)
pairs.panels(d01生物,smooth=F,density=F,ellipses=F,pch =16,rug=F,cex.axis=1.0)
dataSet01<-list(n=nrow(d01生物),y=d01生物[,3], X=d01生物[,c(1,2)] )#入力
ini<-function(){initi(y=d01生物[,3],x1=d01生物[,1],x2=d01生物[,2])}#初期値

###########cmdstanrによる実行 #### コンパイルは 'mod03' を使用する
csrfit01 <- mod03$sample(data = dataSet01,chains = 5,iter_sampling = 20000,
    init =ini,iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext01<-as_draws_df(csrfit01$draws(par))
(colnames(ext01) <- gsub("\\[|\\]|,", "", colnames(ext01)))
#save(ext01,  file="./scrT/obje/stan_obje0301")
#load(file="./scrT/obje/stan_obje0301"); #予めstan( )で作ったobjectをロード

######## 図3.6 パス図(1)
gqcalc3(ext=ext01)
######## 図3.5  3次元散布図回帰平面(1)
plot3d(d01生物[,1],d01生物[,2],d01生物[,3],size=3,type="p"
 ,xlab="z1", ylab="z2", zlab="z3") 
planes3d(a=-0.410,b= 0.749,c=-1,d=0, alpha = 0.1)



#######　3.3.2 (2) x1からyへの直接効果が正で，相関・間接効果が負の場合
d02偏見 <- read.csv("./scrT/dat/ch02/d02偏見_raw.csv",header=T)
######## 図3.7 多変量散布図(2)
pairs.panels(d02偏見,smooth=F,density=F,ellipses=F,pch =16,rug=F,cex.axis=1.0)
dataSet02<-list(n=nrow(d02偏見),y=d02偏見[,3], X=d02偏見[,c(1,2)] )#入力
ini<-function(){initi(y=d02偏見[,3],x1=d02偏見[,1],x2=d02偏見[,2])}#初期値

###########cmdstanrによる実行 #### コンパイルは 'mod03' を使用する
csrfit02 <- mod03$sample(data = dataSet02,chains = 5,iter_sampling = 20000,
    init =ini,iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext02<-as_draws_df(csrfit02$draws(par))
(colnames(ext02) <- gsub("\\[|\\]|,", "", colnames(ext02)))
#save(ext02,  file="./scrT/obje/stan_obje0302")
#load(file="./scrT/obje/stan_obje0302"); #予めstan( )で作ったobjectをロード

######## 図3.9 パス図(2)
gqcalc3(ext=ext02)
######## 図3.8 3 次元散布図回帰平面(2)
plot3d(d02偏見[,1],d02偏見[,2],d02偏見[,3],size=3,type="p"
 ,xlab="z1", ylab="z2", zlab="z3") 
planes3d(a=0.341,b= 0.875,c=-1,d=0, alpha = 0.1)



#######　3.3.3 (4) x1からyへの間接効果が正で，相関・直接効果が負の場合
d04印象 <- read.csv("./scrT/dat/ch02/d04印象_raw.csv",header=T)
######## 図3.11 3 次元散布図回帰平面(4)
pairs.panels(d04印象,smooth=F,density=F,ellipses=F,pch =16,rug=F,cex.axis=1.0)
dataSet04<-list(n=nrow(d04印象),y=d04印象[,3], X=d04印象[,c(1,2)] )#入力
ini<-function(){initi(y=d04印象[,3],x1=d04印象[,1],x2=d04印象[,2])}#初期値

###########cmdstanrによる実行 #### コンパイルは 'mod03' を使用する
csrfit04 <- mod03$sample(data = dataSet04,chains = 5,iter_sampling = 20000,
    init =ini,iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext04<-as_draws_df(csrfit04$draws(par))
(colnames(ext04) <- gsub("\\[|\\]|,", "", colnames(ext04)))
#save(ext04,  file="./scrT/obje/stan_obje0304")
#load(file="./scrT/obje/stan_obje0304"); #予めstan( )で作ったobjectをロード

######## 図3.12 パス図(4)
gqcalc3(ext=ext04)
######## 図3.11 3 次元散布図回帰平面(4)
plot3d(d04印象[,1],d04印象[,2],d04印象[,3],size=3,type="p"
 ,xlab="z1", ylab="z2", zlab="z3") 
planes3d(a=-0.720,b=0.952,c=-1,d=0, alpha = 0.1)



#######　3.3.4 (5) x1からyへの間接効果が負で，相関・直接効果が正の場合
d05歩留 <- read.csv("./scrT/dat/ch02/d05歩留_raw.csv",header=T)
######## 図3.13 多変量散布図(5)
pairs.panels(d05歩留,smooth=F,density=F,ellipses=F,pch =16,rug=F,cex.axis=1.0)
dataSet05<-list(n=nrow(d05歩留),y=d05歩留[,3], X=d05歩留[,c(1,2)] )#入力
ini<-function(){initi(y=d05歩留[,3],x1=d05歩留[,1],x2=d05歩留[,2])}#初期値

###########cmdstanrによる実行 #### コンパイルは 'mod03' を使用する
csrfit05 <- mod03$sample(data = dataSet05,chains = 5,iter_sampling = 20000,
    init =ini,iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext05<-as_draws_df(csrfit05$draws(par))
(colnames(ext05) <- gsub("\\[|\\]|,", "", colnames(ext05)))
#save(ext05,  file="./scrT/obje/stan_obje0305")
#load(file="./scrT/obje/stan_obje0305"); #予めstan( )で作ったobjectをロード

######## 図3.15 パス図(5)
gqcalc3(ext=ext05)
######## 図3.14 3 次元散布図回帰平面(5)
plot3d(d05歩留[,1],d05歩留[,2],d05歩留[,3],size=3,type="p"
 ,xlab="z1", ylab="z2", zlab="z3") 
planes3d(a=0.460,b= 0.540,c=-1,d=0, alpha = 0.1)



#######　3.3.5 (6) 正の相加効果がある場合
d06中3  <- read.csv("./scrT/dat/ch02/d06中3_raw.csv", header=T)
######## 図3.16 多変量散布図(6)
pairs.panels(d06中3, smooth=F,density=F,ellipses=F,pch =16,rug=F,cex.axis=1.0)
dataSet06<-list(n=nrow(d06中3),y=d06中3[,3], X=d06中3[,c(1,2)] )#入力
ini<-function(){initi(y=d06中3[,3],x1=d06中3[,1],x2=d06中3[,2])}#初期値

###########cmdstanrによる実行 #### コンパイルは 'mod03' を使用する
csrfit06 <- mod03$sample(data = dataSet06,chains = 5,iter_sampling = 20000,
    init =ini,iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext06<-as_draws_df(csrfit06$draws(par))
(colnames(ext06) <- gsub("\\[|\\]|,", "", colnames(ext06)))
#save(ext06,  file="./scrT/obje/stan_obje0306")
#load(file="./scrT/obje/stan_obje0306"); #予めstan( )で作ったobjectをロード

######## 図3.18 パス図(6)
gqcalc3(ext=ext06)
######## 図3.17 3 次元散布図回帰平面(6)
plot3d(d06中3[,1],d06中3[,2],d06中3[,3],size=3,type="p"
 ,xlab="z1", ylab="z2", zlab="z3") 
planes3d(a=0.468,b=0.579 ,c=-1,d=0, alpha = 0.1)



#######　3.3.6 (7) 負の相加効果がある場合
d07孤独 <- read.csv("./scrT/dat/ch02/d07孤独_raw.csv",header=T)
######## 図3.19 多変量散布図(7)
pairs.panels(d07孤独,smooth=F,density=F,ellipses=F,pch =16,rug=F,cex.axis=1.0)
dataSet07<-list(n=nrow(d07孤独),y=d07孤独[,3], X=d07孤独[,c(1,2)] )#入力
ini<-function(){initi(y=d07孤独[,3],x1=d07孤独[,1],x2=d07孤独[,2])}#初期値

###########cmdstanrによる実行 #### コンパイルは 'mod03' を使用する
csrfit07 <- mod03$sample(data = dataSet07,chains = 5,iter_sampling = 20000,
    init =ini,iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext07<-as_draws_df(csrfit07$draws(par))
(colnames(ext07) <- gsub("\\[|\\]|,", "", colnames(ext07)))
#save(ext07,  file="./scrT/obje/stan_obje0307")
#load(file="./scrT/obje/stan_obje0307"); #予めstan( )で作ったobjectをロード

######## 図3.21 パス図(7)
gqcalc3(ext=ext07)
######## 図3.20 3 次元散布図回帰平面(7)
plot3d(d07孤独[,1],d07孤独[,2],d07孤独[,3],size=3,type="p"
 ,xlab="z1", ylab="z2", zlab="z3") 
planes3d(a=-0.657,b=0.433 ,c=-1,d=0, alpha = 0.1)



#######　3.3.7 (8) 多重共線性が生じている場合
d08身長 <- read.csv("./scrT/dat/ch02/d08身長_raw.csv",header=T)
######## 図3.22 多変量散布図(8)
pairs.panels(d08身長,smooth=F,density=F,ellipses=F,pch =16,rug=F,cex.axis=1.0)
dataSet08<-list(n=nrow(d08身長),y=d08身長[,3], X=d08身長[,c(1,2)] )#入力
ini<-function(){initi(y=d08身長[,3],x1=d08身長[,1],x2=d08身長[,2])}#初期値

###########cmdstanrによる実行 #### コンパイルは 'mod03' を使用する
csrfit08 <- mod03$sample(data = dataSet08,chains = 5,iter_sampling = 20000,
    init =ini,iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext08<-as_draws_df(csrfit08$draws(par))
(colnames(ext08) <- gsub("\\[|\\]|,", "", colnames(ext08)))
#save(ext08,  file="./scrT/obje/stan_obje0308")
#load(file="./scrT/obje/stan_obje0308"); #予めstan( )で作ったobjectをロード

######## 図3.24 パス図(8)
gqcalc3(ext=ext08)
######## 図3.23 3 次元散布図回帰平面(8)
plot3d(d08身長[,1],d08身長[,2],d08身長[,3],size=3,type="p"
 ,xlab="z1", ylab="z2", zlab="z3") 
planes3d(a= 4.859,b= -4.641 ,c=-1,d=0, alpha = 0.1)



#######　3.3.8 (9) 予測変数間の相関が0 に近い場合
d09筆実 <- read.csv("./scrT/dat/ch02/d09筆実_raw.csv", header=T)
######## 図3.25 多変量散布図(9)
pairs.panels(d09筆実,smooth=F,density=F,ellipses=F,pch =16,rug=F,cex.axis=1.0)
dataSet09<-list(n=nrow(d09筆実),y=d09筆実[,3], X=d09筆実[,c(1,2)] )#入力
ini<-function(){initi(y=d09筆実[,3],x1=d09筆実[,1],x2=d09筆実[,2])}#初期値

###########cmdstanrによる実行 #### コンパイルは 'mod03' を使用する
csrfit09 <- mod03$sample(data = dataSet09,chains = 5,iter_sampling = 20000,
    init =ini,iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext09<-as_draws_df(csrfit09$draws(par))
(colnames(ext09) <- gsub("\\[|\\]|,", "", colnames(ext09)))
#save(ext09,  file="./scrT/obje/stan_obje0309")
#load(file="./scrT/obje/stan_obje0309"); #予めstan( )で作ったobjectをロード

######## 図3.27 パス図(9)
gqcalc3(ext=ext09)
######## 図3.26 3 次元散布図回帰平面(9)
plot3d(d09筆実[,1],d09筆実[,2],d09筆実[,3],size=3,type="p"
 ,xlab="z1", ylab="z2", zlab="z3") 
planes3d(a= 0.434,b= 0.626 ,c=-1,d=0, alpha = 0.1)
