﻿############第4章　ロジスティック回帰/メタ分析
# Working directoryが 'toyoda' であることの確認
(n_wd<-getwd())                #working directory(wd)の確認
(setwd('./scrT'))              #wdの移動
source('../myfunc/myfunc.R')   #自作関数の読み込み
(setwd(n_wd))                  #wd戻す
library(cmdstanr)              #パッケージcmdstanrの呼び出し
library(posterior)             #パッケージposteriorの呼び出し
prob<-c(0.025, 0.05, 0.5, 0.95, 0.975) #確率点の定義

#############4.1　ロジスティック回帰（ベルヌイ分布）
### 表4.1 管理職か否かと年齢
(昇進<-read.csv("./scrT/dat/課長昇進.csv", header = TRUE))

### 表4.2 年齢別の管理職者の人数
table(昇進);                                           #表4.2

###図4.1 散布図と回帰直線
temp<-昇進[,2]+runif(200,-0.5,0.5); #図を見やすくする乱数付加
plot(temp,昇進[,1],ylim=c(-0.2,1.2),xlim=c(30,46),
  xlab="年齢",ylab="管理職");
  abline(a= -3.06335, b=0.09377, lwd=1.8);
  text(40,0.6,"回帰直線",cex=1.5);               #図4.1ここまで

### 4. 1. 2 ロジスティック変換
#オッズ変換　実数全体から確率への変換            (4.4)式
odds<-function(prob){prob/(1-prob)}
plot(odds,0,1,xlab="確率",main="オッズ変換")

#ロジット変換（対数オッズ変換）                  (4.5)式
log_odds<-function(prob){log(prob/(1-prob))}
plot(log_odds,0,1,xlab="確率",main="ロジット変換")

#ロジスティック変換（ロジット変換の逆変換）      (4.7)式
logistic<-function(yhat){1/(1+exp((-1)*yhat))}

###図4.2 逆ロジット関数logit-1( )
plot(logistic,-3,3,xlab="yhat",main="ロジスティック変換")


###############ロジスティック回帰(ベルヌイ分布)のstanコード
logistic_reg_01<-'
data {
  int<lower=0> n;                   //データ数 
  array[n] int<lower=0, upper=1> y; //基準変数(0-1データ)   
  vector[n] X;                      //予測変数ベクトル   
}
parameters {
  real a;                           //切片
  real b;                           //偏回帰係数 
}
transformed parameters {
  vector<lower=0, upper=1>[n] prob;
  for (i in 1 : n) {
    prob[i] = inv_logit(a + X[i] * b);
  }                                 //反応確率 (4.10)式
}
model {
  for (i in 1 : n) {
    y[i] ~ bernoulli(prob[i]);
  }                                 //ベルヌイ分布モデル(4.8)式
}
';

par<-c("a","b")                                        #母数
dataSet01 <-list(n=nrow(昇進),y=昇進[,1], X=昇進[,2] ) #入力

###########cmdstanrによる実行
modfile01 <- write_stan_file(logistic_reg_01)       #書き出す一時ファイル
mod01 <- cmdstan_model(modfile01)                   #コンパイル
csrfit01 <- mod01$sample(data = dataSet01,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext<-as_draws_df(csrfit01$draws(par))
(colnames(ext) <- gsub("\\[|\\]|,", "", colnames(ext)))

#save(ext, file="./scrT/obje/stan_obje0401")
#load(file="./scrT/obje/stan_obje0401"); #予めstan( )で作ったobjectをロード

#####事後分布のヒストグラム（教科書にはない）
hist(ext$a,breaks=100)
hist(ext$b,breaks=100)

###############生成量
p0_5<- (-1)*ext$a/ext$b;         #50%の確率で課長になれる年齢  p.46,l.3
oddshi<-exp(ext$b);              #年齢が1歳変わった時のオッズ比 (4.13)式
p35<- 1/(1+exp((-1)*(ext$a+ext$b*35))); #35歳で昇進できる確率   (4.14)式

###表4.3 母数と生成量の事後分布の要約
gqcal(ext$a);
gqcal(ext$b);
gqcal(p0_5);  
gqcal(oddshi);
gqcal(p35)

###図4.3 年齢の上昇に伴う昇進確率の変化
plot(temp,昇進[,1],ylim=c(-0.2,1.2),xlim=c(30,46),
    xlab="年齢",ylab="管理職",main="ロジスティック回帰")
  par(new=T)
  curve(1/(1+exp((-1)*((-18.679)+0.492*x))),30,46,lwd=1.8,
    ylim=c(-0.2,1.2),xlim=c(30,46),xlab="",ylab="");      #図4.3ここまで


##############  4.2  ロジスティック回帰（2項分布)

###表4.4 失語症の会話促進データ
(失語症訓練<-read.csv("./scrT/dat/失語症訓練.csv", header = TRUE));#表4.4

###図4.4 訓練回数と正答率の折れ線グラフ
(ab<-coef(glm(失語症訓練[,2]/失語症訓練[,1]~失語症訓練[,3])))
plot(失語症訓練[,3],失語症訓練[,2]/失語症訓練[,1],
      ylim=c(0,1),type="o",xlab="訓練回数",ylab="正解率")
      abline(a=ab[1],b=ab[2]);                             #図4.4ここまで


############図4.5   2項分布の確率関数を１つずつ見る
size<-10; 
for (prob in (1:9)/10 ){
  x<-c(0:size)
  plot(x, dbinom(x, size, prob), type="o", 
        main=paste("2項分布　size:",size,", prob:",prob) )
  locator(); #右クリックして"停止"
}

############図4.5   2項分布の確率関数
size<-10;x<-c(0:size)
plot(x, dbinom(x, size, 0.1), type="o",ylim=c(0,0.4),xlab="",ylab="") 
text(1,0.3,paste("p=",prob,sep=""))
for (prob in c(0.3,0.5,0.7,0.9)){
  par(new=T)
  plot(x, dbinom(x, size, prob), type="o",ylim=c(0,0.4),xlab="",ylab="") 
  text(prob*10,0.3,paste("p=",prob,sep=""))
}

###########試行数が多い場合（正規分布と変わらなくなる）教科書にはない
size<-100; 
for (prob in (1:9)/10 ){
  x<-c(0:size)
  plot(x, dbinom(x, size, prob), type="o", 
        main=paste("2項分布　size:",size,", prob:",prob) )
  locator(); #右クリックして"停止"
}

################ロジスティック回帰（2項分布）のstanコード
binom_reg<-'
data {
  int<lower=0> n;          //データ数 
  array[n] int<lower=0> t; //試行数 
  array[n] int<lower=0> y; //成功数 
  vector[n] X;             //予測変数ベクトル   
}
parameters {
  real a;                  //切片
  real b;                  //偏回帰係数
}
transformed parameters {
  vector<lower=0, upper=1>[n] prob;
  for (i in 1 : n) {
    prob[i] = inv_logit(a + X[i] * b);
  }                        //反応確率  ,(4.17)式
}
model {
  for (i in 1 : n) {
    y[i] ~ binomial(t[i], prob[i]);
  }                        //2項分布モデル(4.16)式,(4.17)式
}
';

par<-c("a","b")                                       #母数
dataSet02 <-list(n=nrow(失語症訓練),t=失語症訓練[,1],y=失語症訓練[,2],
                 X=失語症訓練[,3] )                   #入力

###########cmdstanrによる実行
modfile02 <- write_stan_file(binom_reg)             #書き出す一時ファイル
mod02 <- cmdstan_model(modfile02)                   #コンパイル
csrfit02 <- mod02$sample(data = dataSet02,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext<-as_draws_df(csrfit02$draws(par))
(colnames(ext) <- gsub("\\[|\\]|,", "", colnames(ext)))

#save(ext, file="./scrT/obje/stan_obje0402")
#load(file="./scrT/obje/stan_obje0402"); #予めstan( )で作ったobjectをロード

###############事後分布と生成量
p0_5<- (-1)*ext$a/ext$b;      #半分表現できるようになる訓練回数 p,50,l.1
oddshi<-exp(ext$b);           #１回訓練をした場合のオッズ比     p,50,l.4
p35<- 1/(1+exp((-1)*(ext$a+ext$b*35)));    #(4.14)式に35を代入  p,50,l.6
p35aste<-rbinom(length(p35),15,p35);       #(4.19)式

#####表4.5 母数と生成量の事後分布の要約
gqcal(ext$a  ,3);             #切片
gqcal(ext$b  ,3);             #傾き
gqcal(p0_5   ,1);             #半分表現できるようになる訓練回数
gqcal(oddshi ,2);             #１回訓練をした場合のオッズ比
gqcal(p35    ,3);             #あと５回訓練した場合の正答率
gqcal(p35aste,1);             #35回目の基本語数が15の場合の表現数の事後分布

#####図4.6 ロジスティック回帰分析(2 項分布)
plot(失語症訓練[,3],失語症訓練[,2]/失語症訓練[,1],         
    ylim=c(0,1),xlim=c(0,36),type="o",xlab="訓練回数",ylab="正解率")
  par(new=T)
  curve(1/(1+exp((-1)*((-1.803)+0.099*x))),0,36,lwd=1.8,
    ylim=c(0,1),xlim=c(0,36),xlab="",ylab="");
    lines(c(35,35),c(0.747,0.905),lwd=1.8)
    text(33,0.95,"95％確信区間");                          #図4.6ここまで

#####図4.7 x = 35, n = 15 のときの「表現数」の事後予測分布
hist(p35aste)

##### 4.3 メタ分析

######4.3.1 前向き研究
######表4.10 BCG の有効性に関する前向き研究のレビュー
(前向<-read.csv("./scrT/dat/結核前BCG.csv", header = TRUE))

round(risk実前<-前向[1,3]/(前向[1,3]+前向[1,4]),4);   #リスク(4.20)式
round(risk対前<-前向[1,5]/(前向[1,5]+前向[1,6]),4);   #      (4.21)式
round(risk実前/risk対前,3);                           #リスク比
round((前向[1,3]*前向[1,6])/(前向[1,4]*前向[1,5]),3); #オッズ比(4.22)式

######4.3.2 後ろ向き研究
#####表4.12 BCG の有効性に関する後ろ向き研究のレビュー
(後向<-read.csv("./scrT/dat/結核後BCG.csv", header = TRUE))   #表4.12
round(risk実後<-後向[1,3]/(後向[1,3]+後向[1,4]),4);   #リスク(4.23)式
round(risk対後<-後向[1,5]/(後向[1,5]+後向[1,6]),4);   #      (4.24)式
round((後向[1,3]*後向[1,6])/(後向[1,4]*後向[1,5]),3); #オッズ比(4.25)式


###########################メタ分析（オッズ比)のstanコード
meta_ana<-'
data {
  int<lower=0> m;           //研究数 
  array[m] int<lower=0> u1; //オッズ比の分子成功数 
  array[m] int<lower=0> n1; //オッズ比の分子試行数 
  array[m] int<lower=0> u0; //オッズ比の分母成功数 
  array[m] int<lower=0> n0; //オッズ比の分母試行数 
}
parameters {
  array[m] real a;          //切片
  array[m] real b;          //係数
  real mu_a;                //切片平均
  real mu_b;                //係数平均
  real sigma_a;             //切片SD
  real sigma_b;             //係数SD
}
model {
  for (i in 1 : m) {
    a[i] ~ normal(mu_a, sigma_a);                    //切片の事前分布(4.30)式
    b[i] ~ normal(mu_b, sigma_b);                    //係数の事前分布(4.31)式
    u1[i] ~ binomial(n1[i], inv_logit(a[i] + b[i])); //(4.26),(4.28),(4.34)式
    u0[i] ~ binomial(n0[i], inv_logit(a[i]));        //(4.26),(4.29),(4.35)式
  }
}
generated quantities {
  array[m] real Odds_r;
  real Odds_r_mu;
  Odds_r = exp(b);          //研究ごとのオッズ比(4.13)式,p.55,l.4
  Odds_r_mu = exp(mu_b);    //オッズ比の平均 p.55,l.7
}
';

##########4.3.4 前向き研究のメタ分析

par<-c("Odds_r","Odds_r_mu","mu_b","sigma_b")           #母数
dataSet前 <-list(m=nrow(前向),
      u1=前向$u11,n1=前向$u11+前向$u12,
      u0=前向$u21,n0=前向$u21+前向$u22)                 #入力

###########cmdstanrによる実行
modfile前 <- write_stan_file(meta_ana)              #書き出す一時ファイル  
mod前 <- cmdstan_model(modfile前)                   #コンパイル
csrfit前 <- mod前$sample(data = dataSet前,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext前<-as_draws_df(csrfit前$draws(par))
(colnames(ext前) <- gsub("\\[|\\]|,", "", colnames(ext前)))

#save(ext前,  file="./scrT/obje/stan_obje0403")
#load(file="./scrT/obje/stan_obje0403"); #予めstan( )で作ったobjectをロード


#####表4.10の Odds/EAP
gqcal(ext前[,1:13])

#####オッズ比の平均の事後分布のヒストグラム（教科書にはない）
hist(ext前$Odds_r_mu,breaks=200,xlim=c(0,1.0),main="")

#####オッズ比の事後予測分布(4.33)式のヒストグラム（教科書にはない）
Odds_r_aste<-exp(rnorm(length(ext前$mu_b),ext前$mu_b,ext前$sigma_b))
hist(Odds_r_aste,breaks=1000,xlim=c(0,4.0),main="")

####表4.11 前向き研究のオッズの平均の事後分布と事後予測分布の要約
gqcal(exp(ext前$mu_b),3);  #表4.11
gqcal(Odds_r_aste,3)

####phc p,55,bl.3
mean(1.0<Odds_r_aste)
mean(Odds_r_aste<0.8)


###########4.3.5 後ろ向き研究のメタ分析

par<-c("Odds_r","Odds_r_mu","mu_b","sigma_b")                #母数  
dataSet後 <-list(m=nrow(後向),
      u1=後向$u11,n1=後向$u11+後向$u21,
      u0=後向$u12,n0=後向$u12+後向$u22)                      #入力

###########cmdstanrによる実行
                                                       #コンパイルは"mod前"
csrfit後 <- mod前$sample(data = dataSet後,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext後<-as_draws_df(csrfit後$draws(par))
(colnames(ext後) <- gsub("\\[|\\]|,", "", colnames(ext後)))

#save(ext後,  file="./scrT/obje/stan_obje0404")
#load(file="./scrT/obje/stan_obje0404"); #予めstan( )で作ったobjectをロード

####表4.12の Odds/EAP
gqcal(ext後[,1:10])
Odds_r_aste<-exp(rnorm(length(ext後$mu_b),ext後$mu_b,ext後$sigma_b))

####表4.13 後ろ向き研究のオッズの平均の事後分布と事後予測分布の要約
gqcal(exp(ext後$mu_b),3)
gqcal(Odds_r_aste,3)

#####phc p.57.bl.1
mean(1.0<Odds_r_aste)
mean(Odds_r_aste<0.8)

####図4.8 各後ろ向き研究のオッズ比の事後分布
aa<-ext後[[1]]; for (i in 2:10){aa<-c(aa,ext後[[i]])}
boxplot(aa~rep(1:10,each=100000),outline=F,horizontal=T,cex.axis=1.5);
abline(v=1.0,lwd=2.0)


#################実習課題
###  1.
#load(file="./scrT/obje/stan_obje0401"); #予めstan( )で作ったobjectをロード
p0_5<- (-1)*ext$a/ext$b;       #50%の確率で課長になれる年齢
oddshi<-exp(ext$b);              #年齢が1歳変わった時のオッズ比
p35<- 1/(1+exp((-1)*(ext$a+ext$b*35))); #35歳で昇進できる確率
gqcal(p0_5)
gqcal(oddshi)

#
#  関数 phc01 使った部分は割愛
#

###  2.
#load(file="./scrT/obje/stan_obje0402"); #予めstan( )で作ったobjectをロード
p35<- 1/(1+exp((-1)*(ext$a+ext$b*35))); #あと５回訓練した場合の正答率
gqcal(p35)

#
#  関数 phc01 使った部分は割愛
#

###  3.
#load(file="./scrT/obje/stan_obje0404"); #予めstan( )で作ったobjectをロード
gqcal(ext後$Odds_r_mu) 
phc01(seq(0.25,0.70,0.01),ext後$Odds_r_mu,cc="gtc",byoga="yes",
      xlab="オッズの平均")
phc01(seq(0.25,0.70,0.05),ext後$Odds_r_mu,cc="gtc",byoga="no")
Odds_r_aste<-exp(rnorm(length(ext後$mu_b),ext後$mu_b,ext後$sigma_b))
gqcal(Odds_r_aste,3)
phc01(seq(0.05,2.1,0.01),ext後$Odds_r_mu,cc="gtc",byoga="yes",
      xlab="オッズの事後予測分布")
phc01(seq(0.05,0.3,0.05),ext後$Odds_r_mu,cc="gtc",byoga="no")
