﻿############第５章　ポアソンモデル／対数線形モデル
# Working directoryが 'toyoda' であることの確認
(n_wd<-getwd())                #working directory(wd)の確認
(setwd('./scrT'))              #wdの移動
source('../myfunc/myfunc.R')   #自作関数の読み込み
(setwd(n_wd))                  #wd戻す
library(cmdstanr)              #パッケージcmdstanrの呼び出し
library(posterior)             #パッケージposteriorの呼び出し
prob<-c(0.025, 0.05, 0.5, 0.95, 0.975) #確率点の定義

#################5.1 ポアソン分布

###図5.1 ポアソン分布の確率関数
par(mfrow=c(1,2))
  u<-c(0:5);p061<-dpois(u, 0.61);names(p061)<-u
  barplot(p061); text(4,0.3,"λ=0.61",cex=2.0)
  x<-c(0:10)
  plot(x, dpois(x, 2.0), type="o",xlab="",ylab="",xlim=c(0,10),ylim=c(0,0.3))
  for (lambda in seq(3.0,6.0,1.0)){
    par(new=T)
    plot(x,dpois(x,lambda),type="o",xlab="",ylab="",xlim=c(0,10),ylim=c(0,0.3))
  }
  text(c(2,3,4,5,6),c(0.29,0.25,0.22,0.19,0.18),
    paste("λ=",2:6,sep=""),cex=1.5)
par(mfrow=c(1,1));                                    #図5.1ここまで
#dev.copy2eps(file="z0401.eps",family="Japan1")

####lambdaが大きくなると正規分布に近似(教科書にはない)
size<-100; x<-c(0:size);
plot(x, dpois(x, 50), type="o",main=paste("ポアソン分布　lambda:",50))


###5.1.1 数値例(馬に蹴られて死亡した兵士数)
###表5.1 プロシア陸軍で馬に蹴られて死亡した兵士数
Horse<-c(dpois(0, 0.61),dpois(1, 0.61),dpois(2, 0.61),
         dpois(3, 0.61),dpois(4, 0.61),dpois(5, 0.61));
print(0:5);print(c(109, 65, 22, 3, 1, 0));round(Horse,4);round(Horse*200,1)

###5.1.2 数値例(当たりの本数の確率)
ice<-c(dpois(0, 0.25),dpois(1, 0.25),dpois(2, 0.25),           #(5.7)式
   dpois(3, 0.25),dpois(4, 0.25));round(ice,3)

#################5.2　ポアソン分布の推測
####5.2.1　爆弾命中数と区画数のデータ
爆弾<-c(rep(0,229),rep(1,211),rep(2,93),rep(3,35),rep(4,7),rep(5,1))

###表5.2 ロンドンにおける爆弾命中数と事後予測チェック の前半
table(爆弾)

barplot(table(爆弾),xlab="命中数",ylab="区画数");#ヒストグラム(教科書にはない)
round(mean(爆弾),3);                              #平均　p.62,l.2
round(mean((爆弾-mean(爆弾))^2),3);               #分散　p.62,l.3


#################ポアソン分布の推定のstanコード
Poisson<-'
data {
  int<lower=0> n;             //データ数 
  array[n] int<lower=0> u;    //計数データ 
}
parameters {
  real lambda;                //母数
}
model {
  for (i in 1 : n) {
    u[i] ~ poisson(lambda);
  }                           //ポアソン分布 (5.5)式
}
generated quantities {
  int<lower=0> uaste;         //予測分布 
  uaste = poisson_rng(lambda);//ポアソン乱数 (5.8)式
}
';

par<-c("lambda","uaste")                #母数
dataSet爆 <-list(n=length(爆弾),u=爆弾) #入力

###########cmdstanrによる実行
modfile爆 <- write_stan_file(Poisson)                  #書き出す一時ファイル
mod爆 <- cmdstan_model(modfile爆)                      #コンパイル
csrfit爆 <- mod爆$sample(data = dataSet爆,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext爆<-as_draws_df(csrfit爆$draws(par))
(colnames(ext爆) <- gsub("\\[|\\]|,", "", colnames(ext爆)))

#save(ext爆,  file="./scrT/obje/stan_obje0501")
#load(file="./scrT/obje/stan_obje0501"); #予めstan( )で作ったobjectをロード

##########表5.3 母数と生成量の事後分布の要約
sd爆<-sqrt(ext爆$lambda);                               #標準偏差
gqcal(ext爆$lambda,3);                                  #平均事後分布
gqcal(sd爆   ,3);                                       #sd事後分布

###表5.2 ロンドンにおける爆弾命中数と事後予測チェック の後半
round(table(ext爆$uaste)/length(ext爆$uaste),3);        #確率
round(table(ext爆$uaste)/length(ext爆$uaste)*576,1);    #事後予測値

#############5.2.2　授業欠席データ
#####表5.4 「心理統計学」の授業の欠席者数
(欠席<-c(0,1,2,0,0,1,2,0,2,0,1,0,0,1))

round(mean(欠席),3);                #p.63,l.3
round(mean((欠席-mean(欠席))^2),3); #p.63,l.3

par<-c("lambda","uaste")                #母数
dataSet欠 <-list(n=length(欠席),u=欠席) #入力

###########cmdstanrによる実行
#                                コンパイル済の "mod爆" を使用する
csrfit欠 <- mod爆$sample(data = dataSet欠,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext欠<-as_draws_df(csrfit欠$draws(par))
(colnames(ext欠) <- gsub("\\[|\\]|,", "", colnames(ext欠)))

#save(ext欠,  file="./scrT/obje/stan_obje0502")
#load(file="./scrT/obje/stan_obje0502"); #予めstan( )で作ったobjectをロード

######表5.5 母数と生成量の事後分布の要約
sd欠<-sqrt(ext欠$lambda)
gqcal(ext欠$lambda,3)
gqcal(sd欠        ,3)

#####表5.6 欠席者の事後予測チェック
round(table(ext欠$uaste)/length(ext欠$uaste),3);        #表5.6
round(table(ext欠$uaste)/length(ext欠$uaste)*14,1)

#表5.7 母数と生成量の事後分布の要約    (事後予測値の分布の比較）
予測確率2爆<-dpois(2,ext爆$lambda)
予測確率2欠<-dpois(2,ext欠$lambda)
予測値2爆<-予測確率2爆*576
予測値2欠<-予測確率2欠*14
gqcal(予測確率2爆,3)
gqcal(予測値2爆,1)
gqcal(予測確率2欠,3)
gqcal(予測値2欠,2)


############# 5.3   2つのポアソン分布の比較

####### 表5.8 鏡映描写課題における逸脱数の分布
u1<-c(1,0,2,3,2,3,3,1,4,3,1,0,0,1,2,1,3,2,1,4,1,1,1,1,3,2,3,1,1,5)
u2<-c(1,3,2,7,4,4,2,1,0,2,1,2,1,0,1,5,5,3,4,2,1,3,1,1,1,2,3,1,1,3,0)
mean(u1);mean(u2)
table(u1);table(u2)

################# ２つのポアソン分布の推測のstanコード
Poisson2<-'
data {
  int<lower=0> n1;             //データ数 
  int<lower=0> n2;             //データ数 
  array[n1] int<lower=0> u1;   //計数データ 
  array[n2] int<lower=0> u2;   //計数データ 
}
parameters {
  real lambda1;                //母数１
  real lambda2;                //母数２
}
model {
  for (i in 1 : n1) {
    u1[i] ~ poisson(lambda1);
  }                            //(5.10)式
  for (i in 1 : n2) {
    u2[i] ~ poisson(lambda2);
  }                            //(5.10)式
}
';
par<-c("lambda1","lambda2")                                #母数
dataSet鏡2 <-list(n1=length(u1),n2=length(u2),u1=u1,u2=u2) #入力

###########cmdstanrによる実行
modfile鏡2 <- write_stan_file(Poisson2)                #書き出す一時ファイル
mod鏡2 <- cmdstan_model(modfile鏡2)                    #コンパイル
csrfit鏡2 <- mod鏡2$sample(data = dataSet鏡2,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext鏡2<-as_draws_df(csrfit鏡2$draws(par))
(colnames(ext鏡2) <- gsub("\\[|\\]|,", "", colnames(ext鏡2)))

#save(ext鏡2, file="./scrT/obje/stan_obje0503")
#load(file="./scrT/obje/stan_obje0503"); #予めstan( )で作ったobjectをロード

#######表5.9  2群のポアソンモデルの母数の事後分布の要約
gqcal(ext鏡2$lambda1,3)
gqcal(ext鏡2$lambda2,3)

##### 表5.10 ROPE であるphc，ROPE でないphc
sequ<-seq(0,1.2,0.2);a<-ext鏡2$lambda2;b<-ext鏡2$lambda1
  phc01(sequ,a,b,cc="rope",byoga="no")
1-phc01(sequ,a,b,cc="rope",byoga="no")

#### 表5.11 λ2 のほうが大きいというphc
  phc01(sequ,a,b,cc="gtc", byoga="no")


####################5.4  ポアソン回帰

###### 表5.12 鏡映描写課題における逸脱数
(X<-data.frame(
   試行数=c(1:15),
   逸脱数=c(6,4,3,2,2,2,1,1,1,0,0,2,1,0,0)))

##### 図5.3 15 試行における逸脱数
plot(X[,1],X[,2],type="o",xlab="試行数",ylab="逸脱数")
  abline(a=coef(glm(X[,2]~X[,1]))[[1]],coef(glm(X[,2]~X[,1]))[[2]])

##### 5. 4. 1 指数変換   実数全体から正の実数への変換
##### 図5.4 指数変換exp( )
plot(exp,-2,2,xlab="a+bx",ylab="λ",main="", lwd=2.0,cex.lab=1.5)

####################ポアソン回帰のstanコード
Poisson_reg<-'
data {
  int<lower=0> n;              //データ数 
  array[n] int<lower=0> y;     //基準変数(計数データ)   
  vector[n] X;                 //予測変数ベクトル   
}
parameters {
  real a;                      //切片
  real b;                      //偏回帰係数
}
transformed parameters {
  vector<lower=0>[n] lambda;   //ラムダ 
  for (i in 1 : n) {
    lambda[i] = exp(a + X[i] * b);
  }                            //(5.14)式, (5.19)式
}
model {
  for (i in 1 : n) {
    y[i] ~ poisson(lambda[i]);
  }                            //ポアソン分布モデル(5.13)式,(5.15)式
}
generated quantities {
  array[n] int<lower=0> yaste; //予測分布 
  for (i in 1 : n) {
    yaste[i] = poisson_rng(lambda[i]);
  }                            //ポアソン分布モデル(5.20)式
}
';

par<-c("a","b","lambda","yaste")                  #母数
dataSet回帰 <-list(n=nrow(X),y=X[,2], X=X[,1] )   #入力

###########cmdstanrによる実行
modfile回帰 <- write_stan_file(Poisson_reg)            #書き出す一時ファイル
mod回帰 <- cmdstan_model(modfile回帰)                  #コンパイル
csrfit回帰 <- mod回帰$sample(data = dataSet回帰, chains=5,iter_sampling=20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext回帰<-as_draws_df(csrfit回帰$draws(par))
(colnames(ext回帰) <- gsub("\\[|\\]|,", "", colnames(ext回帰)))

#save(ext回帰,  file="./scrT/obje/stan_obje0504")
#load(file="./scrT/obje/stan_obje0504"); #予めstan( )で作ったobjectをロード

#### 生成量：予測変数を一単位変化させたときの母数の変化率
expb<-exp(ext回帰$b);    #(5.21)式

###### 表5.13 ポアソン回帰モデルの母数・生成量等の事後分布の要約
gqcal(ext回帰$a,3)
gqcal(ext回帰$b,3)
gqcal(ext回帰$lambda7,3)
gqcal(ext回帰$yaste7,3)
gqcal(expb,3)

###### 図5.5 逸脱数のポアソン回帰
lambda_i<-apply(ext回帰[,3:17],2,mean)
plot(X[,1],X[,2],type="o",xlab="試行数",ylab="逸脱数",xlim=c(1,15),ylim=c(0,6))
   par(new=T);                                        
plot(X[,1],lambda_i,type="l",xlab="",ylab="",lwd=2,xlim=c(1,15),ylim=c(0,6))


############# 5.5 対数線形モデル
############# 5.5.1　度数を構造化する
############# 表5.14 広告B の宣伝効果を調べる調査の結果
(u<-matrix(c(29,21,50,145),2,2,byrow=T))

##################分割表の分析01・度数の分解（2*2）のstanコード
Poisson_Cross01<-'
data {
  array[2, 2] int<lower=0> u;              //基準変数(計数データ)   
}
parameters {
  real mu;                                 //全平均
  real A1;                                 //要因Aの主効果
  real B1;                                 //要因Bの主効果
  real AB11;                               //交互作用
}
transformed parameters {
  matrix<lower=0>[2, 2] lambda;            //ラムダ 
  lambda[1, 1] = exp(mu + A1 + B1 + AB11); //(5.23)式, (5.24)式
  lambda[2, 1] = exp(mu - A1 + B1 - AB11);
  lambda[1, 2] = exp(mu + A1 - B1 - AB11);
  lambda[2, 2] = exp(mu - A1 - B1 + AB11);
}
model {
  u[1, 1] ~ poisson(lambda[1, 1]);         //(5.25)式
  u[2, 1] ~ poisson(lambda[2, 1]);
  u[1, 2] ~ poisson(lambda[1, 2]);
  u[2, 2] ~ poisson(lambda[2, 2]);
}
';

par<-c("mu","A1","B1","AB11","lambda")  #母数
dataSet分割表01<-list(u=u )             #入力

###########cmdstanrによる実行
modfile分割表01 <- write_stan_file(Poisson_Cross01)  #書き出す一時ファイル
mod分割表01 <- cmdstan_model(modfile分割表01)        #コンパイル
csrfit分割表01 <- mod分割表01$sample(data = dataSet分割表01,chains = 5,
  iter_sampling=20000, iter_warmup=1000, parallel_chains=5, seed=1234)
ext分割表01<-as_draws_df(csrfit分割表01$draws(par))
(colnames(ext分割表01) <- gsub("\\[|\\]|,", "", colnames(ext分割表01)))

#save(ext分割表01, file="./scrT/obje/stan_obje0505")
#load(file="./scrT/obje/stan_obje0505"); #予めstan( )で作ったobjectをロード

##### 表5.15 広告効果の分析の母数・生成量の事後分布の要約
gqcal(ext分割表01$mu)
gqcal(ext分割表01$A1)
gqcal(ext分割表01$B1)
gqcal(ext分割表01$AB11)


############# 5.5.2　比率を構造化する

############# 表5.16 タイタニック号の乗員・乗客の犠牲者数
(u<-matrix(c(4,118, 13,154, 106,422, 3,670),4,2,byrow=T));    #犠牲者
(n<-matrix(c(145,180, 106,179, 196,510, 23,862),4,2,byrow=T));#人数

#################分割表の分析02・比率の分解（a*b）のstanコード
Poisson_Cross02<-'
functions {
  vector zero_sum_vector(int a, vector m1A) {    //主効果ゼロ和
    vector[a] muA;
    for (i in 1 : (a - 1)) {
      muA[i] = m1A[i];
    }                                            //a-1まではソックリ代入
    muA[a] = -sum(m1A);                          //a番目に、和の-1倍代入
    return muA;
  }
  matrix zero_sum_matrix(int a, int b, matrix m1AB) { //交互作用ゼロ和
    vector[a - 1] m1a;                           //行和を入れるベクトル
    vector[b - 1] m1b;                           //列和を入れるベクトル
    matrix[a, b] muAB;                           //リターンする行列
    for (i in 1 : (a - 1)) {
      m1a[i] = 0.0;
    }                                            //ゼロクリア
    for (j in 1 : (b - 1)) {
      m1b[j] = 0.0;
    }                                            //ゼロクリア
    for (i in 1 : (a - 1)) {
      for (j in 1 : (b - 1)) {
        muAB[i, j] = m1AB[i, j];
      }
    }                                            //ソックリ代入
    for (i in 1 : (a - 1)) {
      for (j in 1 : (b - 1)) {
        m1a[i] = m1a[i] + m1AB[i, j];
      }
    }                                            //行和作る
    for (j in 1 : (b - 1)) {
      for (i in 1 : (a - 1)) {
        m1b[j] = m1b[j] + m1AB[i, j];
      }
    }                                            //列和作る
    for (i in 1 : (a - 1)) {
      muAB[i, b] = (-1) * m1a[i];
    }                                            //第b列に行和の-1倍代入
    for (j in 1 : (b - 1)) {
      muAB[a, j] = (-1) * m1b[j];
    }                                            //第a行に列和の-1倍代入
    muAB[a, b] = sum(m1a); //第a行第b列要素に合計代入、注意：マイナスではない
    return muAB;
  }
}
data {
  int<lower=0> a;                                //A水準数
  int<lower=0> b;                                //B水準数
  array[a, b] int<lower=0> u;                    //観察度数  
  array[a, b] int<lower=0> n;                    //試行数  
}
parameters {
  real mu;                                       //全平均
  vector[a - 1] m1A;                             //A平均1つ少ない
  vector[b - 1] m1B;                             //B平均1つ少ない
  matrix[a - 1, b - 1] m1AB;                     //交互作用1つ少ない
}
transformed parameters {
  matrix<lower=0>[a, b] lambda;                  //ラムダ 
  vector[a] A;                                   //A主効果
  vector[b] B;                                   //B主効果
  matrix[a, b] AB;                               //AB交互作用
  matrix[a, b] p;                                //比率
  A = zero_sum_vector(a, m1A);                   //制約 (5.28)式
  B = zero_sum_vector(b, m1B);                   //制約 (5.29)式
  AB = zero_sum_matrix(a, b, m1AB);              //制約 (5.30)式,(5.31)式
  for (i in 1 : a) {
    for (j in 1 : b) {
      p[i, j] = exp(mu + A[i] + B[j] + AB[i, j]);//(5.27)式, 実験計画
      lambda[i, j] = n[i, j] * p[i, j];          //(5.26)式, λの構造
    }
  }
}
model {
  for (i in 1 : a) {
    for (j in 1 : b) {
      u[i, j] ~ poisson(lambda[i, j]);           //(5.26)式, ボアソンモデル
    }
  }
}
';

par<-c("mu","A","B","AB","p")                             #母数
dataSet分割表02 <-list(a=nrow(u) ,b=ncol(u), u=u ,n=n )   #入力

###########cmdstanrによる実行
modfile分割表02 <- write_stan_file(Poisson_Cross02)  #書き出す一時ファイル
mod分割表02 <- cmdstan_model(modfile分割表02)        #コンパイル
csrfit分割表02 <- mod分割表02$sample(data = dataSet分割表02,chains = 5,
  iter_sampling=20000, iter_warmup=1000, parallel_chains=5, seed=1234)
ext分割表02<-as_draws_df(csrfit分割表02$draws(par))
(colnames(ext分割表02) <- gsub("\\[|\\]|,", "", colnames(ext分割表02)))

#save(ext分割表02,  file="./scrT/obje/stan_obje0506")
#load(file="./scrT/obje/stan_obje0506"); #予めstan( )で作ったobjectをロード

##表5.17 タイタニック号事件の犠牲者の分析の母数・生成量の事後分布の要約
gqcal(ext分割表02[1:23])

###表5.18 水準・交互作用の効果が0 より大きい(小さい) 確率
#表5.18 左
  round(mean(ext分割表02$A1>0),3)
  round(mean(ext分割表02$A2>0),3)
  round(mean(ext分割表02$A3>0),3)
  round(mean(ext分割表02$A4>0),3)
#表5.18 中
  round(mean(ext分割表02$B1>0),3)
#表5.18 右
  round(mean(ext分割表02$AB11>0),3)
  round(mean(ext分割表02$AB21>0),3)
  round(mean(ext分割表02$AB31>0),3)
  round(mean(ext分割表02$AB41>0),3)

###表5.19 行の比率が列の比率より大きいphc
phc02(0,ext分割表02[,c(16,20,17,21,18,22,19,23)])

#################実習課題
p_11<-ext分割表02$p11
p_31<-ext分割表02$p31
gqcal(p_11)
gqcal(p_31)
phc01(seq(0,1,0.01),p_31-p_11,cc="gtc",byoga="yes",xlab="死亡率のリスク差")
phc01(seq(0.38,0.48,0.01),p_31-p_11,cc="gtc",byoga="no")

phc01(seq(1,30,1),p_31/p_11,cc="gtc",byoga="yes",xlab="死亡率のリスク比")
phc01(seq(5,15,1),p_31/p_11,cc="gtc",byoga="no")

