﻿##########第６章　数種の分布による独立した１要因の推測
# Working directoryが 'toyoda' であることの確認
(n_wd<-getwd())                #working directory(wd)の確認
(setwd('./scrT'))              #wdの移動
source('../myfunc/myfunc.R')   #自作関数の読み込み
(setwd(n_wd))                  #wd戻す
library(cmdstanr)              #パッケージcmdstanrの呼び出し
library(posterior)             #パッケージposteriorの呼び出し
prob<-c(0.025, 0.05, 0.5, 0.95, 0.975) #確率点の定義

###################6.1 対数正規分布

#####図6.1 対数正規分布の確率密度関数
par(mfrow=c(1,2))
for (i in c(0.5,1.0,1.8)){
  d1<-function(x){dlnorm(x,0,i)}
  curve(d1,0,3,ylim=c(0,1.2),xlab="",ylab="",cex.axis=2.0)
  par(new=T)
}
  text(1.3,0.8,"σ=0.5",cex=2)
  text(0.8,0.5,"σ=1.0",cex=2)
  text(0.4,1.1,"σ=1.8",cex=2)
  text(2.3,0.8,"μ=0",cex=2.5);                      #1枚目ここまで
  par(new=F)
for (i in c(0.0,0.2,0.4,0.6)){
  d1<-function(x){dlnorm(x,i,1)}
  curve(d1,0,4,ylim=c(0,0.7),xlab="",ylab="",cex.axis=2.0)
  par(new=T)
}
  text(1.0,0.6,"μ=0.0",cex=2)
  text(1.2,0.5,"μ=0.2",cex=2)
  text(1.4,0.4,"μ=0.4",cex=2)
  text(0.8,0.3,"μ=0.6",cex=2)
  text(3.0,0.50,"σ=1.0",cex=2.5)
  par(new=F)
par(mfrow=c(1,1));                                    #図6.1ここまで
#dev.copy2eps(file="z0501.eps",family="Japan1")

###################6.1.1 数値例

### 図6.2 ある集団の年収の分布
hist(rlnorm(10000,15.3,0.5)/10000,xlim=c(0,2000),breaks=30,
  xlab="万円",ylab="",cex.axis=2.0,cex.lab=2.0)

### p.77 下部から p.78 上部
exp(15.3+0.5*0.5^2);                                  #平均   (6.3)式
exp(15.3);                                            #中央値 (6.5)式
exp(15.3-0.5^2);                                      #最頻値 (6.6)式
qlnorm(0.95, 15.3,0.5);                               #％点
qlnorm(0.99, 15.3,0.5)
qlnorm(0.999,15.3,0.5)

###################6.1.2 対数正規分布の推定
###################表6.1　職種I の30 人の年収(万円)
x1<-c(367, 1611, 263, 405, 754, 415, 837, 341, 396, 342,
      282,  296, 886, 412, 572, 471, 781, 531, 757, 723,
      870,  933, 392, 612, 394, 343, 372, 747, 280, 941)

#####################対数正規分布の推定のstanコード
lognormal01<-'
data {
  int<lower=0> n;                          //データ数 
  array[n] real<lower=0> x;                //データ 
}
parameters {
  real mu;                                 //母数μ (6.1)式
  real<lower=0> sigma;                     //母数σ (6.1)式
}
model {
  for (i in 1 : n) {
    x[i] ~ lognormal(mu, sigma);
  }                                        //対数正規分布 (6.2)式, (6.8)式
}
generated quantities {
  real heikin;                             //平均値
  real tyuouti;                            //中央値
  real saihin;                             //最頻値
  real sigma0;                             //標準偏差
  heikin = exp(mu + 0.5 * sigma ^ 2);      // (6.3)式
  tyuouti = exp(mu);                       // (6.5)式
  saihin = exp(mu - sigma ^ 2);            // (6.6)式
  sigma0 = sqrt(exp(2 * mu + sigma ^ 2) * (exp(sigma ^ 2) - 1)); // (6.4)式
}
';

par<-c("mu","sigma","heikin","tyuouti","saihin","sigma0")    #母数
dataSet対1 <-list(n=length(x1),x=x1)                         #入力

###########cmdstanrによる実行
modfile対1 <- write_stan_file(lognormal01)             #書き出す一時ファイル
mod対1 <- cmdstan_model(modfile対1)                   #コンパイル
csrfit対1 <- mod対1$sample(data = dataSet対1,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext対1<-as_draws_df(csrfit対1$draws(par))
(colnames(ext対1) <- gsub("\\[|\\]|,", "", colnames(ext対1)))

#save(ext対1,  file="./scrT/obje/stan_obje0601")
#load(file="./scrT/obje/stan_obje0601"); #予めstan( )で作ったobjectをロード

###### 表6.2 職種I の母数と生成量の事後分布の要約
gqcal(ext対1$mu)
gqcal(ext対1$sigma)
gqcal(ext対1$heikin ,0)
gqcal(ext対1$tyuouti,0)
gqcal(ext対1$saihin ,0)
gqcal(ext対1$sigma0 ,0)

##################6.1.3  2つの対数正規分布の比較
##################表6.3 職種II の30 人の年収(万円)
x2<-c(697, 681, 330, 176,  307, 1668, 641, 1150, 301, 365,
     1033, 373, 836, 367,  265,  217, 941,  278, 504, 299,
      865, 504, 790, 273, 2292,  385, 277,  622, 475,1205)

###################２つの対数正規分布の推測のstanコード
lognormal02<-'
data {
  int<lower=0> n1;                      //データ数 I
  int<lower=0> n2;                      //データ数 II
  array[n1] real<lower=0> x1;           //データ I
  array[n2] real<lower=0> x2;           //データ II 
}
parameters {
  array[2] real mu;                     //母数μ1,μ2 (6.8)式
  array[2] real<lower=0> sigma;         //母数σ1,σ2 (6.8)式
}
transformed parameters {
  
}
model {
  for (i in 1 : n1) {
    x1[i] ~ lognormal(mu[1], sigma[1]);
  }                                     //対数正規分布
  for (i in 1 : n2) {
    x2[i] ~ lognormal(mu[2], sigma[2]);
  }                                     //(6.8)式
}
generated quantities {
  array[2] real heikin;                 //平均値2つ
  array[2] real tyuouti;                //中央値2つ
  array[2] real saihin;                 //最頻値2つ
  array[2] real sigma0;                 //標準偏差2つ
  heikin[1] = exp(mu[1] + 0.5 * sigma[1] ^ 2); // (6.3)式 I
  tyuouti[1] = exp(mu[1]);              // (6.5)式 I
  saihin[1] = exp(mu[1] - sigma[1] ^ 2);// (6.6)式 I
  sigma0[1] = sqrt(exp(2 * mu[1] + sigma[1] ^ 2) * 
     (exp(sigma[1] ^ 2) - 1));          // (6.4)式 I
  heikin[2] = exp(mu[2] + 0.5 * sigma[2] ^ 2); // (6.3)式 II
  tyuouti[2] = exp(mu[2]);              // (6.5)式 II
  saihin[2] = exp(mu[2] - sigma[2] ^ 2);// (6.6)式 II
  sigma0[2] = sqrt(exp(2 * mu[2] + sigma[2] ^ 2) * 
      (exp(sigma[2] ^ 2) - 1));         // (6.4)式 II
}
';

par<-c("mu","sigma","heikin","tyuouti","saihin","sigma0")  #母数
dataSet対2 <-list(n1=length(x1),n2=length(x2),x1=x1,x2=x2) #入力

###########cmdstanrによる実行
modfile対2 <- write_stan_file(lognormal02)             #書き出す一時ファイル
mod対2 <- cmdstan_model(modfile対2)                   #コンパイル
csrfit対2 <- mod対2$sample(data = dataSet対2,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext対2<-as_draws_df(csrfit対2$draws(par))
(colnames(ext対2) <- gsub("\\[|\\]|,", "", colnames(ext対2)))

#save(ext対2, file="./scrT/obje/stan_obje0602")
#load(file="./scrT/obje/stan_obje0602"); #予めstan( )で作ったobjectをロード

######表6.4 職種II の母数と生成量の事後分布の要約
gqcal(ext対2$mu2)
gqcal(ext対2$sigma2)
gqcal(ext対2$heikin2 ,0)
gqcal(ext対2$tyuouti2,0)
gqcal(ext対2$saihin2 ,0)
gqcal(ext対2$sigma02 ,0)

#表6.5  2つの対数正規分布の位置の差の事後分布の要約  平均値 中央値 最頻値の差
gqcal(ext対2$heikin2-ext対2$heikin1)
gqcal(ext対2$tyuouti2-ext対2$tyuouti1)
gqcal(ext対2$saihin2-ext対2$saihin1)

######表6.6 平均値，中央値，最頻値に差のあるphc
sequ66<-c(0,10,20,30)
phc01(sequ66,ext対2$heikin2 ,ext対2$heikin1 ,cc="gtc",byoga="no")
phc01(sequ66,ext対2$tyuouti2,ext対2$tyuouti1,cc="gtc",byoga="no")
phc01(sequ66,ext対2$saihin1 ,ext対2$saihin2 ,cc="gtc",byoga="no")

######表6.7 平均値，中央値，最頻値がROPE であるphc
sequ67<-c(0,50,100,120)
phc01(sequ67,ext対2$heikin2 ,ext対2$heikin1 ,cc="rope",byoga="no")
phc01(sequ67,ext対2$tyuouti2,ext対2$tyuouti1,cc="rope",byoga="no")
phc01(sequ67,ext対2$saihin1 ,ext対2$saihin2 ,cc="rope",byoga="no")


########################6.2  対数正規分布による1要因実験
#####表6.9 鏡映描写課題の群ごとの15 試行目のタイムと逸脱数
(鏡映描写01<-read.csv("./scrT/dat/鏡映描写01.csv", header = TRUE))

#####図6.3 鏡映描写課題における群ごとのタイム(秒)
boxplot(鏡映描写01$time~鏡映描写01$group,             #図6.3
     xlab="group", ylab="",cex.axis=2.0,cex.lab=2.0)
#dev.copy2eps(file="z0503.eps",family="Japan1")
tapply(鏡映描写01$time,鏡映描写01$group,mean); #群ごとの平均タイム p.82


########################対数正規分布による1要因実験のstanコード
one_factor_lognormal<-'
data {
  int<lower=0> n;                         //全データ数 
  int<lower=0> J;                         //群数
  vector[n] y;                            //基準変数 
  array[n] int<lower=0> k;                //分類変数 
}
parameters {
  vector[J] mu;                           //各群のmu     J個
  vector<lower=0>[J] sigma;               //各群のsigma  J個
}
model {
  for (i in 1 : n) {
    y[i] ~ lognormal(mu[k[i]], sigma[k[i]]);
  }                                       //(6.14)式
}
generated quantities {
  vector[J] mu0;                          //各群の平均     J個
  vector[J] medi0;                        //各群の中央値   J個
  vector[J] mode0;                        //各群の最頻値   J個
  vector<lower=0>[J] sigma0;              //各群の標準偏差 J個
  for (j in 1 : J) {
    mu0[j] = exp(mu[j] + (0.5 * sigma[j] ^ 2)); // (6.3)式
    medi0[j] = exp(mu[j]);                // (6.5)式
    mode0[j] = exp(mu[j] - sigma[j] ^ 2); // (6.6)式
    sigma0[j] = sqrt(exp(2 * mu[j] + sigma[j] ^ 2) * 
        (exp(sigma[j] ^ 2) - 1));         // (6.4)式
  }
}
';
par<-c("mu","sigma","mu0","medi0","mode0","sigma0")               #母数
dataSetlogn <-list(n=length(鏡映描写01$time), 
  J=max(鏡映描写01$group),y=鏡映描写01$time, k=鏡映描写01$group ) #入力

###########cmdstanrによる実行
modfilelogn <- write_stan_file(one_factor_lognormal)    #書き出す一時ファイル
modlogn <- cmdstan_model(modfilelogn)                   #コンパイル
csrfitlogn <- modlogn$sample(data=dataSetlogn,chains=5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
extlogn<-as_draws_df(csrfitlogn$draws(par))
(colnames(extlogn) <- gsub("\\[|\\]|,", "", colnames(extlogn)))

#save(extlogn, file="./scrT/obje/stan_obje0603")
#load(file="./scrT/obje/stan_obje0603"); #予めstan( )で作ったobjectをロード

##### 表6.10 1 要因対数正規分布モデルの母数の事後分布
gqcal(extlogn[,1:18])

##### 図6.4 鏡映描写課題・群ごとのタイムの分布(対数正規分布)
lognor01<-function(x){dlnorm(x,3.153, 0.685)}
lognor02<-function(x){dlnorm(x,2.981, 0.449)}
lognor03<-function(x){dlnorm(x,2.822, 0.488)}
curve(lognor01,0,80,ylim=c(0,0.055),xlab="",ylab="",lty=1,lwd=2)
par(new=T)
curve(lognor02,0,80,ylim=c(0,0.055),xlab="",ylab="",lty=2,lwd=2)
par(new=T)
curve(lognor03,0,80,ylim=c(0,0.055),xlab="time",
  ylab="確率密度",lty=3,lwd=2)
text(60,0.01,"第1群",cex=1.6)
text(30,0.04,"第2群",cex=1.6)
text( 5,0.05,"第3群",cex=1.6)                       #図6.4ここまで

#表6.11 行j の水準の平均が列j′の水準の平均より大きい確率
phc02(0,extlogn[,7:9])

#休憩群が非利き手群より長く、かつ　非利き手群が利き手群より長い確率
round(mean((extlogn$mu01>extlogn$mu02)&
           (extlogn$mu02>extlogn$mu03)),3)    #(6.17)式,(6.18)式


########################6. 3 ポアソン分布による1 要因実験
#####表6.9 鏡映描写課題の群ごとの15 試行目のタイムと逸脱数
(鏡映描写01<-read.csv("./scrT/dat/鏡映描写01.csv", header = TRUE))

##### 図6.5 鏡映描写課題における群 逸脱数ごとの人数の棒グラフ
barplot(table(鏡映描写01$逸脱,鏡映描写01$group),
        xlab="群ごとの逸脱数",ylab="人数",beside=T,legend.text=0:5 )

#群ごとの平均逸脱数　p.84,bl.8
tapply(鏡映描写01$逸脱,鏡映描写01$group,mean)


############################ポアソン分布による1要因実験のstanコード
one_factor_Poisson<-'
data {
  int<lower=0> n;            //全データ数 
  int<lower=0> J;            //群数
  array[n] int<lower=0> y;   //基準変数   
  array[n] int<lower=0> k;   //分類変数 
}
parameters {
  vector<lower=0>[J] lambda; //各群のlambda J個
}
model {
  for (i in 1 : n) {
    y[i] ~ poisson(lambda[k[i]]);
  }                          //(6.19)式
}
';

par<-c("lambda")                                                   #母数
dataSetpo <-list(n=length(鏡映描写01$逸脱),
   J=max(鏡映描写01$group),y=鏡映描写01$逸脱, k=鏡映描写01$group ) #入力

###########cmdstanrによる実行
modfilepo <- write_stan_file(one_factor_Poisson)    #書き出す一時ファイル
modpo <- cmdstan_model(modfilepo)                   #コンパイル
csrfitpo <- modpo$sample(data = dataSetpo,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
extpo<-as_draws_df(csrfitpo$draws(par))
(colnames(extpo) <- gsub("\\[|\\]|,", "", colnames(extpo)))

#save(extpo, file="./scrT/obje/stan_obje0604")
#load(file="./scrT/obje/stan_obje0604"); #予めstan( )で作ったobjectをロード

##### 表6.12 1 要因ポアソン分布モデルの母数の事後分布
gqcal(extpo[,1:3])

##### 図6.6 鏡映描写実験・群ごとの逸脱数の分布(ポアソン分布)
x<-0:5
plot(x,dpois(x,1.100),type="o",ylim=c(0,0.7),xlab="",ylab="",lty=1,lwd=2)
par(new=T)
plot(x,dpois(x,0.666),type="o",ylim=c(0,0.7),xlab="",ylab="",lty=2,lwd=2)
par(new=T)
plot(x,dpois(x,0.401),type="o",ylim=c(0,0.7),
  xlab="逸脱数",ylab="確率",lty=3,lwd=2)
  legend(3,0.6,paste("第",1:3,"群"),lty=1:3)          #図6.6ここまで

##表6.13 行 j の水準の平均が列 j' の水準の平均より大きい確率
phc02(0,extpo[,1:3])

##休憩群が非利き手群より長く、かつ　非利き手群が利き手群より長い確率
round(mean((extpo$lambda1>extpo$lambda2)
          &(extpo$lambda2>extpo$lambda3)),3)    #(6.20)式


###################6.4   2項分布による1 要因実験
####表6.14 会話促進に関する訓練における群ごとの基本語数と表現数
(会話促進<-read.csv("./scrT/dat/会話促進.csv",header = TRUE))

####図6.7 会話促進訓練における群ごとの表現率
boxplot(会話促進$表現数/会話促進$基本語数~会話促進$group,  
  xlab="group", ylab="", cex.axis=2.0,cex.lab=2.0)
#dev.copy2eps(file="z0507.eps",family="Japan1")

#####群ごとの表現率の平均　p.86,bl.7
round(tapply(会話促進$表現数/会話促進$基本語数,会話促進$group,mean),3)

#############################2項分布による1要因実験のstanコード
one_factor_binom<-'
data {
  int<lower=0> n;                //全データ数 
  int<lower=0> J;                //群数
  array[n] int<lower=0> y;       //基準変数   
  array[n] int<lower=0> m;       //試行数  
  array[n] int<lower=0> k;       //分類変数 
}
parameters {
  vector<lower=0, upper=1>[J] p; //各群の確率 J個
}
model {
  for (i in 1 : n) {
    y[i] ~ binomial(m[i], p[k[i]]);
  }                              //(6.21)式
}
';

par<-c("p")                                                       #母数
dataSetbi <-list(n=length(会話促進$表現数),J=max(会話促進$group),
   y=会話促進$表現数, m=会話促進$基本語数, k=会話促進$group )     #入力

###########cmdstanrによる実行
modfilebi <- write_stan_file(one_factor_binom)      #書き出す一時ファイル
modbi <- cmdstan_model(modfilebi)                   #コンパイル
csrfitbi <- modbi$sample(data = dataSetbi,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
extbi<-as_draws_df(csrfitbi$draws(par))
(colnames(extbi) <- gsub("\\[|\\]|,", "", colnames(extbi)))

#save(extbi, file="./scrT/obje/stan_obje0605")
#load(file="./scrT/obje/stan_obje0605"); #予めstan( )で作ったobjectをロード

###### 表6.15 1 要因2 項分布モデルの母数の事後分布
gqcal(extbi[,1:3])

##### 図6.8 EAP 推定値による各群の2 項分布
x<-0:10
plot(x,dbinom(x,10,0.338),type="o",ylim=c(0,0.3),xlab="",ylab="",lty=1,lwd=2)
par(new=T)
plot(x,dbinom(x,10,0.510),type="o",ylim=c(0,0.3),xlab="",ylab="",lty=2,lwd=2)
par(new=T)
plot(x,dbinom(x,10,0.613),type="o",ylim=c(0,0.3),
  xlab="表現数",ylab="確率",lty=3,lwd=2)
legend(8,0.27,paste("第",1:3,"群"),lty=1:3);          #図6.8ここまで


###表6.16 行 j の水準の比率が列 j' の水準の比率より大きいphc
phc02(0,extbi[,1:3])

#連言命題が正しい確率(仮説C に対応　教科書にはない)
round(mean((extbi$p1<extbi$p2)&(extbi$p2<extbi$p3)),3)


############6. 5 正規分布による1 要因実験(変量モデル)
###表6.17 陸上100m 種目の選手6 名の10 回の成績
(M100走<-read.csv("./scrT/dat/M100走.csv", header = TRUE))

####図6.9 陸上100m 種目の選手6 名の10 回のタイムのボックスプロット
boxplot(M100走$time~M100走$選手, xlab="選手番号", ylab="time");#図6.9
#dev.copy2eps(file="z0501.eps",family='Japan1')

##############正規分布による１要因実験(変量モデル)のstanコード
one_factor_normal<-'
data {
  int<lower=0> n;                                //全データ数 
  int<lower=0> J;                                //群数
  vector[n] y;                                   //基準変数   
  array[n] int<lower=0> k;                       //分類変数 
}
parameters {
  vector[J] mu;                                  //各群の平均 J個
  real<lower=0> sigma;                           //誤差SD
  real<lower=0> s_mu;                            //要因SD
  real mu0;                                      //全平均
}
model {
  y ~ normal(mu[k], sigma);                      //(6.24)式
  mu ~ normal(mu0, s_mu);                        //(6.25)式
}
generated quantities {
  real<lower=0, upper=1> eta2;                   //説明率
  eta2 = (s_mu ^ 2) / ((s_mu ^ 2) + (sigma ^ 2));//(6.27)式
}
';

par<-c("mu","sigma","s_mu","mu0","eta2")     #母数
dataSetnorm <-list(n=length(M100走$time), 
   J=max(M100走$選手),y=M100走$time, k=M100走$選手 ) #入力

###########cmdstanrによる実行
modfilenorm <- write_stan_file(one_factor_normal)       #書き出す一時ファイル
modnorm <- cmdstan_model(modfilenorm)                   #コンパイル
csrfitnorm <- modnorm$sample(data=dataSetnorm,chains=5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
extnorm<-as_draws_df(csrfitnorm$draws(par))
(colnames(extnorm) <- gsub("\\[|\\]|,", "", colnames(extnorm)))

#save(extnorm, file="./scrT/obje/stan_obje0606")
#load(file="./scrT/obje/stan_obje0606"); #予めstan( )で作ったobjectをロード

##### 表6.18 変量モデルの母数の事後分布の要約
gqcal(extnorm[,1:10])

##### 表6.19 行の選手の平均が列の選手の平均より大きい確率
phc02(0,extnorm[,1:6])

#################実習課題
###  1. 6.1.2項「年収の中央値はc円より大きい」のphc曲線とphcテーブルを作成しなさい。

#
#     正解は割愛
#

###  2.  6.1.3項「職種IIのほうが職種Iより年収の最頻値がc円高い」のphc曲線とphcテーブルを作成しなさい。

#
#     正解は割愛
#

###  3.   6.2節、(6.15)式と(6.16)式のphcテーブルを作成しなさい。

#
#     正解は割愛
#

###  4.
#
#     正解は割愛
#

###  5.
lambda_sa12 <- extpo$lambda1-extpo$lambda2
lambda_sa23 <- extpo$lambda2-extpo$lambda3
phc01(seq(0,1,0.05),lambda_sa12,cc="gtc",byoga="yes",
      xlab="休憩群と非利き手群の差")
phc01(seq(0,1,0.05),lambda_sa23,cc="gtc",byoga="yes",
      xlab="非利き手群と利き手群の差")
phc01(seq(0,0.3,0.05),lambda_sa12,cc="gtc",byoga="no")
phc01(seq(0,0.2,0.05),lambda_sa23,cc="gtc",byoga="no")


###  6.
#
#     正解は割愛
#


###  7.
hiritu_sa21 <- extbi$p2-extbi$p1
hiritu_sa32 <- extbi$p3-extbi$p2
phc01(seq(0,0.3,0.01),hiritu_sa21,cc="gtc",byoga="yes",
      xlab="2か月と1か月の差")
phc01(seq(0,0.2,0.01),hiritu_sa32,cc="gtc",byoga="yes",
      xlab="3か月と2か月の差")
phc01(seq(0,0.15,0.03),hiritu_sa21,cc="gtc",byoga="no")
phc01(seq(0,0.1,0.02),hiritu_sa32,cc="gtc",byoga="no")


###  8.
eta2<-extnorm$eta2
phc01(seq(0.1,0.8,0.05),eta2,cc="gtc",byoga="yes",
      xlab="説明率")
phc01(seq(0,3,0.02),extnorm$mu1-extnorm$mu2,cc="gtc",byoga="yes",
      xlab="選手1と選手2の差")
phc01(seq(0,0.15,0.03),eta2,cc="gtc",byoga="no")
phc01(seq(0,1,0.2),extnorm$mu1-extnorm$mu2,cc="gtc",byoga="no")


###############################鏡映描写実験データ
#タイム
休time<-c(40.03,35.04,32.38,23.25,24.45,
      30.66,32.26,21.18,25.53,37.66,
      22.51,33.46,14.87,23.52,26.63,
      35.27,54.38,17.59,39.75,29.63,
      36.72,26.77,24.67,61.77,31.28,
      27.83,27.78,33.15)
非time<-c(21.39,13.89,24.76,24.20,17.77,
      24.90,24.15,13.56,21.86,17.58,
      34.03,20.41,19.09,27.90,29.91,
      20.88,26.44,21.29,10.71,21.92,
      17.09,31.98,31.18,18.37,16.53,
      27.40,26.03)
利time<-c(08.63,12.40,15.11,23.68,16.28,
      29.56,25.40,12.53,19.00,21.59,
      21.93,23.03,17.09,11.37,23.84,
      22.03,18.98,12.40,25.77,12.62,
      13.78,10.43,24.49,34.28,13.74,
      18.22,18.56,21.22)

#逸脱回数
休freq<-c(1,1,0,2,1,0,2,2,2,2,0,4,2,0,1,2,2,1,6,5,1,2,3,1,0,1,2,2)
非freq<-c(2,0,0,0,1,2,0,3,0,1,2,1,0,0,1,0,1,2,0,0,2,1,1,0,2,4,1)
利freq<-c(0,1,0,1,1,0,1,1,0,1,0,0,1,2,0,1,0,0,0,0,0,0,0,0,0,0,1,0)
