﻿############第7章　共分散分析/傾向スコア
# Working directoryが 'toyoda' であることの確認
(n_wd<-getwd())                #working directory(wd)の確認
(setwd('./scrT'))              #wdの移動
source('../myfunc/myfunc.R')   #自作関数の読み込み
(setwd(n_wd))                  #wd戻す
library(cmdstanr)              #パッケージcmdstanrの呼び出し
library(posterior)             #パッケージposteriorの呼び出し
library(pROC)
library(Matching)
prob<-c(0.025, 0.05, 0.5, 0.95, 0.975) #確率点の定義


####################### 第7章で利用する関数ここから
#2群の2変量データの散布図をpchを変えて描く関数
plot2g<-function(y,x,a,ba=5,xlab="pre test"){
  plot(x,y,pch=ifelse(a-1,1,16),xlab=xlab,ylab="post test",
     cex.lab=1.5,cex.axis=1.5,xlim=c(min(x)-ba,max(x)+ba),
     ylim=c(min(y)-ba,max(y)+ba))
}
#2群の散布図と2本の傾き共通の回帰直線を描く関数
plot2gequ<-function(y,x,a,ext,ba=5,xlab="pre test"){
  plot(x,y,pch=ifelse(a-1,1,16),xlab=xlab,ylab="post test",
     cex.lab=1.5,cex.axis=1.5,xlim=c(min(x)-ba,max(x)+ba),
     ylim=c(min(y)-ba,max(y)+ba))
  abline(mean(ext$a1),mean(ext$b))     
  abline(mean(ext$a2),mean(ext$b))     
}
#標本分散
bunsan<-function(x){mean((x-mean(x))^2)}

#散布図に傾きの異なった2本の回帰直線を描く関数
plot2gdef<-function(y,x,a,ext,ba=3){
  plot(x,y,pch=ifelse(a-1,1,16),xlab="pre test",ylab="post test",
     cex.lab=1.5,cex.axis=1.5,xlim=c(min(x)-ba,max(x)+ba),
     ylim=c(min(y)-ba,max(y)+ba))
  abline(mean(ext$a1),mean(ext$b1))     
  abline(mean(ext$a2),mean(ext$b2))     
}
#傾きの異なった2本の回帰直線の予測値の差のEAPと95％確信区間を描く関数}
plot2gdefdef<-function(x,ext){
  leng=30
  xlabe<-seq(min(x),max(x),length=leng)
  yhat_sa<-matrix(0,nrow(ext),leng)
  yhat_sa_HL<-matrix(0,leng,2)
  for (i in (1:leng)){
    yhat_sa[,i]<-ext$a1-ext$a2+(ext$b1-ext$b2)*xlabe[i]}
  for (i in (1:leng)){
    yhat_sa_HL[i,]<-quantile(yhat_sa[,i],prob=c(0.025,0.975))}
  plot(xlabe,colMeans(yhat_sa),ylim=c(min(yhat_sa_HL),max(yhat_sa_HL)),
    type="l",ylab="",xlab="",lwd=2.0)
  abline(a=0,b=0,lwd=2.0)
  par(new=T)
  plot(xlabe,yhat_sa_HL[,1],ylim=c(min(yhat_sa_HL),max(yhat_sa_HL)),
    type="l",ylab="",xlab="",lty=2,lwd=2.0)
  par(new=T)
  plot(xlabe,yhat_sa_HL[,2],ylim=c(min(yhat_sa_HL),max(yhat_sa_HL)),
    type="l",ylab="postの予測値の差",xlab="pre値",lty=2,lwd=2.0)
}
#######################第７章で利用する関数ここまで

##### 7.2 傾きが共通した共分散分析

##### 7. 2. 1 「データ1」の分析

##### 表7.1 「データ1」の平均値
学習01<-read.csv("./scrT/dat/学習01.csv", header = TRUE)
round(tapply(学習01$post,学習01$subject,mean),1)
round(tapply(学習01$pre ,学習01$subject,mean),1)

##### 図7.1 「データ1」の散布図
plot2g(y=学習01$post,x=学習01$pre,a=学習01$subject,ba=3)
#dev.copy2eps(file="z0701.eps",family='Japan1')

#############################共分散分析(傾きが共通のモデル)のstanコード
ANCOVAequ<-'
data {
  int<lower=0> n;                   //データ数 
  vector[n] y;                      //基準変数   
  vector[n] x;                      //予測変数   
  array[n] int<lower=0, upper=2> k; //分類変数 
}
parameters {
  vector[2] a;                      //切片
  real b;                           //回帰係数 
  real<lower=0> sigma;              //誤差標準偏差
}
transformed parameters {
  vector[n] yhat;                   //予測値 yhat  
  for (i in 1 : n) {
    yhat[i] = a[k[i]] + x[i] * b;
  }                                 //(7.4)式,(7.5)式
}
model {
  y ~ normal(yhat, sigma);          //(7.1)式,(7.2)式,(7.3)式
}
';

par<-c("a","b","sigma");                                                  #母数
dataSetequ01<-list(n=nrow(学習01),y=学習01[,2],x=学習01[,1],k=学習01[,3]);#入力

###########cmdstanrによる実行
modfileequ01 <- write_stan_file(ANCOVAequ)             #書き出す一時ファイル
modequ01 <- cmdstan_model(modfileequ01)                #コンパイル
csrfitequ01<-modequ01$sample(data=dataSetequ01,chains=5,iter_sampling=20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
extequ01<-as_draws_df(csrfitequ01$draws(par))
(colnames(extequ01) <- gsub("\\[|\\]|,", "", colnames(extequ01)))

#save(extequ01,  file="./scrT/obje/stan_obje0701")
#load(file="./scrT/obje/stan_obje0701"); #予めstan( )で作ったobjectをロード

##### 表7.2 「データ1」の傾きが共通したモデルの母数の事後分布の要約
gqcal(extequ01$a1,2)
gqcal(extequ01$a2,2)
gqcal(extequ01$b    ,2)
gqcal(extequ01$sigma,2)

##### 図7.2 「データ1」の回帰直線(傾き共通)
plot2gequ(y=学習01$post,x=学習01$pre,
          a=学習01$subject,ext=extequ01,ba=3);
#dev.copy2eps(file="z0702.eps",family='Japan1')

a_sa01<-extequ01$a1-extequ01$a2;                #切片の差の事後分布
gqcal(a_sa01,1);                                      #p.96,l.6

##### 図7.3 「データ1」の切片の差の事後分布
hist(a_sa01,breaks=100,ylab="",xlab="切片の差",main="",cex.lab=2,cex.axis=1.5)
#dev.copy2eps(file="z0703.eps",family='Japan1')


####### 7.2.2 「データ2」の分析
##### 表7.3 「データ2」の平均値
学習02<-read.csv("./scrT/dat/学習02.csv", header = TRUE)
round(tapply(学習02$post,学習02$subject,mean),1)
round(tapply(学習02$pre,学習02$subject,mean),1)

##### 図7.4 「データ2」の散布図
plot2g(y=学習02$post,x=学習02$pre,a=学習02$subject,ba=3)
#dev.copy2eps(file="z0704.eps",family='Japan1')

par<-c("a","b","sigma");                                                  #母数
dataSetequ02<-list(n=nrow(学習02),y=学習02[,2],x=学習02[,1],k=学習02[,3]);#入力

###########cmdstanrによる実行
### コンパイル結果は (傾きが共通のモデル)'modequ01'を利用する
csrfitequ02<-modequ01$sample(data=dataSetequ02,chains=5,iter_sampling=20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
extequ02<-as_draws_df(csrfitequ02$draws(par))
(colnames(extequ02) <- gsub("\\[|\\]|,", "", colnames(extequ02)))

#save(extequ02,  file="./scrT/obje/stan_obje0702")
#load(file="./scrT/obje/stan_obje0702"); #予めstan( )で作ったobjectをロード


##### 表7.4 「データ2」の傾きが共通したモデルの母数の事後分布の要約
gqcal(extequ02$a1,2)
gqcal(extequ02$a2,2)
gqcal(extequ02$b    ,2)
gqcal(extequ02$sigma,2)

##### 図7.5 「データ2」の回帰直線(傾き共通)
plot2gequ(y=学習02$post,x=学習02$pre,a=学習02$subject,ext=extequ02,ba=3)
#dev.copy2eps(file="z0705.eps",family='Japan1')

a_sa02<-extequ02$a1-extequ02$a2;                   #切片の差の事後分布
gqcal(a_sa02,1);                                         #p.97,bl.7

##### 図7.6 「データ2」の切片の差の事後分布
hist(a_sa02,breaks=100,ylab="",xlab="切片の差",   
     main="",cex.lab=2,cex.axis=1.5)
#dev.copy2eps(file="z0706.eps",family='Japan1')

mean(abs(a_sa02)<1.0);                                #ROPEであるというphc
mean(abs(a_sa02)<1.5);                                #p.97,bl.4

########## 7. 3 傾きが異なる共分散分析

###### 7.3.1 「データ3」の分析
###### 表7.5 「データ3」の平均値
学習03<-read.csv("./scrT/dat/学習03.csv", header = TRUE)
round(tapply(学習03$post,学習01$subject,mean),1)
round(tapply(学習03$pre ,学習01$subject,mean),1)

###### 図7.7 「データ3」の散布図
plot2g(y=学習03$post,x=学習03$pre,a=学習02$subject,ba=3);#図7.7
#dev.copy2eps(file="z0707.eps",family='Japan1')

#############################共分散分析(傾きが異なるモデル)のstanコード
ANCOVAdef<-'
//共分散分析(傾きが異なるモデル)
data {
  int<lower=0> n;                   //データ数 
  vector[n] y;                      //基準変数   
  vector[n] x;                      //予測変数   
  array[n] int<lower=0, upper=2> k; //分類変数 
}
parameters {
  vector[2] a;                      //切片
  vector[2] b;                      //回帰係数 
  real<lower=0> sigma;              //誤差標準偏差
}
transformed parameters {
  vector[n] yhat;                   //予測値 yhat  
  for (i in 1 : n) {
    yhat[i] = a[k[i]] + x[i] * b[k[i]];
  }                                 //(7.7)式,(7.8)式
}
model {
  y ~ normal(yhat, sigma);          //(7.9)式
}
';

par<-c("a","b","sigma")                       
dataSetdef03 <-list(n=nrow(学習03),y=学習03[,2],x=学習03[,1],k=学習03[,3])

###########cmdstanrによる実行
modfiledef03 <- write_stan_file(ANCOVAdef)             #書き出す一時ファイル
moddef03 <- cmdstan_model(modfiledef03)                #コンパイル
csrfitdef03<-moddef03$sample(data=dataSetdef03,chains = 5,iter_sampling=20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
extdef03<-as_draws_df(csrfitdef03$draws(par))
(colnames(extdef03) <- gsub("\\[|\\]|,", "", colnames(extdef03)))

#save(extdef03,  file="./scrT/obje/stan_obje0703")
#load(file="./scrT/obje/stan_obje0703"); #予めstan( )で作ったobjectをロード

###### 表7.6 「データ3」の傾きが異なるモデルによる母数の事後分布の要約
gqcal(extdef03$a1,2)
gqcal(extdef03$a2,2)
gqcal(extdef03$b1,2)
gqcal(extdef03$b2,2)
gqcal(extdef03$sigma,2)

###### 図7.8 「データ3」の回帰直線(傾き異なる)
plot2gdef(y=学習03$post,x=学習03$pre,       
          a=学習03$subject,ext=extdef03,ba=3)
#dev.copy2eps(file="z0708.eps",family='Japan1')

######  図7.9 「データ3」の予測値の差の点推定値と95%予測区間
plot2gdefdef(x=学習03[,1],extdef03);                  #図7.9
#dev.copy2eps(file="z0709.eps",family='Japan1')

########7. 3. 2 「データ1」と「データ2」の再分析

#「データ1」の傾きが異なるモデルでの再分析
###########cmdstanrによる実行
### コンパイル結果は(傾きが異なるモデル)'moddef03'データは'dataSetequ01'
csrfitdef01<-moddef03$sample(data=dataSetequ01,chains=5,iter_sampling=20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
extdef01<-as_draws_df(csrfitdef01$draws(par))
(colnames(extdef01) <- gsub("\\[|\\]|,", "", colnames(extdef01)))

#save(extdef01,  file="./scrT/obje/stan_obje0704")
#load(file="./scrT/obje/stan_obje0704"); #予めstan( )で作ったobjectをロード

#### 「データ1」の傾きが異なるモデルによる母数の事後分布の要約(教科書にない)
gqcal(extdef01$a1,2)
gqcal(extdef01$a2,2)
gqcal(extdef01$b1,2)
gqcal(extdef01$b2,2)
gqcal(extdef01$sigma,2)


###### 図7.10 「データ1」の回帰直線(傾き異なる)
plot2gdef(y=学習01$post,x=学習01$pre,a=学習01$subject,ext=extdef01)
#dev.copy2eps(file="z0710.eps",family='Japan1')

###### 図7.11 「データ1」の予測値の差の点推定値と95%予測区間
plot2gdefdef(x=学習01[,1],extdef01); abline(h=4.8)
#dev.copy2eps(file="z0711.eps",family='Japan1')


#「データ2」の傾きが異なるモデルでの再分析
###########cmdstanrによる実行
### コンパイル結果は(傾きが異なるモデル)'moddef03'データは'dataSetequ02'
csrfitdef02<-moddef03$sample(data=dataSetequ02,chains=5,iter_sampling=20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
extdef02<-as_draws_df(csrfitdef02$draws(par))
(colnames(extdef02) <- gsub("\\[|\\]|,", "", colnames(extdef02)))

#save(extdef02,  file="./scrT/obje/stan_obje0705")
#load(file="./scrT/obje/stan_obje0705"); #予めstan( )で作ったobjectをロード


###### 図7.12 「データ2」の回帰直線(傾き異なる)
plot2gdef(y=学習02$post,x=学習02$pre,a=学習02$subject,ext=extdef02)
#dev.copy2eps(file="z0712.eps",family='Japan1')

###### 図7.13 「データ2」の予測値の差の点推定値と95%予測区間
plot2gdefdef(x=学習02[,1],extdef02)
#dev.copy2eps(file="z0713.eps",family='Japan1')

#自習1.  図7.12によって、「データ2」の傾きが異なるモデルによる2本の回帰直線は
#ほぼ同じであることが目視で分かった。念のために、表7.6に相当する
#事後分布の要約を示しなさい。


########7. 3. 3 傾きの差の事後分布
###### 表7.7 傾きの差の事後分布の要約
b_sa01<-extdef01$b1-extdef01$b2
gqcal(b_sa01)
b_sa02<-extdef02$b1-extdef02$b2
gqcal(b_sa02)
b_sa03<-extdef03$b1-extdef03$b2
gqcal(b_sa03)


#自習2.   7.3.3傾きの差の事後分布における表7.7において、
#傾きの差の事後分布の要約が示された。
#分布と0との位置関係を目視で確認するために、
#それら3つのヒストグラムを描きなさい。


###### 7.4  傾向スコア
###### 7.4.2  データの説明
###### 表7.8 学習塾の既存のクラスを利用した新教授法の効果の検証実験データ
学習04<-read.csv("./scrT/dat/propen.csv", header = TRUE)
head(学習04)
tail(学習04)

###### 変数ごとの度数（群 p.103, bl.4 以外は、教科書にはない）
table(学習04$群)
table(学習04$高校)
table(学習04$収入)
table(学習04$母教育)
table(学習04$父教育)

###### 表7.9 実験群と対照群の平均値
round(tapply(学習04$成績,学習04$群,mean),1)
round(tapply(学習04$高校,学習04$群,mean),1)
round(tapply(学習04$収入,学習04$群,mean),1)
round(tapply(学習04$母教育,学習04$群,mean),1)
round(tapply(学習04$父教育,学習04$群,mean),1)

#################################傾向スコアによる分析のstanコード
propensity01<-'
data {
  int<lower=0> n;                         //データ数 
  int<lower=0> p;                         //予測変数の 
  vector[n] y;                            //基準変数   
  matrix[n, p] x;                         //予測変数   
  array[n] int<lower=1, upper=2> k;       //分類変数 1実験群２対照群
}
transformed data {
  array[n] int<lower=0, upper=1> kk;      //分類変数 
  for (i in 1 : n) {
    kk[i] = k[i] * (-1) + 2;
  }
}
parameters {
  vector[2] a;                            //切片
  real b;                                 //回帰係数 
  real alpha;                             //ロジスティック回帰切片
  vector[p] beta;                         //ロジスティック回帰係数
  real<lower=0> sigma;                    //誤差標準偏差
}
transformed parameters {
  vector[n] yhat;                         //yhat  
  vector<lower=0, upper=1>[n] propensity; //傾向スコア
  for (i in 1 : n) {
    propensity[i] = inv_logit(alpha + x[i,  : ] * beta);
    yhat[i] = a[k[i]] + propensity[i] * b;
  }
}
model {
  for (i in 1 : n) {
    kk[i] ~ bernoulli(propensity[i]);      //ベルヌイ分布モデル
    y[i] ~ normal(yhat[i], sigma);         //正規分布モデル
  }
}
generated quantities {
  
}
';

par<-c("a","b","sigma","alpha","beta","propensity")    #母数
dataSetpropen01 <-list(n=nrow(学習04),p=4,y=学習04[,2],
                       x=学習04[,c(3:6)],k=学習04[,1]) #入力

###########cmdstanrによる実行
modfilepropen01 <- write_stan_file(propensity01)    #書き出す一時ファイル
modpropen01 <- cmdstan_model(modfilepropen01)       #コンパイル
csrfitpropen01 <- modpropen01$sample(data = dataSetpropen01,chains = 5,
   iter_sampling=20000,iter_warmup=1000,parallel_chains= 5,seed=1234)  #MCMC
extpropen01<-as_draws_df(csrfitpropen01$draws(par))
(colnames(extpropen01) <- gsub("\\[|\\]|,", "", colnames(extpropen01)))

#save(extpropen01,  file="./scrT/obje/stan_obje0706")
#load(file="./scrT/obje/stan_obje0706"); #予めstan( )で作ったobjectをロード


###### 表7.10 傾向スコアを共変量とした共分散分析の母数の事後分布の要約
gqcal(extpropen01[,1:13],2)


###### 図7.14 群別の傾向スコアのヒストグラム
propen01<-apply(extpropen01[,10:159],2,mean);       #傾向スコアのEAP
階級<-seq(0,1,0.1)
barplot(table(学習04$群,cut(propen01,階級,include.lowest = T,right = F)),
   xlab="propensity score", axisnames = F,cex.lab=2.0,cex.axis=1.5)
#dev.copy2eps(file="z0714.eps",family='Japan1')

###### 図7.15 ROC 曲線
(ROC1<-roc(学習04$群 ~ propen01,ci=T))
plot(ROC1,cex.lab=2.0,cex.axis=1.5)
#dev.copy2eps(file="z0715.eps",family='Japan1')

###### 図7.16 傾向スコアを共変量とした共分散分析
plot2gequ(y=学習04$成績,x=propen01,a=学習04$群,
          ext=extpropen01,ba=0,xlab="propensity score")
#dev.copy2eps(file="z0716.eps",family='Japan1')

###### 図7.17 切片の差の事後分布
a_sa04<-extpropen01$a1-extpropen01$a2
gqcal(a_sa04,1);
hist(a_sa04,br=100,cex.lab=2.0,cex.axis=1.5,xlab="a1 ー a2",ylab="",main="")
#dev.copy2eps(file="z0717.eps",family='Japan1')

#######「切片の差がROPE(1.0)である」のphc  p,108,l.3
mean(abs(a_sa04)<1)

#以下実習はしない
set.seed(1234)
match<-Match(Tr= 学習04$群*-1+2, X=propen01, M=1, caliper=0.2, replace=F)
match
