﻿############第8章　さらに進んだ実験計画
# Working directoryが 'toyoda' であることの確認
(n_wd<-getwd())                #working directory(wd)の確認
(setwd('./scrT'))              #wdの移動
source('../myfunc/myfunc.R')   #自作関数の読み込み
(setwd(n_wd))                  #wd戻す
library(cmdstanr)              #パッケージcmdstanrの呼び出し
library(posterior)             #パッケージposteriorの呼び出し
prob<-c(0.025, 0.05, 0.5, 0.95, 0.975) #確率点の定義

###################8. 1 対応のある1 要因の推測

###### 8. 1. 1 触2 点閾の感覚実験

###### 表8.1 触2 点閾の感覚実験データ
(触２点閾<-read.csv("./scrT/dat/触２点閾.csv", header = TRUE))

###### 図8.1 触2 点閾の測定値(部位ごとmm)
boxplot(触２点閾$長さ~触２点閾$bui,xlab="部位",ylab="",cex.axis=2,cex.lab=2)
#dev.copy2eps(file="z0601.eps",family='Japan1')

###### 部位ごとの触2 点閾の測定値の平均・sd    p.111,bl.2
tapply(触２点閾$長さ,触２点閾$部位,mean)
tapply(触２点閾$長さ,触２点閾$部位,sd)

###### 図8.2 触2 点閾の測定値(被験者ごとmm)
boxplot(触２点閾$長さ~触２点閾$被験者,xlab="被験者",cex.axis=1.4,cex.lab=1.5)
#dev.copy2eps(file="z0602.eps",family='Japan1')

###### 被験者ごとの触2 点閾の測定値の平均・sd  p.112,下部、具体的な数値はなし
tapply(触２点閾$長さ,触２点閾$被験者,mean)
tapply(触２点閾$長さ,触２点閾$被験者,sd)

#######################対応のある1要因の推測のstanコード
RBlockD<-'
data {
  int<lower=0> n;                //全データ数 
  int<lower=0> J;                //群数
  int<lower=0> K;                //ブロック数
  vector[n] y;                   //基準変数   
  array[n] int<lower=0> j;       //分類変数A 
  array[n] int<lower=0> k;       //ブロック変数B 
}
parameters {
  vector[J] mu;                  //各群の効果
  vector[K] s;                   //ブロックの効果
  real<lower=0> s_e;             //誤差SD
  real<lower=0> s_s;             //ブロックSD
}
model {
  y ~ normal(mu[j] + s[k], s_e); //(8.1)式,(8.2)式
  s ~ normal(0, s_s);            //(8.3)式
}
generated quantities {
  real tmu;                      //全平均
  real s_mu;                     //要因SD
  tmu = mean(mu);                //μjの平均
  s_mu = sqrt(variance(mu) * 
        (J - 1) / J);            //μjのｓｄ
}
';
par<-c("mu","s_e","s_s","tmu","s_mu","s");                      #母数
dataSetran <-list(n=length(触２点閾$長さ),J=max(触２点閾$部位), #入力
  K=max(触２点閾$被験者),y=触２点閾$長さ,j=触２点閾$部位,k=触２点閾$被験者)

###########cmdstanrによる実行
modfileran <- write_stan_file(RBlockD)             #書き出す一時ファイル
modran <- cmdstan_model(modfileran)                   #コンパイル
csrfitran <- modran$sample(data = dataSetran,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
extran<-as_draws_df(csrfitran$draws(par))
(colnames(extran) <- gsub("\\[|\\]|,", "", colnames(extran)))

#save(extran,  file="./scrT/obje/stan_obje0801")
#load(file="./scrT/obje/stan_obje0801"); #予めstan( )で作ったobjectをロード

###### 表8.2 乱塊計画の母数の事後分布
eta2 = (extran$s_mu^2)/((extran$s_mu^2)+(extran$s_e^2));#説明率(8.5)式
gqcal(extran$mu1)
gqcal(extran$mu2)
gqcal(extran$mu3)
print('------------------------------------------')
gqcal(extran$s_e   )
gqcal(extran$s_s   )
print('------------------------------------------')
gqcal(extran$tmu   )
gqcal(extran$s_mu  )
print('------------------------------------------')
gqcal(extran$s4 )
print('------------------------------------------')
gqcal(eta2)

###### 群間に差がある確率（意味があるための必要条件の確認）p.114,bl.2
mean(extran$mu1>extran$mu2)
mean(extran$mu2>extran$mu3)
mean(extran$mu1>extran$mu3)

###### 図8.3 平均値の差に関するphc 曲線
par(mfrow=c(2,1));                                    #図8.3
phc01(seq01=seq(0,1.2,0.05),a=extran$mu1,b=extran$mu2,
  xlab="c<mu1-mu2")
phc01(seq01=seq(0,1.2,0.05),a=extran$mu2,b=extran$mu3,
  xlab="c<mu2-mu3")
par(mfrow=c(1,1))
#dev.copy2eps(file="z0603.eps",family='Japan1')

###### 実質的に差があると明言できる確率 p115,l.3
phc01(0.7,a=extran$mu1,b=extran$mu2,byoga="no")
phc01(0.5,a=extran$mu2,b=extran$mu3,byoga="no")


#############8. 2 ネストした1 要因の推測
####### 8. 2. 1 迷路課題におけるネズミの潜在学習

####### 表8.3 迷路課題におけるネズミの潜在学習のデータ
(潜在学習<-read.csv("./scrT/dat/潜在学習.csv", header = TRUE))

###### 図8.4 迷路課題におけるネズミの潜在学習(条件ごと)
boxplot(潜在学習$time~潜在学習$jyok,xlab="条件",cex.lab=1.2,cex.axis=1.8)
#dev.copy2eps(file="z0604.eps",family='Japan1')

###### 図8.5 迷路課題におけるネズミの潜在学習(ネズミごと)
boxplot(潜在学習$time~潜在学習$ネズミ, xlab="ネズミ", ylab="")
#dev.copy2eps(file="z0605.eps",family='Japan1')

###### 条件ごとの平均値とsd （教科書には具体的数値なし）
tapply(潜在学習$time,潜在学習$条件,mean)
tapply(潜在学習$time,潜在学習$条件,sd)

############################ネストした1要因の推測のstanコード
nestedD<-'
data {
  int<lower=0> n;               //全データ数 
  int<lower=0> J;               //群数
  int<lower=0> K;               //ブロック数
  vector[n] y;                  //基準変数   
  array[n] int<lower=0> j;      //分類変数A 
  array[n] int<lower=0> k;      //ブロック変数B 
}
parameters {
  vector[J] mu;                 //各群の効果
  vector[J * K] s;              //ブロックの効果
  real<lower=0> s_e;            //誤差SD
  real<lower=0> s_s;            //ブロックSD
}
model {
  y ~ normal(mu[j] + s[k], s_e);//(8.6)式,(8.7)式
  s ~ normal(0, s_s);           //(8.8)式
}
';

par<-c("mu","s_e","s_s")                                          #母数
dataSetnest <-list(n=length(潜在学習$time), J=max(潜在学習$条件), #入力
 K=max(潜在学習$ネズミ),y=潜在学習$time,j=潜在学習$条件,k=潜在学習$ネズミ)

###########cmdstanrによる実行
modfilenest <- write_stan_file(nestedD)             #書き出す一時ファイル
modnest <- cmdstan_model(modfilenest)               #コンパイル
csrfitnest <- modnest$sample(data=dataSetnest,chains=5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
extnest<-as_draws_df(csrfitnest$draws(par))
(colnames(extnest) <- gsub("\\[|\\]|,", "", colnames(extnest)))

#save(extnest,  file="./scrT/obje/stan_obje0802")
#load(file="./scrT/obje/stan_obje0802"); #予めstan( )で作ったobjectをロード

###### 表8.4 枝分かれ計画の母数と生成量の事後分布の要約(前半)
gqcal(extnest$mu1)
gqcal(extnest$mu2)
gqcal(extnest$mu3)
gqcal(extnest$s_e   )
gqcal(extnest$s_s   )

###### 表8.4（後半）平均値の差の事後分布
def32 = extnest$mu3-extnest$mu2;    #(8.10)式
def13 = extnest$mu1-extnest$mu3;    #(8.11)式
gqcal(def32);
gqcal(def13);                                         #表8.4ここまで

#自習1.　　8.2節、ネストした実験、p.118, (8.10)式,(8.11)式の群間差の
#ヒストグラムを描きなさい。可能であれば、共通の横軸に描いた平均値の差の２つの
#事後分布のヒストグラムを描きなさい。


#####################8. 3 対応ある2 要因の推測
###### 8. 3. 1 説得的メッセージの比較調査

###### 表8.5 説得的メッセージの比較調査データ
(便所標語<-read.csv("./scrT/dat/便所標語.csv", header = TRUE))

###### 図8.6 セルごとの箱ひげ図
boxplot(便所標語$評定~便所標語$標語+便所標語$katu,xlab="標語/活字",ylab="評定")
#dev.copy2eps(file="z0606.eps",family='Japan1')

###### 図8.7 被験者ごとの箱ひげ図
boxplot(便所標語$評定~便所標語$被験者,xlab="被験者", ylab="評定")
#dev.copy2eps(file="z0607.eps",family='Japan1')

#######################################対応ある2要因の推測のstanコード
RBFD<-'
//乱塊要因計画・被験者内2要因 実習１　２
functions {
  vector zero_sum_vector(int a, vector m1A) {
                                      //主効果ゼロ和
    vector[a] muA;
    for (i in 1 : (a - 1)) {
      muA[i] = m1A[i];
    }                                 //a-1まではソックリ代入
    muA[a] = -sum(m1A);               //a番目に、和の-1倍代入
    return muA;
  }
  matrix zero_sum_matrix(int a, int b, matrix m1AB) {
                                      //交互作用ゼロ和
    vector[a - 1] m1a;                //行和を入れるベクトル
    vector[b - 1] m1b;                //列和を入れるベクトル
    matrix[a, b] muAB;                //リターンする行列
    for (i in 1 : (a - 1)) {
      m1a[i] = 0.0;
    }                                 //ゼロクリア
    for (j in 1 : (b - 1)) {
      m1b[j] = 0.0;
    }                                 //ゼロクリア
    for (i in 1 : (a - 1)) {
      for (j in 1 : (b - 1)) {
        muAB[i, j] = m1AB[i, j];
      }
    }                                 //ソックリ代入
    for (i in 1 : (a - 1)) {
      for (j in 1 : (b - 1)) {
        m1a[i] = m1a[i] + m1AB[i, j];
      }
    }                                 //行和作る
    for (j in 1 : (b - 1)) {
      for (i in 1 : (a - 1)) {
        m1b[j] = m1b[j] + m1AB[i, j];
      }
    }                                 //列和作る
    for (i in 1 : (a - 1)) {
      muAB[i, b] = (-1) * m1a[i];
    }                                 //第b列に行和の-1倍代入
    for (j in 1 : (b - 1)) {
      muAB[a, j] = (-1) * m1b[j];
    }                                 //第a行に列和の-1倍代入
    muAB[a, b] = sum(m1a); //第a行第b列要素に合計代入、注意：マイナスではない
    return muAB;
  }
}
data {
  int<lower=0> n;                     //全データ数 
  int<lower=0> a;                     //A水準数
  int<lower=0> b;                     //B水準数
  int<lower=0> s;                     //ブロック数
  vector[n] y;                        //基準変数   
  array[n] int<lower=0> A;            //分類変数A 
  array[n] int<lower=0> B;            //分類変数B 
  array[n] int<lower=0> S;            //ブロック変数s 
}
parameters {
  real mu;                            //全平均
  vector[a - 1] m1A;                  //A平均1つ少ない
  vector[b - 1] m1B;                  //B平均1つ少ない
  vector[s - 1] m1S;                  //S平均1つ少ない
  matrix[a - 1, b - 1] m1AB;          //交互作用1つ少ない
  matrix[a - 1, s - 1] m1AS;          //交互作用AS1つ少ない
  matrix[b - 1, s - 1] m1BS;          //交互作用BS1つ少ない
  real<lower=0> s_E;                  //E標準偏差
  real<lower=0> s_S;                  //S標準偏差
  real<lower=0> s_AS;                 //AS標準偏差
  real<lower=0> s_BS;                 //BS標準偏差
}
transformed parameters {
  vector[a] muA;                      //A平均
  vector[b] muB;                      //B平均
  matrix[a, b] muAB;                  //交互作用
  muA = zero_sum_vector(a, m1A);      //p.121(8.12)式2行下、1つ目
  muB = zero_sum_vector(b, m1B);      //2つ目
  muAB = zero_sum_matrix(a, b, m1AB); //3，4つ目
}
model {
  vector[s] muS;                      //S平均
  matrix[a, s] muAS;                  //交互作用AS
  matrix[b, s] muBS;                  //交互作用BS
  m1S ~ normal(0, s_S);               //p.121(8.12)式1行下、2つ目
  muS = zero_sum_vector(s, m1S);      //p.125(8.13)式2行下、1つ目
  for (j in 1 : a - 1) {
    for (l in 1 : s - 1) {
      m1AS[j, l] ~ normal(0, s_AS);
    }
  }                                   //  3つ目
  for (k in 1 : b - 1) {
    for (l in 1 : s - 1) {
      m1BS[k, l] ~ normal(0, s_BS);
    }
  }                                   //  4つ目
  muAS = zero_sum_matrix(a, s, m1AS); //asの和を0にする
  muBS = zero_sum_matrix(b, s, m1BS); //bsの和を0にする
  for (i in 1 : n) {
    y[i] ~ normal(mu + muA[A[i]] + muB[B[i]] + muS[S[i]] + muAB[A[i], B[i]]
                  + muAS[A[i], S[i]] + muBS[B[i], S[i]], s_E);
  }                                   //(8.12)式
}
generated quantities {
  real s_a;                           //要因AのSD
  real s_b;                           //要因BのSD
  real s_ab;                          //交互作用ABのSD
  s_a = sqrt(variance(muA) * (a - 1) / a); //要因Aのsd
  s_b = sqrt(variance(muB) * (b - 1) / b); //要因Bのsd
  s_ab = sqrt(variance(muAB) * ((a * b) - 1) / (a * b)); //交互作用ABのsd
}
';

par<-c("mu","muA","muB","muAB","s_S","s_AS",
       "s_BS","s_E","s_a","s_b","s_ab")                           #母数
dataSetRBFD <-list(n=length(便所標語$評定), a=max(便所標語$標語),
   b=max(便所標語$活字), s=max(便所標語$被験者), y=便所標語$評定,
   A=便所標語$標語, B=便所標語$活字, S=便所標語$被験者)           #入力

###########cmdstanrによる実行
modfileRBFD <- write_stan_file(RBFD)             #書き出す一時ファイル
modRBFD <- cmdstan_model(modfileRBFD)            #コンパイル
csrfitRBFD <- modRBFD$sample(data=dataSetRBFD,chains=5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
extRBFD<-as_draws_df(csrfitRBFD$draws(par))
(colnames(extRBFD) <- gsub("\\[|\\]|,", "", colnames(extRBFD)))

#save(extRBFD,  file="./scrT/obje/stan_obje0803")
#load(file="./scrT/obje/stan_obje0803"); #予めstan( )で作ったobjectをロード

###### 表8.6 乱塊要因計画の母数の事後分布
gqcal(extRBFD$mu       )
gqcal(extRBFD$muA1   )
gqcal(extRBFD$muA2   )
gqcal(extRBFD$muA3   )
gqcal(extRBFD$muB1   )
gqcal(extRBFD$muAB11)
gqcal(extRBFD$muAB21)
gqcal(extRBFD$muAB31)
gqcal(extRBFD$s_S      )
gqcal(extRBFD$s_AS     )
gqcal(extRBFD$s_BS     )
gqcal(extRBFD$s_E      )
print('------------------------------------------')
gqcal(extRBFD$s_a      )
gqcal(extRBFD$s_b      )
gqcal(extRBFD$s_ab     )


###### 8. 3. 4 PHC  「種類」の水準間に差がある確率
phc02(0,extRBFD[,2:4])

#「フォント」の水準間に差がある確率
mean(extRBFD$muB2-extRBFD$muB1>0)

#連言命題が正しい確率 「罰」＜「直截」　かつ　「直截」＜「公共心」
round(mean((extRBFD$muA1 < extRBFD$muA2) & (extRBFD$muA2 < extRBFD$muA3)),3)

###### 図8.8 水準の効果にc 以上の差のある確率の曲線
par(mfrow=c(3,1))
phc01(seq01=seq(0.5,1.4,0.05),a=extRBFD$muA2,b=extRBFD$muA1,
        xlab="c<a2-a1",cex.lab=2.5)
phc01(seq01=seq(1,2,0.05),a=extRBFD$muA3,b=extRBFD$muA2,
        xlab="c<a3-a2",cex.lab=2.5)
phc01(seq01=seq(1,2,0.05),a=extRBFD$muB2,b=extRBFD$muB1,
        xlab="c<b2-b1",cex.lab=2.5)
par(mfrow=c(1,1))
#dev.copy2eps(file="z0608.eps",family='Japan1')

###### 実質的に差があると明言できる確率 p.123,l.3　以降このページの最後まで
phc01(seq01=0.8,a=extRBFD$muA2,b=extRBFD$muA1,byoga="no")
phc01(seq01=1.3,a=extRBFD$muA3,b=extRBFD$muA2,byoga="no")
phc01(seq01=1.5,a=extRBFD$muB2,b=extRBFD$muB1,byoga="no")


##################8. 4 混合した2 要因の推測
###### 8. 4. 1 健康診断の追跡調査

###### 表8.7 健康診断の追跡調査
(うつ追跡<-read.csv("./scrT/dat/うつ追跡.csv", header = TRUE))

###### 図8.9 うつ病検査の追跡調査 乱数を加え散布図を見やすくしている
ma<-57;mi<-48
a1<-うつ追跡[  1: 50,1]+runif(50)
a2<-うつ追跡[ 51:100,1]+runif(50)
b1<-うつ追跡[101:150,1]+runif(50)
b2<-うつ追跡[151:200,1]+runif(50)
plot(a1,a2,pch=21,xlim=c(mi,ma),ylim=c(mi,ma),xlab="pre.test",ylab="post.test")
abline(0,1)
par(new=T)
plot(b1,b2,pch=19,xlim=c(mi,ma),ylim=c(mi,ma),xlab="",ylab="")
legend(49,56,c("対照群","介入群"),pch=c(21,19));               #図8.9ここまで


#####################混合した2要因の推測のstanコード
SPD<-'
functions {
  vector zero_sum_vector(int a, vector m1A) {
                                             //主効果ゼロ和
    vector[a] muA;
    for (i in 1 : (a - 1)) {
      muA[i] = m1A[i];
    }                                        //a-1まではソックリ代入
    muA[a] = -sum(m1A);                      //a番目に、和の-1倍代入
    return muA;
  }
  matrix zero_sum_matrix(int a, int b, matrix m1AB) {
                                             //交互作用ゼロ和
    vector[a - 1] m1a;                       //行和を入れるベクトル
    vector[b - 1] m1b;                       //列和を入れるベクトル
    matrix[a, b] muAB;                       //リターンする行列
    for (i in 1 : (a - 1)) {
      m1a[i] = 0.0;
    }                                        //ゼロクリア
    for (j in 1 : (b - 1)) {
      m1b[j] = 0.0;
    }                                        //ゼロクリア
    for (i in 1 : (a - 1)) {
      for (j in 1 : (b - 1)) {
        muAB[i, j] = m1AB[i, j];
      }
    }                                        //ソックリ代入
    for (i in 1 : (a - 1)) {
      for (j in 1 : (b - 1)) {
        m1a[i] = m1a[i] + m1AB[i, j];
      }
    }                                        //行和作る
    for (j in 1 : (b - 1)) {
      for (i in 1 : (a - 1)) {
        m1b[j] = m1b[j] + m1AB[i, j];
      }
    }                                        //列和作る
    for (i in 1 : (a - 1)) {
      muAB[i, b] = (-1) * m1a[i];
    }                                        //第b列に行和の-1倍代入
    for (j in 1 : (b - 1)) {
      muAB[a, j] = (-1) * m1b[j];
    }                                        //第a行に列和の-1倍代入
    muAB[a, b] = sum(m1a); //第a行第b列要素に合計代入、注意：マイナスではない
    return muAB;
  }
}
data {
  int<lower=0> n;                            //全データ数 
  int<lower=0> a;                            //A水準数（標本間要因）
  int<lower=0> b;                            //B水準数（標本内要因）
  int<lower=0> s;                            //ブロック数
  vector[n] y;                               //基準変数   
  array[n] int<lower=0> A;                   //分類変数A（標本間要因）
  array[n] int<lower=0> B;                   //分類変数B（標本内要因）
  array[n] int<lower=0> S;                   //ブロック変数s 
}
parameters {
  real mu;                                   //全平均
  vector[a - 1] m1A;                         //A平均1つ少ない
  vector[b - 1] m1B;                         //B平均1つ少ない
  vector[s * a - 1] m1S;                     //S平均1つ少ない
  matrix[a - 1, b - 1] m1AB;                 //交互作用1つ少ない
  real<lower=0> s_E;                         //E標準偏差
  real<lower=0> s_S;                         //S標準偏差
}
transformed parameters {
  vector[a] muA;                             //A平均
  vector[b] muB;                             //B平均
  matrix[a, b] muAB;                         //交互作用
  muA = zero_sum_vector(a, m1A);             //p.125(8.13)式2行下、1つ目
  muB = zero_sum_vector(b, m1B);             //2つ目
  muAB = zero_sum_matrix(a, b, m1AB);        //3，4つ目
}
model {
  vector[s * a] muS;                         //S平均
  m1S ~ normal(0, s_S);                      //p.125(8.13)式1行下、2つ目
  muS = zero_sum_vector(s * a, m1S);         //p.125(8.13)式2行下、1つ目
  for (i in 1 : n) {
    y[i] ~ normal(mu + muA[A[i]] + muB[B[i]] + muS[S[i]] + muAB[A[i], B[i]],
                  s_E);
  }                                          //(8.13)式
}
generated quantities {
  real s_a;                                  //要因AのSD
  real s_b;                                  //要因BのSD
  real s_ab;                                 //交互作用ABのSD
  s_a = sqrt(variance(muA) * (a - 1) / a);   //要因Aのsd
  s_b = sqrt(variance(muB) * (b - 1) / b);   //要因Bのsd
  s_ab = sqrt(variance(muAB) * ((a * b) - 1) / (a * b)); //交互作用ABのsd
}
';
par<-c("mu","muA","muB","muAB","s_S","s_E","s_a","s_b","s_ab")    #母数
dataSetSPD <-list(n=length(うつ追跡$score),a=max(うつ追跡$群),
    b=max(うつ追跡$時期),s=max(うつ追跡$学生),y=うつ追跡$score,
    A=うつ追跡$群, B=うつ追跡$時期, S=うつ追跡$学生)              #入力

###########cmdstanrによる実行
modfileSPD <- write_stan_file(SPD)             #書き出す一時ファイル
modSPD <- cmdstan_model(modfileSPD)                   #コンパイル
csrfitSPD <- modSPD$sample(data = dataSetSPD,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
extSPD<-as_draws_df(csrfitSPD$draws(par))
(colnames(extSPD) <- gsub("\\[|\\]|,", "", colnames(extSPD)))

#save(extSPD,  file="./scrT/obje/stan_obje0804")
#load(file="./scrT/obje/stan_obje0804"); #予めstan( )で作ったobjectをロード

######表8.8 混合計画の母数の事後分布の要約
gqcal(extSPD$mu        )
gqcal(extSPD$muA1   )
gqcal(extSPD$muB1   )
gqcal(extSPD$muAB11)
gqcal(extSPD$muAB12)
gqcal(extSPD$muAB21)
gqcal(extSPD$muAB22)
gqcal(extSPD$s_S       )
gqcal(extSPD$s_E       )
print('------------------------------------------')
gqcal(extSPD$s_a       )
gqcal(extSPD$s_b       )
gqcal(extSPD$s_ab      )


####生成量(j 番目の群のk 番目の時期のセルjk の効果)の事後分布  (8.15)式
ce11<-extSPD$muA1+extSPD$muB1+extSPD$muAB11
ce12<-extSPD$muA1+extSPD$muB2+extSPD$muAB12
ce21<-extSPD$muA2+extSPD$muB1+extSPD$muAB21
ce22<-extSPD$muA2+extSPD$muB2+extSPD$muAB22

###### 表8.9 群内の時期差の事後分布
gqcal(ce11-ce12);             #(8.16)式
gqcal(ce21-ce22);

###### 群内の時期差の事後分布のヒストグラム（教科書にはない）
hist(ce11-ce12,main="",ylab="")
hist(ce21-ce22,main="",ylab="")

#自習2.　　8.4節、混合実験、p.126,表8.9に登場するセル$jk$の効果に関して
#phc($c<ce_{11}-ce_{12}$)と
#ROPEのphc($|ce_{11}-ce_{12}|<c$)とphc($c<ce_{21}-ce_{22}$)
#の曲線とテーブルを作成し、差を考察しなさい。


################################# 実習課題
#1
#def32
phc01(seq01=seq(-3,6,0.05),a=def32,cc="gtc",byoga="yes",xlab="d32",cex.lab=2.5)
phc01(seq01=seq(-3,6,0.5 ),a=def32,cc="gtc",byoga="no")

#def32,ROPE
phc01(seq01=seq(0,6,0.05),a=def32,cc="rope",byoga="yes",xlab="|d32|<c",
      cex.lab=2.5)
phc01(seq01=seq(0,6,0.5 ),a=def32,cc="rope",byoga="no")

#def13
phc01(seq01=seq(17,25,0.05),a=def13,cc="gtc",byoga="yes",xlab="d32",
      cex.lab=2.5)
phc01(seq01=seq(17,25,0.5),a=def13,cc="gtc",byoga="no")

#2

#
#     正解は割愛
#
