﻿############# 第9章　階層線形モデル
# Working directoryが 'toyoda' であることの確認
(n_wd<-getwd())                #working directory(wd)の確認
(setwd('./scrT'))              #wdの移動
source('../myfunc/myfunc.R')   #自作関数の読み込み
(setwd(n_wd))                  #wd戻す
library(cmdstanr)              #パッケージcmdstanrの呼び出し
library(posterior)             #パッケージposteriorの呼び出し
prob<-c(0.025, 0.05, 0.5, 0.95, 0.975) #確率点の定義

###### 9. 1 切片と回帰係数に分布を仮定したモデル
###### 9. 1. 1 知覚された長さに関する実験

###### 表9.1 知覚された長さに関する実験
(知覚長11<-read.csv("./scrT/dat/知覚長11.csv", header = TRUE))

###### 図9.1 知覚された長さの実験(2 名分)
X<-知覚長11[((知覚長11$学生==17)|(知覚長11$学生==1)), ]
plot(X$実測,X$目測,pch=X$学生,xlab="実測",ylab="目測")
  legend(60,160,c("学生1","学生17"),pch=c(1,17),cex=1.5)
#dev.copy2eps(file="z0806.eps",family='Japan1')

###### 図9.2 知覚された長さの実験(25 名分)
plot(知覚長11$実測,知覚長11$目測,pch=知覚長11$学生,   #図9.2
     xlab="実測",ylab="目測")

###############切片と回帰係数に分布を仮定したモデルのstanコード
HLM01<-'
data {
  int<lower=0> n;          //全データ数 
  int<lower=0> J;          //群数
  vector[n] y;             //基準変数   
  vector[n] x;             //予測変数   
  array[n] int<lower=0> k; //分類変数 
}
parameters {
  vector[J] a;             //切片
  vector[J] b;             //回帰係数 
  real<lower=0> sigma;     //誤差標準偏差
  real a0;                 //切片平均 
  real b0;                 //回帰係数平均 
  real<lower=0> s_a;       //切片SD
  real<lower=0> s_b;       //回帰係数SD
}
transformed parameters {
  vector[n] yhat;          //予測値
  for (i in 1 : n) {
    yhat[i] = a[k[i]] + x[i] * b[k[i]];
  }                        //(9.1式)
}
model {
  y ~ normal(yhat, sigma); //(9.2式)
  a ~ normal(a0, s_a);     //(9.3式)
  b ~ normal(b0, s_b);     //(9.4式)
}
';

par<-c("sigma","a0","b0","s_a","s_b","a","b")                    #母数
dataSet81 <-list(n=length(知覚長11$目測), J=max(知覚長11$学生), 
  y=知覚長11$目測, x=知覚長11$実測, k=知覚長11$学生)             #入力データ

###########cmdstanrによる実行
modfile81 <- write_stan_file(HLM01)           #書き出す一時ファイル
mod81 <- cmdstan_model(modfile81)                   #コンパイル
csrfit81 <- mod81$sample(data = dataSet81,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext81<-as_draws_df(csrfit81$draws(par))
(colnames(ext81) <- gsub("\\[|\\]|,", "", colnames(ext81)))

#save(ext81,  file="./scrT/obje/stan_obje0901")
#load(file="./scrT/obje/stan_obje0901"); #予めstan( )で作ったobjectをロード

###### 表9.2 切片と回帰係数が分布をもつモデルの母数の事後分布の要約
gqcal(ext81$sigma)
gqcal(ext81$a0   )
gqcal(ext81$b0   )
gqcal(ext81$s_a  )
gqcal(ext81$s_b  )

##### 25本の切片・回帰係数のEAP推定値(作図のためであり、教科書にはない)
av<-apply(ext81[, 6:30],2,mean)
bv<-apply(ext81[,31:55],2,mean)
round(rbind(av,bv),2)

###### 図9.3 25 本の単回帰直線
plot(知覚長11$実測,知覚長11$目測,pch=知覚長11$学生,
    xlab="実測",ylab="目測",type="n")
    for (i in 1:25) {abline(av[i],bv[i],lwd=1.8)}

##### 図9.4 回帰係数の標準偏差の事後分布
hist(ext81$s_b,breaks=100,main="",ylab="",xlab="回帰係数の標準偏差")

##### 図9.5 切片と回帰係数の散布図
plot(av,bv,pch=16,xlab="切片",ylab="回帰係数")

cor(av,bv);                  #切片と傾きの相関係数 p.131,l.3


###### 9. 2 切片に分布を仮定したモデル
###### 9. 2. 1 高等学校別の中学の成績と高校の偏差値

##### 表9.3 高等学校別の中学の成績と高校の偏差値
(学力変化<-read.csv("./scrT/dat/学力変化.csv", header = TRUE))

##### 図9.6 高校2 と9 の中・高校時の偏差値
X<-学力変化[((学力変化$学校==2)|(学力変化$学校==9)), ]
plot(X$中学,X$高校,pch=X$学校,xlab="中学３年偏差値",
   ylab="高校１年偏差値", xlim=c(25,75),ylim=c(25,75))
legend(30,70,c("高校２","高校９"),pch=c(2,9))

##### 図9.7   10校の中学時と高校時の偏差値
plot(学力変化$中学,学力変化$高校,pch=学力変化$学校,xlab="中学３年偏差値",
      ylab="高校１年偏差値", xlim=c(25,75),ylim=c(25,75))

###### まずは切片と回帰係数に分布を仮定したモデル
par<-c("sigma","a0","b0","s_a","s_b","a","b")                      #母数
dataSet821 <-list(n=length(学力変化$高校),J=max(学力変化$学校),
                  y=学力変化$高校,x=学力変化$中学, k=学力変化$学校)#入力

###########cmdstanrによる実行
#コンパイル結果は 'mod81' を利用する
csrfit821 <- mod81$sample(data = dataSet821,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext821<-as_draws_df(csrfit821$draws(par))
(colnames(ext821) <- gsub("\\[|\\]|,", "", colnames(ext821)))

#save(ext821,  file="./scrT/obje/stan_obje0902")
#load(file="./scrT/obje/stan_obje0902"); #予めstan( )で作ったobjectをロード

###### 表9.4 切片と回帰係数に分布のあるモデルの母数
###### の事後分布の要約(偏差値データ)
gqcal(ext821$sigma)
gqcal(ext821$a0   )
gqcal(ext821$b0   )
gqcal(ext821$s_a  )
gqcal(ext821$s_b  )

##### 切片・回帰係数のEAP推定値(作図のためであり、教科書にはない)
av<-apply(ext821[, 6:15],2,mean)
bv<-apply(ext821[,16:25],2,mean)
round(rbind(av,bv),2)

##### 図9.8 高校別の10 本の単回帰直線
plot(学力変化$中学,学力変化$高校,pch=学力変化$学校,xlab="中学３年偏差値",
     ylab="高校１年偏差値", xlim=c(25,75),ylim=c(25,75),type="n")
     for (i in 1:10) {abline(av[i],bv[i],lwd=1.8)}

##### 図9.9 回帰係数の標準偏差の事後分布
hist(ext821$s_b,breaks=100,main="",ylab="",xlab="回帰係数の標準偏差")

###################切片に分布を仮定したモデルのstanコード
HLM02<-'
data {
  int<lower=0> n;          //全データ数 
  int<lower=0> J;          //群数
  vector[n] y;             //基準変数   
  vector[n] x;             //予測変数   
  array[n] int<lower=0> k; //分類変数 
}
parameters {
  vector[J] a;             //切片
  real b;                  //回帰係数 
  real<lower=0> sigma;     //誤差標準偏差
  real a0;                 //切片平均 
  real<lower=0> s_a;       //切片SD
}
transformed parameters {
  vector[n] yhat;          //予測値
  for (i in 1 : n) {
    yhat[i] = a[k[i]] + x[i] * b;
  }                        //(9.5式),(9.8式)
}
model {
  y ~ normal(yhat, sigma); //(9.6式)
  a ~ normal(a0, s_a);     //(9.7式)
}
';

par<-c("sigma","a0","b","s_a","a")                               #母数
dataSet822 <-list(n=length(学力変化$高校),J=max(学力変化$学校),
   y=学力変化$高校,x=学力変化$中学, k=学力変化$学校)             #入力

###########cmdstanrによる実行
modfile822 <- write_stan_file(HLM02)             #書き出す一時ファイル
mod822 <- cmdstan_model(modfile822)                   #コンパイル
csrfit822 <- mod822$sample(data = dataSet822,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext822<-as_draws_df(csrfit822$draws(par))
(colnames(ext822) <- gsub("\\[|\\]|,", "", colnames(ext822)))

#save(ext822,  file="./scrT/obje/stan_obje0903")
#load(file="./scrT/obje/stan_obje0903"); #予めstan( )で作ったobjectをロード

###### 表9.5 切片に分布があるモデルの母数の事後分布の要約
gqcal(ext822$sigma )
gqcal(ext822$a0    )
gqcal(ext822$b     )
gqcal(ext822$s_a   )
print('-------------------------------------')
gqcal(ext822$a1 )
gqcal(ext822$a2 )
gqcal(ext822$a3 )
gqcal(ext822$a4 )
gqcal(ext822$a5 )
gqcal(ext822$a6 )
gqcal(ext822$a7 )
gqcal(ext822$a8 )
gqcal(ext822$a9 )
gqcal(ext822$a10)

###### 表9.6 切片に関して高等学校i 行がj 列より大きい確率
phc02(0,ext822[,5:14],digits=2)

###### 図9.10 高等学校8 と7 の切片の差がc 以上である確率
phc01(seq(2,8,0.2),ext822$a8,ext822$a7,xlab="高等学校8と7の切片の差")

###### 9. 3 切片と傾きに相関のあるモデル
###### 9. 3. 1 減量開始から4 週間の体重変化

###### 表9.7 減量開始から4 週間の体重変化
(減量経過<-read.csv("./scrT/dat/減量経過.csv", header = TRUE))

###### 図9.11 4 週間の体重変化(4 人分)
Y<-減量経過[減量経過$女性==14,]
plot(Y$週数,Y$体重,pch=Y$女性,xlab="減量開始からの週数",
  ylab="体重",ylim=c(46,54),type="b")
  for (i in 15:17){
    Y<-減量経過[減量経過$女性==i,]
    par(new=T)
    plot(Y$週数,Y$体重,pch=Y$女性,xlab="",ylab="",
      ylim=c(46,54),type="b")
  }
  legend(3.2,53.5,c("女性14","女性15","女性16","女性17"),pch=c(14:17))
#dev.copy2eps(file="z0811.eps",family='Japan1')

###### 図9.12 4 週間の体重変化(全員)
Y<-減量経過[減量経過$女性==1,]
plot(Y$週数,Y$体重,xlab="減量開始からの週数",
  ylab="体重",ylim=c(46,54),type="l")
  for (i in 2:50){
    Y<-減量経過[減量経過$女性==i,]
    par(new=T)
    plot(Y$週数,Y$体重,xlab="",ylab="",ylim=c(46,54),type="l")
}

par<-c("sigma","a0","b0","s_a","s_b","a","b")                     #母数
dataSet91 <-list(n=length(減量経過$体重), J=max(減量経過$女性), 
    y=減量経過$体重, x=減量経過$週数, k=減量経過$女性)            #入力

###########cmdstanrによる実行
#コンパイル結果は 'mod81' を利用する
csrfit91 <- mod81$sample(data = dataSet91,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext91<-as_draws_df(csrfit91$draws(par))
(colnames(ext91) <- gsub("\\[|\\]|,", "", colnames(ext91)))

#save(ext91,  file="./scrT/obje/stan_obje0904")
#load(file="./scrT/obje/stan_obje0904"); #予めstan( )で作ったobjectをロード

###### 表9.8 切片と回帰係数に分布を仮定したモデルの事後分布の要約
gqcal(ext91$sigma)
gqcal(ext91$a0   )
gqcal(ext91$b0   )
gqcal(ext91$s_a  )
gqcal(ext91$s_b  )

##### 切片・回帰係数のEAP推定値
av<-apply(ext91[,c(6:55)],2,mean)
bv<-apply(ext91[,c(56:105)],2,mean)
round(rbind(av,bv),2)

###### 図9.13 切片と回帰係数の散布図
plot(av,bv,pch=16,xlab="切片",ylab="回帰係数");  abline(h=0.0)

cor(av,bv);                  #切片と傾きの相関係数 p.137,l.6


######################切片と傾きに相関のあるモデルのstanコード
HLM03<-'
data {
  int<lower=0> n;             //全データ数 
  int<lower=0> J;             //群数
  vector[n] y;                //基準変数   
  vector[n] x;                //予測変数   
  array[n] int<lower=0> k;    //分類変数 
}
parameters {
  matrix[J, 2] ab;            //切片・回帰係数 
  real<lower=0> sigma;        //誤差標準偏差
  real a0;                    //切片平均 
  real b0;                    //回帰係数平均 
  real<lower=0> s_a;          //切片SD
  real<lower=0> s_b;          //回帰係数SD
  real<lower=-1, upper=1> rho;//切片と回帰係数の相関
}
transformed parameters {
  vector[n] yhat;             //予測値
  vector[2] mu;               //切片・回帰係数平均
  cov_matrix[2] Sigma;        //切片・回帰係数の分散共分散行列
  mu[1] = a0;                 //切片の平均をμの第1要素に代入
  mu[2] = b0;                 //回帰係数の平均をμの第2要素に代入
  Sigma[1, 1] = pow(s_a, 2);  //切片の分散
  Sigma[2, 2] = pow(s_b, 2);  //回帰係数の分散
  Sigma[1, 2] = s_a * s_b * rho; //切片と回帰係数の共分散
  Sigma[2, 1] = Sigma[1, 2];  //同上
  for (i in 1 : n) {          
    yhat[i] = ab[k[i], 1] + x[i] * ab[k[i], 2];
  }                           //(9.10)式 予測値の構造
}
model {                       //yはyhatの周りで正規分布に従う
  y ~ normal(yhat, sigma);    //(9.11)式　ベクトル表記でfor文なし
  for (j in 1 : J) {          //切片と回帰係数は２変量正規分布に従う
    ab[j,  : ] ~ multi_normal(mu, Sigma);
  }                           //(9.12)式
}
';

par<-c("sigma","a0","b0","s_a","s_b","rho","ab")                 #母数
dataSet92 <-list(n=length(減量経過$体重), J=max(減量経過$女性), 
    y=減量経過$体重, x=減量経過$週数, k=減量経過$女性 )          #入力

###########cmdstanrによる実行
modfile92 <- write_stan_file(HLM03)             #書き出す一時ファイル
mod92 <- cmdstan_model(modfile92)                   #コンパイル
csrfit92 <- mod92$sample(data = dataSet92,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext92<-as_draws_df(csrfit92$draws(par))
(colnames(ext92) <- gsub("\\[|\\]|,", "", colnames(ext92)))

#save(ext92,  file="./scrT/obje/stan_obje0905")
#load(file="./scrT/obje/stan_obje0905"); #予めstan( )で作ったobjectをロード

###### 表9.9 切片と回帰係数に相関のあるモデルの母数の事後分布の要約
gqcal(ext92$sigma)
gqcal(ext92$a0   )
gqcal(ext92$b0   )
gqcal(ext92$s_a  )
gqcal(ext92$s_b  )
gqcal(ext92$rho  )

##### 切片・回帰係数のEAP推定値
av<-apply(ext92[, 7: 56],2,mean)
bv<-apply(ext92[,57:106],2,mean)
round(rbind(av,bv),2)

#### 図9.14 50 本の単回帰直線
plot(減量経過$週数,減量経過$体重,pch=減量経過$女性,
    xlab="週数",ylab="体重",type="n")
    for (i in 1:50) {abline(av[i],bv[i],lwd=1.8)}

##### 図9.15 切片と回帰係数の相関の事後分布
hist(ext92$rho,breaks=100,main="",ylab="",xlab="切片と回帰係数の相関")

cor(av,bv);                  #切片と傾きの相関係数(教科書にはなし)


####### 9. 4 レベル2 の変数があるモデル
####### 9. 4. 1 勤続年数と年収の分析

##############表9.10 企業別の勤続年数と年収
(勤続年収<-read.csv("./scrT/dat/勤続年収.csv", header = TRUE))

##############表9.11 企業の資本金の規模
(資本金  <-read.csv("./scrT/dat/資本金.csv",   header = TRUE))

############### 3つの企業の勤続年数と年収の散布図（教科書にはない）
X<-勤続年収[((勤続年収$企業==3)|(勤続年収$企業==15)|
             (勤続年収$企業==21)), ]
plot(X$勤続年,X$年収,pch=X$企業,xlab="入社からの年数",ylab="年収")
    legend(2.5,850,c("企業3","企業15","企業21"),pch=c(3,15,21))

##############図9.16 勤続年数と年収の分析
plot(勤続年収$勤続年,勤続年収$年収,pch=勤続年収$企業,
    xlab="入社からの年数",ylab="年収");               #図9.16


############################レベル２の変数があるモデルのstanコード
HLM04<-'
data {
  int<lower=0> n;                 //全データ数 
  int<lower=0> J;                 //群数
  vector[n] y;                    //基準変数   
  vector[n] x;                    //予測変数   
  vector[J] z;                    //レベル２の変数   
  array[n] int<lower=0> k;        //分類変数 
}
parameters {
  vector[J] a;                    //切片
  vector[J] b;                    //回帰係数 
  real<lower=0> sigmae;           //誤差標準偏差
  real a0;                        //切片の切片 
  real b0;                        //回帰係数の切片 
  real ca;                        //切片の係数 
  real cb;                        //回帰係数の係数 
  real<lower=0> sigmaa;           //切片SD
  real<lower=0> sigmab;           //回帰係数SD
}
transformed parameters {
  vector[n] yhat;                 //予測値
  for (i in 1 : n) {
    yhat[i] = a[k[i]] + x[i] * b[k[i]];
  }                               //(9.14)式
}
model {
  y ~ normal(yhat, sigmae);       //(9.15)式
  a ~ normal(a0 + z * ca, sigmaa);//(9.16)式
  b ~ normal(b0 + z * cb, sigmab);//(9.17)式
}
';

par<-c("sigmae","a0","b0","ca","cb","sigmaa","sigmab","a","b")#母数
dataSet93 <-list(n=length(勤続年収$年収),J=max(勤続年収$企業), 
 y=勤続年収$年収, x=勤続年収$勤続年, z=資本金$区分, k=勤続年収$企業 )#入力

###########cmdstanrによる実行
modfile93 <- write_stan_file(HLM04)             #書き出す一時ファイル
mod93 <- cmdstan_model(modfile93)                   #コンパイル
csrfit93 <- mod93$sample(data = dataSet93,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext93<-as_draws_df(csrfit93$draws(par))
(colnames(ext93) <- gsub("\\[|\\]|,", "", colnames(ext93)))

#save(ext93,  file="./scrT/obje/stan_obje0906")
#load(file="./scrT/obje/stan_obje0906"); #予めstan( )で作ったobjectをロード

###### 表9.12 レベル2 の変数があるモデルの母数の事後分布
gqcal(ext93$sigmae)
gqcal(ext93$a0)
gqcal(ext93$b0)
gqcal(ext93$ca)
gqcal(ext93$cb)
gqcal(ext93$sigmaa)
gqcal(ext93$sigmab)

#### 切片・回帰係数のEAP推定値
av<-apply(ext93[, 8:32],2,mean)
bv<-apply(ext93[,33:57],2,mean)
round(rbind(av,bv),2)

######図9.17 レベル1 の25 本の回帰直線
plot(勤続年収$勤続年,勤続年収$年収,xlab="勤続年",ylab="年収",type="n")
  for (i in 1:25) {abline(av[i],bv[i],lwd=1.8,lty=(資本金[i,2]+3))}
  legend(2.5,900,paste("区分",-2:2,sep=""),lty=1:5)

######図9.18 レベル2 の変数による切片に対する回帰直線
plot(資本金[,2],av,ylab="レベル１の切片",xlab="資本金区分",pch=16)
  abline(a=mean(ext93$a0),b=mean(ext93$ca))

######図9.19 レベル2 の変数による回帰係数に対する回帰直線
plot(資本金[,2],bv,ylab="レベル１の回帰係数",xlab="資本金区分",pch=16)
  abline(a=mean(ext93$b0),b=mean(ext93$cb))

cor(av,bv);                  #切片と傾きの相関係数(教科書にはなし)


###### 9. 5 傾きが共通でレベル2 の質的変数があるモデル

######表9.13 特別英語教育を実施しているか否かに関するレベル2 の変数
(学力変化<-read.csv("./scrT/dat/学力変化.csv", header = TRUE))
特別英語<-c(0,0,1,1,0,0,1,1,1,1)


################傾きが共通でレベル２の質的変数があるモデルのstanコード
HLM05<-'
data {
  int<lower=0> n;                 //全データ数 
  int<lower=0> J;                 //群数
  vector[n] y;                    //基準変数   
  vector[n] x;                    //予測変数   
  vector[J] z;                    //レベル２の変数   
  array[n] int<lower=0> k;        //分類変数 
}
parameters {
  vector[J] a;                    //切片
  real b;                         //回帰係数 
  real<lower=0> sigmae;           //誤差標準偏差
  real a0;                        //切片平均 
  real ca;                        //切片の係数 
  real<lower=0> sigmaa;           //切片SD
}
transformed parameters {
  vector[n] yhat;                 //予測値
  for (i in 1 : n) {
    yhat[i] = a[k[i]] + x[i] * b;
  }                               //(9.18)式
}
model {
  y ~ normal(yhat, sigmae);       //(9.19)式
  a ~ normal(a0 + z * ca, sigmaa);//(9.20)式,(9.21)式
}
';

par<-c("sigmae","a0","ca","sigmaa","b","a")                     #母数
dataSet94 <-list(n=length(学力変化$高校),J=max(学力変化$学校),
   y=学力変化$高校,x=学力変化$中学, z=特別英語, k=学力変化$学校)#入力

###########cmdstanrによる実行
modfile94 <- write_stan_file(HLM05)             #書き出す一時ファイル
mod94 <- cmdstan_model(modfile94)                   #コンパイル
csrfit94 <- mod94$sample(data = dataSet94,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext94<-as_draws_df(csrfit94$draws(par))
(colnames(ext94) <- gsub("\\[|\\]|,", "", colnames(ext94)))

#save(ext94,  file="./scrT/obje/stan_obje0907")
#load(file="./scrT/obje/stan_obje0907"); #予めstan( )で作ったobjectをロード

### 表9.14 傾きが共通でレベル2 の質的変数があるモデルの母数の事後分布の要約
gqcal(ext94$sigmae)
gqcal(ext94$a0)
gqcal(ext94$ca)
gqcal(ext94$sigmaa)
gqcal(ext94$b)

#### 切片・回帰係数のEAP推定値
av<-apply(ext94[,6:15],2,mean)
bv<-mean(ext94$b)
round(rbind(av,bv),2)

###### 図9.20 特別教育をしているか否かによる回帰直線の相違
plot(学力変化$中学,学力変化$高校,xlab="中学３年偏差値",
  ylab="高校１年偏差値", xlim=c(25,75),ylim=c(25,75),type="n")
  for (i in 1:10) {
    abline(av[i],bv,lwd=1.9,lty=((特別英語[i])*(-1)+2) )
  }
  legend(30,70,c("特別教育あり","特別教育なし"),lty=c(1,2))

mean(ext94$ca>0);   #特別教育の効果がある確率 p.143,l.3

##### 図9.21 特別教育をしている高校の切片の平均の差がc 以上である確率
phc01(seq(0,10,0.5),ext94$ca,0,xlab="切片の差")

################################# 実習課題

#
#     正解は割愛
#
