﻿##############第10章　間接質問法
# Working directoryが 'toyoda' であることの確認
(n_wd<-getwd())                #working directory(wd)の確認
(setwd('./scrT'))              #wdの移動
source('../myfunc/myfunc.R')   #自作関数の読み込み
(setwd(n_wd))                  #wd戻す
library(cmdstanr)              #パッケージcmdstanrの呼び出し
library(posterior)             #パッケージposteriorの呼び出し
prob<-c(0.025, 0.05, 0.5, 0.95, 0.975) #確率点の定義

#######################################本章で利用する関数
#MAP推定値を与える関数
MAP<-function(ext){density(ext)$x[which.max(density(ext)$y)]}

###### 10. 1 混合分布
###### 図10.1 大人と子供の身長のヒストグラム
(身長<-read.csv("./scrT/dat/身長.csv", header = TRUE))
hist(身長$x,breaks = 12,col=8,main="",xlab="身長(cm)",ylab="人数");#図10.1

#######################################混合分布モデルのstanコード
mix01<-'
data {                      //入力部
  int<lower=0> n;           //データ数
  array[n] real x;          //データ
  real<lower=0, upper=1> p; //混合比率
}
parameters {                //母数定義部
  ordered[2] mu;            //平均
  array[2] real<lower=0> sigma; //標準偏差
}
model {                     //モデル記述部
  for (i in 1 : n) {
    target += log_sum_exp(  //(10.9)式
                  log(p) + normal_lpdf(x[i] | mu[1], sigma[1]),
                  log(1 - p) + normal_lpdf(x[i] | mu[2], sigma[2]));
  }
}
';

par<-c("mu","sigma")                               #母数
dataSetmix <-list(n=length(身長$x),x=身長$x,p=0.4) #入力

###########cmdstanrによる実行
modfilemix <- write_stan_file(mix01)              #書き出す一時ファイル  
modmix <- cmdstan_model(modfilemix)                   #コンパイル
csrfitmix <- modmix$sample(data = dataSetmix,chains = 5,iter_sampling = 20000,
    init = function(){list(mu=c(138,172),sigma=c(5.8,6.8))},
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
extmix<-as_draws_df(csrfitmix$draws(par))
(colnames(extmix) <- gsub("\\[|\\]|,", "", colnames(extmix)))

#save(extmix,file="./scrT/obje/stan_obje1001")
#load(file="./scrT/obje/stan_obje1001"); #予めstan( )で作ったobjectをロード

########### 表10.1 混合正規モデルの事後分布の要約
gqcal(extmix[,1:4])

###### 10. 2 ランダム回答法
yes<-1168; no<-3072;       
x<-c(rep(1,yes),rep(0,no));                #危険ドラッグの経験比率
2.0*(yes/(yes+no))-0.5 ;                   #簡便解 (10.11)式

###########################################ランダム回答法のstanコード
random01<-'                 //ランダム回答法
data {                      //入力部
  int<lower=0> n;           //データ数
  array[n] int<lower=0> x;  //データ
  real<lower=0, upper=1> q; //マスク比率
}
parameters {                //母数定義部
  real<lower=0, upper=1> p; //知りたい比率
}
model {                     //モデル記述部
  for (i in 1 : n) {
    target += log_sum_exp(  //(10.12)式の対数の指数の和の対数
                    log(0.5) + bernoulli_lpmf(x[i] | p),
                    log(0.5) + bernoulli_lpmf(x[i] | q));
  }
}
';

par<-c("p")                                 #母数
dataSetrand <-list(n=length(x),x=x,q=0.5)     #入力

###########cmdstanrによる実行
modfilerand <- write_stan_file(random01)              #書き出す一時ファイル  
modrand <- cmdstan_model(modfilerand)                   #コンパイル
csrfitrand <- modrand$sample(data=dataSetrand,chains=5,iter_sampling=20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
extrand<-as_draws_df(csrfitrand$draws(par))
(colnames(extrand) <- gsub("\\[|\\]|,", "", colnames(extrand)))

#save(extrand,file="./scrT/obje/stan_obje1002")
#load(file="./scrT/obje/stan_obje1002"); #予めstan( )で作ったobjectをロード

###### 表10.2 危険ドラッグの経験比率の事後分布の要約
gqcal(extrand$p)

###### 図10.2 危険ドラッグの経験比率の事後分布
hist(extrand$p,breaks=50,main="",xlab="比率",ylab="",xlim=c(0,0.1),
  cex.lab=2.0,cex.axis=1.5)
#dev.copy2eps(file="z0902.eps",family='Japan1')

MAP(extrand$p);                   #MAP推定値　p.150,l.1

###### 図10.3 p(p > c) という研究仮説が正しい確率
phc01(seq(0,0.10,0.005),extrand$p,0,xlab="比率 > c")
#dev.copy2eps(file="z0903.eps",family='Japan1')


######## 10. 3 Aggregated Response 法
######## 10. 3. 1 異性との性行為の経験人数
x<-c(
-3,-2, 1, 3,-9, 4,-1,-3,-5, 2, 0,-7,-1,-4,-3,10, 2, 6, 2,-2,-1,-6,-6,-1, 8,-6,
-6, 1, 6, 1, 1, 5, 6, 1, 2, 9, 3, 3,-2, 5, 6, 5, 0, 7,-3,-1,-1, 6, 6, 4, 4,-8,
 5, 2, 7, 7, 5,-1, 6,10, 9, 2,-8,-6,-8,-7,-2, 7, 4, 6,-5, 0,-1,14, 0, 5, 9,-2,
 7, 4,-2, 2,-8,-3,11,-5,-1,-5, 3, 6,-5,-4, 4, 4, 2,10,-3, 1, 3, 5,-6,-3,-1, 2,
-9, 2,-4, 3,-1, 1,-6, 7, 4,-5, 0, 7, 4,-7,-4,-3, 2, 2,-3, 6,-4,-6,-3, 0, 5, 3,
 6,-2, 0,10, 9, 1, 8,10, 3, 4, 5, 4, 4, 9,-5, 8,-1, 1,-7, 2, 9,-2, 5,-7,-1, 4,
 1, 3, 1, 2, 0,-4, 1,-5, 3,-6,-4,-4, 2,-1,-3,-2,-8,-3,-4, 1, 0, 1, 0, 7,-4, 7,
10, 4, 9,11, 7,10,-1, 5, 5, 0, 9,-3,-7,-6, 9, 3, 0, 7, 2, 7, 0, 3,-1,-3,-8, 0,
 0,10, 7, 1,-2,-3,-9,-7,-4, 6,-2,11,13, 8, 8,-4, 0,-1,-8, 7,10, 8, 3,10, 4,-6,
 6, 7,-5, 4,-8, 7,-7,-3, 2,-8, 3, 3,-2, 2,-3,-4,-5,-2, 7,-3, 2,-1,-7,-1,-6,-6,
10,11,-5, 6, 7, 5,-6, 8,-4,-1, 4,-2,-1,-6, 6,-5,11, 5, 1, 3, 0,-2,-7,-1,-6, 9,
-6,-7,-6, 4, 3,10, 4, 3,-1, 5, 7, 6,-4, 3, 0,12,11, 7, 8, 4,-2, 1,-6, 6,-1,-1,
 5, 4, 6,11,-7, 4,-1, 1,-2, 2,-6,-3,-7, 2,-5, 4, 2,-3)

###### 図10.4 回答数字のヒストグラム
hist(x,main="",xlab="回答した数字",ylab="",
     xlim=c(-9.5,14.5),cex.lab=2.0,cex.axis=1.5,breaks=24)
#dev.copy2eps(file="z0904.eps",family='Japan1')

mean(x);                     #平均　　簡便解 (10.16)式
max(x);min(x);               #最大・最小

####################Aggregated Response法のstanコード
AR01<-'
data {                  //Aggregated Response法
  int<lower=0> n;       //データ数
  int<lower=0> m;       //マスク数
  array[n] int x;       //データ
  array[m] int mask;    //マスク要素
  real pro;             //一様分布の確率
}
parameters {            //母数定義部
  real<lower=0> lambda; //ポアソン分布母数
}
model {                 //モデル記述部
  real j;               //そのデータからは存在するマスク数
  int jj;               //
  real pr;              //可能性のあるマスクの数の逆数
  vector[m] lp;
  for (i in 1 : n) {
    j = 0.0;
    for (k in 1 : m) {
      if (0 <= x[i] - mask[k]) {
        j = j + 1.0;
      }
    }
    pr = 1.0 / j;
    jj = 0;
    for (k in 1 : m) { //kはマスク
      if (0 <= x[i] - mask[k]) {  //可能性のあるマスクだけ計算
        jj = jj + 1;
        lp[jj] = log(pr) + poisson_lpmf(x[i] - mask[k] | lambda); //(10.24)式
      }
    }
    target += log_sum_exp(lp[1 : jj]);
  }
}
';

par<-c("lambda")                                         #母数
mask<-c(-9:0,0:9); m<-length(mask)                       #マスク
dataSetAR <-list(n=length(x),m=m,x=x,mask=mask,pro=1/m)  #入力

###########cmdstanrによる実行
modfileAR <- write_stan_file(AR01)              #書き出す一時ファイル  
modAR <- cmdstan_model(modfileAR)               #コンパイル
csrfitAR <- modAR$sample(data = dataSetAR,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
extAR<-as_draws_df(csrfitAR$draws(par))
(colnames(extAR) <- gsub("\\[|\\]|,", "", colnames(extAR)))

#save(extAR,file="./scrT/obje/stan_obje1003")
#load(file="./scrT/obje/stan_obje1003"); #予めstan( )で作ったobjectをロード


###### 表10.3 λの事後分布の要約
gqcal(extAR$lambda)

###### 図10.5 ポアソン分布の母数の事後分布
hist(extAR$lambda,breaks=100,main="",xlab="λ",ylab="",cex.lab=2,cex.axis=1.5)
#dev.copy2eps(file="z0905.eps",family='Japan1')

###### 図10.6 p(λ > c) という研究仮説が正しい確率
phc01(seq(0.5,2.5,0.1),extAR$lambda,0,xlab="平均λ > x")
#dev.copy2eps(file="z0906.eps",family='Japan1')

###### 図10.7 経験人数の事後予測分布
T<-length(extAR$lambda);pre<-numeric(T);         #格納の箱
for(i in 1:T){ pre[i]<-rpois(1,extAR$lambda[i])} #事後予測分布
barplot(table(pre),axes=F, xlab="経験人数",cex.lab=2.0,cex.axis=2.0)
#dev.copy2eps(file="z0907.eps",family='Japan1')

###### 表10.4 経験人数の確率と累積確率
round(table(pre)/T,3)         #確率
round(cumsum(table(pre)/T),3) #累積確率
summary(pre)                  #平均・中央値・最大値
round(sd(pre),3)              #標準偏差

######################10. 4 アイテムカウント法
##### 表10.5 短リストと長リストに対する該当数による度数分布
a<-c(107,133,160,80,75,110, 96)
b<-c( 98,142,166,75,82,104,135, 0)
K<-6

#######################################アイテムカウント法のstanコード
itemcount02<-'
data {                         //入力部
  int<lower=0> K;              //短リストのカテゴリ数
  array[K + 1] int<lower=0> a; //短リストの反応数
  array[K + 2] int<lower=0> b; //長リストの反応数
}
transformed data {             //データ変換部
}
parameters {                   //母数定義部
  simplex[K + 1] theta;        //短リストの確率
  simplex[K + 1] pi;           //pによる単リストの条件付き確率
  real<lower=0, upper=1.0> p;  //キー項目にあてはまる確率
}
transformed parameters {       //母数変換部
  vector[K + 1] pi1;           //短リストがkでマスクが1となる同時確率
  simplex[K + 2] theas;        //長リストの母比率
  real temp;
  pi1 = pi * p;
  theas[1] = theta[1] - pi1[1];//(10.38)式
  for (j in 2 : K + 1) {
    theas[j] = theta[j] - pi1[j] + pi1[j - 1]; //(10.39)式
  }
  temp = 0;
  for (j in 1 : K + 1) {
    temp = temp + theas[j];
  }
  theas[K + 2] = 1 - temp;
}
model {                        //モデル記述部
  a ~ multinomial(theta);      //短リストの多項分布
  b ~ multinomial(theas);      //長リストの多項分布
}
';

par<-c("p","theta","theas")     #母数
dataSet違法 <-list(K=K,a=a,b=b)   #入力

###########cmdstanrによる実行
modfile違法 <- write_stan_file(itemcount02)         #書き出す一時ファイル  
mod違法 <- cmdstan_model(modfile違法)               #コンパイル
csrfit違法 <-mod違法$sample(data=dataSet違法,chains=5,iter_sampling=20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
ext違法<-as_draws_df(csrfit違法$draws(par))
(colnames(ext違法) <- gsub("\\[|\\]|,", "", colnames(ext違法)))

#save(ext違法,file="./scrT/obje/stan_obje1004")
#load(file="./scrT/obje/stan_obje1004"); #予めstan( )で作ったobjectをロード

###### 表10.6 p の事後分布の要約
gqcal(ext違法[,1:16])
