﻿############# 第11章　項目反応理論
# Working directoryが 'toyoda' であることの確認
(n_wd<-getwd())                #working directory(wd)の確認
(setwd('./scrT'))              #wdの移動
source('../myfunc/myfunc.R')   #自作関数の読み込み
(setwd(n_wd))                  #wd戻す
library(cmdstanr)              #パッケージcmdstanrの呼び出し
library(posterior)             #パッケージposteriorの呼び出し
library(psych)
library(irtoys)
library(ltm)
prob<-c(0.025, 0.05, 0.5, 0.95, 0.975) #確率点の定義

##########11. 1 2 値項目の項目特性曲線
##### 11. 1. 3 項目困難度
##### 図11.1 項目困難度の役割
(ip<-matrix(cbind(1.7,-2:2,0),5,3))
plot(irf(ip),main="")
text(-2.5,0.5,"b= ー2",cex=1.5)
text(-1.5,0.5,"b= ー1",cex=1.5)
text(-0.5,0.5,"b= 0",cex=1.5)
text( 0.5,0.5,"b= 1",cex=1.5)
text( 1.5,0.5,"b= 2",cex=1.5)
abline(v=0)
abline(v=1)
#dev.copy2eps(file="z1001.eps",family='Japan1');

round(irf(ip,x=0.0)[[2]],3)   #p.165,bl.3 θ=0.0 の正答率
round(irf(ip,x=1.0)[[2]],3)   #p.165, l.2 θ=1.0 の正答率

###### 11. 1. 4 項目識別力
###### 図11.2 項目識別力の役割
(ip<-matrix(cbind(1.7*c(0.5,1,2),0,0),3,3))
plot(irf(ip),main="")
text(-2.5,0.2,"a=0.5",cex=1.5)
text(-1.4,0.17,"a=1.0",cex=1.5)
text(-0.4,0.05,"a=2.0",cex=1.5)
abline(v=-2)
abline(v=1)
#dev.copy2eps(file="z1002.eps",family='Japan1')

round(irf(ip[c(1,3),],x=-2)[[2]],3)   #p.164,bl.5 θ=-2.0 の正答率
round(irf(ip[c(1,3),],x= 1)[[2]],3)   #p.165,bl.2 θ= 1.0 の正答率

######## 表11.1 指導性・リーダーシップ尺度の項目(2 値)
a<-c(0.607,0.854,0.637,1.789,0.985,0.446)
b<-c(-0.947,-0.047,-0.250,-0.744,-0.289,-0.253)
(ip<-matrix(cbind(1.7*a,b,0),6,3))

######## 図11.3 指導性・リーダーシップの尺度の項目のICC
plot(irf(ip),main="")
text(-0.4,0.9, 4,cex=2);  #識別力が高い
text(   2,0.8, 6,cex=2);  #識別力が低い
text(-1.5,0.45,1,cex=2);  #困難度低い
text(-0.1,0.4, 2,cex=2);  #困難度高い
#dev.copy2eps(file="abcd.eps",family='Japan1')

##########指導性・リーダーシップテストの分析
miki<-c(
"私の意見は仲間に反映されることが多い",
"人を引っ張っていく力がある",
"集団の中ではリーダーシップを発揮する",
"仲間に明確なアドバイスを与えることができる",
"自分は仲間の中でまとめ役である",
"対立している仲間の仲裁が得意である")
I<-length(miki);u<-numeric(I)
#for (i in 1:I){                           #自分でやってみる場合のデータ収集
#  u[i]<-menu(c("いいえ","はい"), graphics = F,title = miki[i])}

u<-c(2, 1, 2, 1, 1, 1);               #例題データ　(11.11)式

###################多値項目の正規累積項目反応モデルのstanコード
irt<-'
data {
  int<lower=2> K;                              //反応カテゴリ数
  int<lower=1> p;                              //項目数
  array[p] int<lower=1, upper=K> u;            //試験結果(1からKまで)
  array[p] real a;                             //識別力
  matrix[p, K - 1] b;                          //困難度
  real mu;                                     //平均（事前分布）
  real<lower=0> sigma;                         //標準偏差（事前分布）
}
parameters {
  real theta;                                  //被験者母数
}
model {
  vector[K] pro;                               //反応確率
  theta ~ normal(mu, sigma);                   //尺度値事前分布(11.9)式右辺後半
  for (j in 1 : p) {
    pro[1] = 1 - Phi(a[j] * (theta - b[j, 1]));//(11.14)式,(11.22)式,(11.6)式
    for (k in 2 : (K - 1)) {
      pro[k] = Phi(a[j] * (theta - b[j, k - 1]))
              - Phi(a[j] * (theta - b[j, k])); //(11.15)式,(11.22-25)式
    }
    pro[K] = Phi(a[j] * (theta - b[j, K - 1]));//(11.16)式,(11.26)式,(11.6)式
    u[j] ~ categorical(pro);                   //尤度,(11.8)式
  }
}
generated quantities {                         //生成量
  real hensati;                                //偏差値
  hensati = theta * 10 + 50;
}
';

par<-c("theta","hensati")                                 #母数
a<-c(0.607,0.854,0.637,1.789,0.985,0.446)                 #識別力
b<-c(-0.947,-0.047,-0.250,-0.744,-0.289,-0.253)           #困難度
dataSetirt02 <-list(K=2,p=I, u=u,a=a,b=matrix(b,I,1),mu=0,sigma=1)

###########cmdstanrによる実行
modfileirt02 <- write_stan_file(irt)             #書き出す一時ファイル
modirt02 <- cmdstan_model(modfileirt02)                   #コンパイル
csrfitirt02<-modirt02$sample(data=dataSetirt02,chains=5,iter_sampling=20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
irt02<-as_draws_df(csrfitirt02$draws(par))
(colnames(irt02) <- gsub("\\[|\\]|,", "", colnames(irt02)))

#save(irt02,file="./scrT/obje/stan_obje1101")
#load(file="./scrT/obje/stan_obje1101"); #予めstan( )で作ったobjectをロード

####### 表11.2 尺度値θ と偏差値の事後分布の要約統計量
gqcal(irt02[,1:2])


###########11.2   3値項目の段階反応モデル

###### 図11.4 ICC とIRCCC の関係
p1<-function(theta){pnorm(theta-(-1))} 
p2<-function(theta){pnorm(theta-1)} 
plot(p1,-3,3,ylim=c(0,1),xlab="",ylab="",lwd=2)
  par(new=T)
plot(p2,-3,3,ylim=c(0,1),xlab="",ylab="",lwd=2)
text(-0.7,0.9,"f(u=1|θ=0)",cex=2); 
text(0.0,0.5,"f(u=2|θ=0)",cex=2); 
text(0.7,0.1,"f(u=3|θ=0)",cex=2); 
arrows(0,0,0,0.15,code=3,angle=15,lwd=1.8)
arrows(0,0.17,0,0.83,code=3,angle=15,lwd=1.8)
arrows(0,0.87,0,1,code=3,angle=15,lwd=1.8)
text(-1.6,0.4,"p1(θ)",cex=2); 
text( 1.6,0.6,"p2(θ)",cex=2);                    #図11.4ここまで
#dev.copy2eps(file="z1004.eps",family='Japan1')


########## 11.2.1 社会的外向性テスト
miki<-c(
"1 話し好きである",
"2 人と広く付き合うほうだ",
"3 無口である",
"4 自分はわりと人気者だ",
"5 生き生きしていると人に言われる",
"6 陽気である",
"7 初対面のひとには自分のほうから話しかける",
"8 よく人から相談を持ちかけられる",
"9 話題には事欠かないほうだ",
"10 誰とでも気さくに話せる")
I<-length(miki);u<-numeric(I)
#for (i in 1:I){                          #自分でやってみる場合のデータ収集
#  u[i]<-menu(c("あてはまらない","どちらともいえない",
#                   "あてはまる"), graphics = F,title = miki[i])}
#u[3]<- (-1)*u[3]+4;   #逆転項目処理
u<-c(3,3,2,1,1,3,3,1,3,3)                 #例題データ (11.17)式

###### 表11.3 社会的外向性尺度の項目(3 値)
a<-c(1.241,1.096,0.856,0.735,0.761,1.331,0.710,0.528,0.883,1.461); #識別力
b<-matrix(
   c(-1.283, 0.086,
     -1.060, 0.199,
     -1.333,-0.021,
     -0.974, 1.989,
     -0.395, 1.379,
     -1.388, 0.146,
     -1.015, 0.768,
     -1.291, 1.221,
     -0.585, 1.180,
     -0.979, 0.306),10,2,T);                  #困難度
p1<-function(theta){pnorm(aa*(theta-b1))} ;   #(11.12)式
p2<-function(theta){pnorm(aa*(theta-b2))} ;   #(11.13)式
q1<-function(theta){1-p1(theta)}  ;           #(11.14)式
p21<-function(theta){p1(theta)-p2(theta)} ;   #(11.15)式

###### 図11.5 困難度レベルの違い　p.170
###### 図11.6 識別力の違い        p.170
###### 図11.7 困難度の差の違い    p.171
par(mfrow=c(3,3))
for(i in c(1:9)){
  aa<-a[i];b1<-b[i,1];b2<-b[i,2]
  plot(q1,-3,3,ylim=c(0,1),xlab="",ylab="")
  par(new=T)
  plot(p21,-3,3,ylim=c(0,1),xlab="",ylab="")
  par(new=T)
  plot(p2,-3,3,ylim=c(0,1),xlab=miki[i],ylab="",cex.lab=1.5)
}
par(mfrow=c(1,1))
#dev.copy2eps(file="z13043.eps",family='Japan1')

par<-c("theta","hensati")                                 #母数
dataSetirt03 <-list(K=3,p=I, u=u,a=a,b=b,mu=0,sigma=1)    #入力

###########cmdstanrによる実行
#コンパイルは 'modirt02' を利用する
csrfitirt03<-modirt02$sample(data=dataSetirt03,chains=5,iter_sampling=20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
irt03<-as_draws_df(csrfitirt03$draws(par))
(colnames(irt03) <- gsub("\\[|\\]|,", "", colnames(irt03)))

#save(irt03,file="./scrT/obje/stan_obje1102")
#load(file="./scrT/obje/stan_obje1102"); #予めstan( )で作ったobjectをロード

#表11.4 尺度値θ と偏差値の事後分布の要約統計量
gqcal(irt03[,1:2])


########## 11.3   5値項目の段階反応モデル

##### 11.3.2 共感性テスト
miki<-c(
"人の話をじっくり聞くことが得意である",
"人から悩みを相談されることが多い",
"涙もろく、もらい泣きをするほうである",
"他人の立場にたって共感することができる",
"悲しんでいる相手の話を聞くと、同様の悲しさを感じてしまう",
"仲間の関心事には積極的に興味を示す")
I<-length(miki);u<-numeric(I)
#for (i in 1:I){                        #自分でやってみる場合のデータ収集
#  u[i]<-menu(c("あてはまらない","ややあてはまらない","どちらともいえない",
#      "ややあてはまる","あてはまる"), graphics = F,title = miki[i])}

u<-c(2, 3, 3, 2, 5, 4)             #例題データ (11.28)式

####### 表11.5 共感性尺度の項目(5 値)
a<-c(1.974,1.665,1.407,1.543,1.008,1.770)        #識別力
b<-matrix(c(
-2.164,   -0.499,    0.149,    1.426,
-2.452,   -0.517,    0.598,    2.265,
-2.434,   -0.968,   -0.345,    0.945,
-2.043,   -0.402,    0.634,    2.103,
-4.150,   -1.679,   -0.348,    1.863,
-2.208,   -0.727,    0.278,    1.644),6,4,T)      #困難度

###各種関数の定義　p1 (11.18)式, p2 (11,19)式, q1 (11.22)式は定義済
p3<-function(theta){pnorm(aa*(theta-b3))};  #(11.20)式
p4<-function(theta){pnorm(aa*(theta-b4))};  #(11.21)式
p21<-function(theta){p1(theta)-p2(theta)};  #(11.23)式
p32<-function(theta){p2(theta)-p3(theta)};  #(11.24)式
p43<-function(theta){p3(theta)-p4(theta)};  #(11.25)式

###### 図11.8 項目識別力の役割
par(mfrow=c(3,2))
for(i in 1:6){
  aa<-a[i];b1<-b[i,1];b2<-b[i,2];b3<-b[i,3];b4<-b[i,4]
  plot(q1,-3,3,ylim=c(0,1),xlab="",ylab="")
  par(new=T)
  plot(p21,-3,3,ylim=c(0,1),xlab="",ylab="")
  par(new=T)
  plot(p32,-3,3,ylim=c(0,1),xlab="",ylab="")
  par(new=T)
  plot(p43,-3,3,ylim=c(0,1),xlab="",ylab="")
  par(new=T)
  plot(p4,-3,3,ylim=c(0,1),xlab=miki[i],ylab="",cex.lab=1.5)
}
par(mfrow=c(1,1))
#dev.copy2eps(file="z1306.eps",family='Japan1')

par<-c("theta","hensati")                                 #母数
dataSetirt05 <-list(K=5,p=I, u=u,a=a,b=b,mu=0,sigma=1)    #入力

###########cmdstanrによる実行
#コンパイルは 'modirt02' を利用する
csrfitirt05<-modirt02$sample(data=dataSetirt05,chains=5,iter_sampling=20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
irt05<-as_draws_df(csrfitirt05$draws(par))
(colnames(irt05) <- gsub("\\[|\\]|,", "", colnames(irt05)))

#save(irt05,file="./scrT/obje/stan_obje1103")
#load(file="./scrT/obje/stan_obje1103"); #予めstan( )で作ったobjectをロード

######## 表11.6 尺度値θと偏差値の事後分布の要約統計量
gqcal(irt05[,1:2])

################## 11 4   2値項目の3母数項目特性曲線

###### 図11.9 3 母数モデルのICC
(ip<-matrix(cbind(1.7,0,c(0.2,0.25,0.33, 0.5)),4,3))
plot(irf(ip),main="")
text(-3.5,0.55,"c= 0.5",cex=1.5)
text(-3.5,0.40,"c= 0.33",cex=1.5)
text(-3.5,0.29,"c= 0.25",cex=1.5)
text(-3.5,0.16,"c= 0.2",cex=1.5)
#dev.copy2eps(file="z1009.eps",family='Japan1')

####### 図11.10 3 母数モデルの10 項目のICC
a<-c(1.55,1.82,1.58,1.81,1.76,1.95,1.53,1.66,1.75,1.55);   #識別力
b<-c(0.23,0.33,0.00,0.07,-0.98,0.61,-0.26,0.06,0.17,-0.54);#困難度
c<-c(0.15,0.00,0.09,0.15,0.18,0.04,0.11,0.10,0.13,0.14);   #当て推量母数
ip<-cbind(a,b,c)
plot(irf(ip),main="")
#dev.copy2eps(file="z1010.eps",family='Japan1')

u<-c(1,1,1,1,2,1,2,2,1,2)                 #例題データ p.176,bl.2

###############2値項目の3母数項目特性曲線のstanコード
irt_guess<-'
data { 
  int<lower=1> p;                   //項目数
  array[p] int<lower=1, upper=2> u; //試験結果
  array[p] real a;                  //識別力
  array[p] real b;                  //困難度
  array[p] real c;                  //当て推量
  real mu;                          //平均（事前分布）
  real<lower=0> sigma;              //標準偏差（事前分布）
}
parameters {
  real theta;                       //被験者母数
}
model {
  vector[2] pro;                    //反応確率
  theta ~ normal(mu, sigma);        //尺度値事前分布
  for (j in 1 : p) {
    pro[2] = c[j] + (1 - c[j]) * Phi(a[j] * (theta - b[j])); //(11.29)式
    pro[1] = 1 - pro[2];
    u[j] ~ categorical(pro);        //尤度
  }
}
generated quantities {              //生成量
  real hensati;                     //偏差値
  hensati = theta * 10 + 50;
}
';

par<-c("theta","hensati")
dataSetirt_guess <-list(p=length(u), u=u,a=a,b=b,c=c,mu=0,sigma=1)

###########cmdstanrによる実行
modfileirt_guess <- write_stan_file(irt_guess)         #書き出す一時ファイル  
modirt_guess <- cmdstan_model(modfileirt_guess)        #コンパイル
csrfitirt_guess<-modirt_guess$sample(data=dataSetirt_guess,chains=5,
   iter_sampling = 20000,iter_warmup=1000,parallel_chains=5,seed=1234)  #MCMC
irt_guess<-as_draws_df(csrfitirt_guess$draws(par))
(colnames(irt_guess) <- gsub("\\[|\\]|,", "", colnames(irt_guess)))

#save(irt_guess,file="./scrT/obje/stan_obje1104")
#load(file="./scrT/obje/stan_obje1104"); #予めstan( )で作ったobjectをロード

###### 表11.7 3 母数モデルによる尺度値 と偏差値の事後分布の要約
gqcal(irt_guess[,1:2])


#######################################実習課題
#自分でデータを作ったら以下はやらない
#irt02<-extract(fitirt02, par) 
#irt03<-extract(fitirt03, par) 
#irt05<-extract(fitirt05, par) 

#########################phc曲線
par(mfrow=c(3,1));     
phc01(seq(30,50,0.5),irt02$hensati,cc="gtc",byoga="yes",xlab="指導性・リーダーシップ尺度(２値)")
phc01(seq(45,65,0.5),irt03$hensati,cc="gtc",byoga="yes",xlab="社会的外向性尺度(３値)")
phc01(seq(40,55,0.5),irt05$hensati,cc="gtc",byoga="yes",xlab="共感性尺度の項目(５値)")
par(mfrow=c(1,1))

#########################phcテーブル
phc01(seq(30,35,1),irt02$hensati,cc="gtc",byoga="no")
phc01(seq(45,50,1),irt03$hensati,cc="gtc",byoga="no")
phc01(seq(43,48,1),irt05$hensati,cc="gtc",byoga="no")


