﻿############　第12章　予測変数を直交化した重回帰分析
# Working directoryが 'toyoda' であることの確認
(n_wd<-getwd())                        #working directoryの確認
(setwd('./scrT'))
source('../myfunc/myfunc.R')      #自作関数の読み込み
(setwd(n_wd))
#install.packages("conjoint")
#install.packages("DoE.base")
library("conjoint")
library("DoE.base")

#####パッケージconjointによる"data.frame"による直交表の作成
#384種類の全てのプロファイルの作成
(omiai_every<-expand.grid(
   喫煙=c("禁煙","喫煙有"),
   年齢=c("29歳迄","制限無"),
   職業=c("公務員","限定無"),
   年収=c("500万以上","限定無"),
   学歴=c("大学卒","高校卒"),
   結婚=c("初婚","限定無"),
   お酒=c("お酒無","お酒有"),
   場所=c("公民館","ホテル","客船上")
))
#384種類から44プロファイルで直交表を作る。
#それでも多すぎる大変だから、パッケージconjointは使わない
(omiai_con<-caFactorialDesign(data=omiai_every,type="orthogonal"))
nrow(omiai_con)

####### 直交表は、パッケージDoE.baseによって作る
#乱数を使用しているので、seedを指定して再現性を確保する
#24プロファイルで優れている、クラスは"design"
(omiai_design<-oa.design(factor.names=list(
   喫煙=c("禁煙","喫煙有"),
   年齢=c("29歳迄","制限無"),
   職業=c("公務員","限定無"),
   年収=c("500万以上","限定無"),
   学歴=c("大学卒","高校卒"),
   結婚=c("初婚","限定無"),
   お酒=c("お酒無","お酒有"),
   場所=c("公民館","ホテル","客船上")
),seed=1234))

#DoE.baseで作った直交表をパッケージ"conjoint"で分析する。
#そのためには"data.frame"に変換する必要がある。水準は数字で表現される
(omiai_doe<-caEncodedDesign(omiai_design))

#バージョンが変わると同じseedに対して、"oa.design"は同一の直交表を
#作らないので"omiai_doe"は保存しておいた方が良い。
#omiai_designの方は分析には使わない、事実、以下に読みこむものとは違う

######## 表12.1 「直交お見合いデータ」に使用する直交表(水準)
(omiai_design<-read.csv(file="scrT/dat/お見合い水準表.csv",header=T))

#####表12.2 「直交お見合いデータ」に使用する直交表(ダミー変数)の<<別表現>>
#数字で水準を表現する（分析にはこちらを使用する）
(omiai_doe<-read.csv(file="scrT/dat/お見合い直交表.csv",header=T))

#水準ラベルの作成　　　p.181,bl.7
omiai_lab<-c("禁煙","喫煙有",
             "29歳迄","制限無",
             "公務員","限定無",
             "500万以上","限定無",
             "大学卒","高校卒",
             "初婚","限定無",
             "お酒無","お酒有",
             "公民館","ホテル","客船上")

######## 表12.3 「直交お見合いデータ」
(omiai_data<-read.csv(file="scrT/dat/お見合い.csv"))

######## 表12.4 「直交お見合いデータ」の部分効用値  の中の「全体」
Conjo<-Conjoint(y=omiai_data,x=omiai_doe,z=omiai_lab) 

######  図12.2 部分効用値の図示 (の左図)　全体の描画
source("scrT/myfuncもうひとつの重回帰分析.R")
omiai_util<-caUtilities(y=omiai_data,x=omiai_doe,z=omiai_lab)
plot.Conjoint(utility=omiai_util,label=omiai_lab)
text(0.4,2,"全体",cex=1.7)

####### 回答者ごとの効用値（全員分、教科書にはない）
(sub_util <- caPartUtilities(y=omiai_data,x=omiai_doe,z=omiai_lab))

####### 図12.2 部分効用値の図示 (の右図)　回答者8の描画
plot.Conjoint(utility=sub_util[8,],label=omiai_lab)
text(0.4,2,"被験者8",cex=1.7)
#dev.copy2eps(file="z1202.eps",family='Japan1')

######## 表12.4 「直交お見合いデータ」の部分効用値  の中のs1からs8
t(sub_util)[,1:8]


######### 表12.5 「直交お見合いデータ」の相対重要度
subimport<-sub.imp(data=omiai_data,design=omiai_doe,label=omiai_lab)
import<-round(colMeans(subimport),1);      #「全体」の相対重要度の計算
cbind(import,round(t(subimport)[,1:8],1)); #「全体」と回答者を同時に表示
round(apply(cbind(import,t(subimport)[,1:8]),2,sum),0);#和が100になる確認

######## 表12.6 直交表に登場したプロファイルの効用値
#総効用値の出力(1つ１つ計算できるが、一度に全部は出ない)
conjyo.sim(data=omiai_data,design=omiai_doe,lab=omiai_lab,num=3)

#回答者ごとの総効用値　　（これは教科書にはない）
#omiai_data[i,]に回答者番号 i を入れる。この例は1番目の回答者
conjyo.sim(data=omiai_data[1,],design=omiai_doe,label=omiai_lab,num=3)

