﻿###########　第13章　質的研究における飽和率・寡占度
# Working directoryが 'toyoda' であることの確認
(n_wd<-getwd())                #working directory(wd)の確認
(setwd('./scrT'))              #wdの移動
source('../myfunc/myfunc.R')   #自作関数の読み込み
(setwd(n_wd))                  #wd戻す
library(cmdstanr)              #パッケージcmdstanrの呼び出し
library(posterior)             #パッケージposteriorの呼び出し
prob<-c(0.025, 0.05, 0.5, 0.95, 0.975) #確率点の定義

###### 13. 1 質的知見収集の特徴
###表13.1 大学入試方法の改善に関する進路指導担当教員からの自由記述意見の度数
N<-  91;               #対象数
x<-  1:N;              #対象名ベクトル
f1<-
c(282,266,265,237,189,163,152,137,135,131,129,99,98,94,93,93,80,78,73,72,58,
58,57,56,52,51,45,44,40,39,39,35,32,29,28,28,28,27,25,23,23,22,20,20,19,
17,16,15,14,14,14,13,11,11,7 ,7 ,6) 
f2<-
c(6,6,6,5,5,5,5,4,4,4,4,4,3,3,3,3,3,2,2,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1)
f<-c(f1,f2)
names(f)<-x
cumsum(f)/sum(f);    #累積確率　p.196,第1パラグラフ

#########図13.1 自由記述意見のヒストグラムの外枠
plot(f,type="l",ylim=c(0,290),xlab="順位",ylab="",
      ,cex.axis=1.8, cex.lab=2.0,lwd=2.5)
lines(c(1,91),c(0,0),lwd=2.5)
lines(c(1,1),c(0,f[1]),lwd=2.5)
lines(c(54,54),c(0,f[54]),lwd=2.5)
lines(c(74,74),c(0,f[74]),lwd=5.5)
text(25,170,"7.5%の時間, 59%の知見",cex=2.0)
text(25,150,"累積確率 97.1%"  ,cex=2.0)
text(65,75,"22.5%の時間, 81%の知見",cex=2.0)
text(65,55,"累積確率 99.4%"  ,cex=2.0)
arrows(65,45,60,f[60],lwd=2.0)
arrows(25,140,16,f[16],lwd=2.0);                    #図13.1ここまで
#dev.copy2eps(file="z1301.eps",family='Japan1')

#######################第13章で利用する関数ここから

#13.2.1 ジップ分布　確率関数                (13.1)式、(13.2)式
zipf_pmf<-function(r,s,N,Q=0){
  if (Q==1) {zeta<-myzeta(s,1000,Q)}
  if (Q==0) {zeta<-myzeta(s,N,Q)}
   (1/(r^s))/zeta
}
#13.2.3 ゼータ関数の近似                            (13.6)式
myzeta<-function(s,m,Q){
  zeta<-0
  for (n in 1:m){zeta <- zeta + (1/(n^s))}
  if (Q==1) { zeta<- zeta -((m^(1-s))/(1-s))}
  return(zeta)
}

#13.2.4 飽和率  zipf分布の累積分布関数           (13.9)式
zipf_cdf<-function(r,s,N,Q=0){
  pro<-0
  for (i in 1:r){
    pro<-pro+zipf_pmf(i,s,N,Q) 
  }
  return(pro)
}
#######################第13章で利用する関数ここまで


#########図13.2 ジップ分布の形状と寡占度s
N<-  8;               #対象数
x<-  1:N;              #対象名ベクトル
p<- numeric(N);       #zipf分布の確率分布
names(p)<-x
for (s in seq(0.6,1.2,0.2)){
   for (i in 1:N) {p[i]<-zipf_pmf(r=i,s=s,N=N, Q=0)}
plot(p,type="l",ylim=c(0,0.5),xlab="順位",ylab="確率",
      ,cex.axis=1.8, cex.lab=1.3,lwd=2.0)
par(new=T)
}
text(1.5,0.42,"s=1.2",cex=1.4)
text(1.0,0.38,"s=1.0",cex=1.4)
text(1.0,0.28,"s=0.8",cex=1.4)
text(1.5,0.18,"s=0.6",cex=1.4)
par(new=F);                                        #図13.2ここまで
#dev.copy2eps(file="z1302.eps",family='Japan1')

##########13.2.3 ゼータ関数の近似
#####小数6桁までの正確な値
(pi^2)/6;                                #(13.7)式
(pi^4)/90;                               #(13.8)式

################近似式の精度
myzeta(2,400,1);  myzeta(2,4000,1);      #p.199,l.2
myzeta(4, 30,1);  myzeta(4,  40,1);      #p.199,l.6

######## ジップ分布と、独立サンプルの一致の確認(教科書にはない)
#状況の設定
n<- 1800;              #被検者数
a<-  5;                #非復元抽出数
N<-  20;               #対象数
s<-  1.5;              #zipf分布の母数
bp<- 0.05;             #途中で想起をやめる確率、最低一つは想起する
x<-  1:N;              #対象名ベクトル
p<- numeric(N);        #zipf分布の確率分布
pro<-c(0.025, 0.5, 0.975); #確率点の定義
par<-c("s","prob");    #sはzupfの母数、probは被覆率

#####zipf分布の確率分布表の作成
names(p)<-x
for (i in 1:N) {p[i]<-zipf_pmf(r=i,s=s,N=N, Q=0)}
maxy<-ceiling(p[1]*10)/10
plot(p,type="b",ylim=c(0,maxy),xlab="")
print(p);

#a×n個のzipf分布に従う乱数を発生
#(グラフが一致し正しく発生されていることを確認)
Gv<-sample(x,a*n,prob=p, replace =T)
Gf<-table(Gv);    #非復元抽出データの度数表
Gs<-Gf/(n*a);     #復元抽出データの相対度数
plot(Gs, ylim=c(0,maxy),lty=1,lwd=5,ylab="",xlab="",main="")
par(new=T)
plot(p,  ylim=c(0,maxy),type="l",xaxt="n",lwd=2,ylab="",xlab="",main="")
######## ジップ分布と、独立サンプルの一致の確認　(ここまで)

#13.3.1   2番目以降に抽出される知見

######## 図13.3 ジップ分布と，そこから非復元抽出した分布の食い違い
#a個非復元抽出数したときのn人分のデータを作る。確率bpで想起止める。
F<-matrix(0,n,a)
for (i in 1:n){
  y<-numeric(0)
  repeat{
    t1<-sample(x,1,prob=p, replace =T)
    if (is.element(t1,y)==0) {y<-c(y,t1)}
    if (length(y)==a) break
    if (sample(c(0,1),1,prob=c(bp,1-bp))==0) break
  }
F[i,1:(length(y))]<-y;         #復元抽出データ
}
head(F,n=50)

#zipf分布と、そこからa個非復元抽出した場合の相対度数が食い違うことを確認
Fv<-as.vector(ifelse(F==0,NA,F))
Ff<-table(Fv);    #非復元抽出データの度数表
Fs<-Ff/(n*a);     #非復元抽出データの相対度数
plot(Fs, ylim=c(0,maxy),lty=1,lwd=5,ylab="",xlab="",main="")
par(new=T)
plot(p,  ylim=c(0,maxy),type="l",xaxt="n",lwd=2,ylab="",xlab="",main="")
#dev.copy2eps(file="zipf01.eps",family='Japan1');              #図13.3ここまで

#######  13. 4 インタビュー調査の寡占度・飽和率・遭遇率

###### 表13.2 就職活動全体を通じての心得・不安の入力データと人数
(F01<-read.csv(file="scrT/dat/順位不安.csv",header=T))
N<-max(F01)
n<-nrow(F01)
a<-ncol(F01)

###########a個非復元抽出する場合のzipf分布の推定のstanコード
zipfa<-'
functions {
  real zeta(real s, int m, int Q) {
    real ii;
    real zet;
    ii = 0;
    zet = 0;
    for (i in 1 : m) {
      ii = ii + 1;
      zet = zet + inv(pow(ii, s)); //与えられた項数までの和
    }
    if (Q == 1) {
      zet = zet - (pow(m, 1 - s) / (1 - s));
    }                              //無限の場合は近似式で補正
    return zet;
  }
}
data {
  int<lower=0> Q;                  //無限モデルなら1,有限モデルなら0
  int<lower=0> N;                  //観察された対象の数（13.9)式のｒ＊
  int<lower=0> n;                  //被験者数
  int<lower=0> a;                  //非復元抽出数の最大値
  array[n, a] real<lower=-1> r;    //ランクデータ 欠測値はゼロ 無効は-1とする
}
transformed data {
  real lb;                         //sの下限値
  lb = 0.1;
  if (Q == 1) {
    lb = 1.001;
  }                                //発散より少しだけ大きい値を指定する
}
parameters {
  real<lower=lb> s;                //母数(寡占度)
}
model {
  real zip;                        //復元抽出でのr[i,j]の確率
  real temp;                       //抽出済の対象の確率の和
  real zet;                        //ゼータ関数の値
  if (Q == 1) {
    zet = zeta(s, 1000, Q);
  }                                //対象数が無限の場合は項数1000で近似
  if (Q == 0) {
    zet = zeta(s, N, Q);
  }                                //対象数が有限の場合のzeta
  for (i in 1 : n) {               // i番目の人の処理開始
    temp = 0;
    for (j in 1 : a) {             // i番目の人のj番目の回答の処理開始
      if (r[i, j] > 0) {           // 欠測や無効でないものだけ処理
        zip = inv(pow(r[i, j], s)) / zet;
        target += log(zip) - log(1 - temp); //(13.19)式
        temp = temp + zip;
      }
    }
  }
}
generated quantities {
  real prob;                       //飽和率
  real Lmprob;                     //遭遇率
  real zet;
  if (Q == 1) {
    zet = zeta(s, 1000, Q);
  }                                //対象数が無限の場合は項数1000で近似
  if (Q == 0) {
    zet = zeta(s, N, Q);
  }                                //対象数が有限の場合のzeta
  prob = 0;
  for (i in 1 : N) {
    prob = prob + inv(pow(i, s));
  }                                //(13.9)式,(13.1)式,(13.2)式 
  prob = prob / zet;
  Lmprob = 1 - prob;
}
';

par<-c("s","prob","Lmprob"); #sは寡占度、probは飽和率、Lmprobは遭遇率
dataSet01 <-list(Q=1,N=N,n=n,a=a,r=F01)

###########cmdstanrによる実行
modfile01 <- write_stan_file(zipfa)              #書き出す一時ファイル  
mod01 <- cmdstan_model(modfile01)                   #コンパイル
csrfit01 <- mod01$sample(data = dataSet01,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
extlec01<-as_draws_df(csrfit01$draws(par))
(colnames(extlec01) <- gsub("\\[|\\]|,", "", colnames(extlec01)))

#save(extlec01,  file="./scrT/obje/stan_obje1301")
#load(file="./scrT/obje/stan_obje1301"); #予めstan( )で作ったobjectをロード


####### 表13.3 寡占度・飽和率・遭遇率の事後分布の要約
gqcal(extlec01[,1:3])

####### 図13.4 寡占度の事後分布
hist(extlec01$s, breaks=100,cex.axis=2.0,cex.lab=2.0,
 ylab="", xlab="寡占度", main="");
#dev.copy2eps(file="z1304.eps",family='Japan1')#図13.4

####### 図13.5 飽和率のphc 曲線
phc01(seq(1.30,1.45,0.01),extlec01$s,cc="ltc",byoga="yes",xlab="寡占度")
phc01(seq(0.75,0.90,0.01),extlec01$prob,cc="gtc",byoga="yes",xlab="飽和率")
#dev.copy2eps(file="z1305.eps",family='Japan1')

####### 表13.4 飽和率のphc テーブル
phc01(c(0.76,0.78,0.80,0.82),extlec01$prob,cc="gtc",byoga="no",xlab="飽和率")


#実習課題
(F02<-read.csv(file="scrT/dat/順位ES.csv",header=T))
N<-max(F02)
n<-nrow(F02)
a<-ncol(F02)

par<-c("s","prob","Lmprob"); #sは寡占度、probは飽和率、Lmprobは遭遇率
dataSet02 <-list(Q=1,N=N,n=n,a=a,r=F02)

###########cmdstanrによる実行
modfile02 <- write_stan_file(zipfa)              #書き出す一時ファイル  
mod02 <- cmdstan_model(modfile02)                   #コンパイル
csrfit02 <- mod02$sample(data = dataSet02,chains = 5,iter_sampling = 20000,
    iter_warmup = 1000,parallel_chains = 5,seed=1234)  #MCMC
extlec02<-as_draws_df(csrfit02$draws(par))
(colnames(extlec02) <- gsub("\\[|\\]|,", "", colnames(extlec02)))

#save(extlec02,  file="./scrT/obje/stan_obje1302")
#load(file="./scrT/obje/stan_obje1302"); #予めstan( )で作ったobjectをロード

gqcal(extlec02[,1:3],4)
hist(extlec02$s, breaks=200,cex.axis=2.0);
